@@ -1,17 +1,18 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
-** All rights reserved.
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the demonstration applications of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
@@ -21,8 +22,8 @@
 ** ensure the GNU Lesser General Public License version 2.1 requirements
 ** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
@@ -33,8 +34,7 @@
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
+**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -64,15 +64,15 @@ TabBar::TabBar(QWidget *parent)
 {
     setContextMenuPolicy(Qt::CustomContextMenu);
     setAcceptDrops(true);
-    connect(this, SIGNAL(customContextMenuRequested(const QPoint &)),
-            this, SLOT(contextMenuRequested(const QPoint &)));
+    connect(this, SIGNAL(customContextMenuRequested(QPoint)),
+            this, SLOT(contextMenuRequested(QPoint)));
 
-    QString alt = QLatin1String("Alt+%1");
+    QString ctrl = QLatin1String("Ctrl+%1");
     for (int i = 1; i <= 10; ++i) {
         int key = i;
         if (key == 10)
             key = 0;
-        QShortcut *shortCut = new QShortcut(alt.arg(key), this);
+        QShortcut *shortCut = new QShortcut(ctrl.arg(key), this);
         m_tabShortcuts.append(shortCut);
         connect(shortCut, SIGNAL(activated()), this, SLOT(selectTabAction()));
     }
@@ -221,7 +221,7 @@ TabWidget::TabWidget(QWidget *parent)
     connect(m_tabBar, SIGNAL(closeOtherTabs(int)), this, SLOT(closeOtherTabs(int)));
     connect(m_tabBar, SIGNAL(reloadTab(int)), this, SLOT(reloadTab(int)));
     connect(m_tabBar, SIGNAL(reloadAllTabs()), this, SLOT(reloadAllTabs()));
-    connect(m_tabBar, SIGNAL(tabMoved(int, int)), this, SLOT(moveTab(int, int)));
+    connect(m_tabBar, SIGNAL(tabMoved(int,int)), this, SLOT(moveTab(int,int)));
     setTabBar(m_tabBar);
     setDocumentMode(true);
 
@@ -257,8 +257,8 @@ TabWidget::TabWidget(QWidget *parent)
     m_recentlyClosedTabsMenu = new QMenu(this);
     connect(m_recentlyClosedTabsMenu, SIGNAL(aboutToShow()),
             this, SLOT(aboutToShowRecentTabsMenu()));
-    connect(m_recentlyClosedTabsMenu, SIGNAL(triggered(QAction *)),
-            this, SLOT(aboutToShowRecentTriggeredAction(QAction *)));
+    connect(m_recentlyClosedTabsMenu, SIGNAL(triggered(QAction*)),
+            this, SLOT(aboutToShowRecentTriggeredAction(QAction*)));
     m_recentlyClosedTabsAction = new QAction(tr("Recently Closed Tabs"), this);
     m_recentlyClosedTabsAction->setMenu(m_recentlyClosedTabsMenu);
     m_recentlyClosedTabsAction->setEnabled(false);
@@ -304,18 +304,18 @@ void TabWidget::currentChanged(int index)
 
     WebView *oldWebView = this->webView(m_lineEdits->currentIndex());
     if (oldWebView) {
-        disconnect(oldWebView, SIGNAL(statusBarMessage(const QString&)),
-                this, SIGNAL(showStatusBarMessage(const QString&)));
-        disconnect(oldWebView->page(), SIGNAL(linkHovered(const QString&, const QString&, const QString&)),
-                this, SIGNAL(linkHovered(const QString&)));
+        disconnect(oldWebView, SIGNAL(statusBarMessage(QString)),
+                this, SIGNAL(showStatusBarMessage(QString)));
+        disconnect(oldWebView->page(), SIGNAL(linkHovered(QString,QString,QString)),
+                this, SIGNAL(linkHovered(QString)));
         disconnect(oldWebView, SIGNAL(loadProgress(int)),
                 this, SIGNAL(loadProgress(int)));
     }
 
-    connect(webView, SIGNAL(statusBarMessage(const QString&)),
-            this, SIGNAL(showStatusBarMessage(const QString&)));
-    connect(webView->page(), SIGNAL(linkHovered(const QString&, const QString&, const QString&)),
-            this, SIGNAL(linkHovered(const QString&)));
+    connect(webView, SIGNAL(statusBarMessage(QString)),
+            this, SIGNAL(showStatusBarMessage(QString)));
+    connect(webView->page(), SIGNAL(linkHovered(QString,QString,QString)),
+            this, SIGNAL(linkHovered(QString)));
     connect(webView, SIGNAL(loadProgress(int)),
             this, SIGNAL(loadProgress(int)));
 
@@ -438,6 +438,7 @@ WebView *TabWidget::newTab(bool makeCurrent)
         addTab(emptyWidget, tr("(Untitled)"));
         connect(this, SIGNAL(currentChanged(int)),
             this, SLOT(currentChanged(int)));
+        currentChanged(currentIndex());
         return 0;
     }
 
@@ -450,16 +451,16 @@ WebView *TabWidget::newTab(bool makeCurrent)
             this, SLOT(webViewIconChanged()));
     connect(webView, SIGNAL(iconChanged()),
             this, SLOT(webViewIconChanged()));
-    connect(webView, SIGNAL(titleChanged(const QString &)),
-            this, SLOT(webViewTitleChanged(const QString &)));
-    connect(webView, SIGNAL(urlChanged(const QUrl &)),
-            this, SLOT(webViewUrlChanged(const QUrl &)));
+    connect(webView, SIGNAL(titleChanged(QString)),
+            this, SLOT(webViewTitleChanged(QString)));
+    connect(webView, SIGNAL(urlChanged(QUrl)),
+            this, SLOT(webViewUrlChanged(QUrl)));
     connect(webView->page(), SIGNAL(windowCloseRequested()),
             this, SLOT(windowCloseRequested()));
-    connect(webView->page(), SIGNAL(geometryChangeRequested(const QRect &)),
-            this, SIGNAL(geometryChangeRequested(const QRect &)));
-    connect(webView->page(), SIGNAL(printRequested(QWebFrame *)),
-            this, SIGNAL(printRequested(QWebFrame *)));
+    connect(webView->page(), SIGNAL(geometryChangeRequested(QRect)),
+            this, SIGNAL(geometryChangeRequested(QRect)));
+    connect(webView->page(), SIGNAL(printRequested(QWebFrame*)),
+            this, SIGNAL(printRequested(QWebFrame*)));
     connect(webView->page(), SIGNAL(menuBarVisibilityChangeRequested(bool)),
             this, SIGNAL(menuBarVisibilityChangeRequested(bool)));
     connect(webView->page(), SIGNAL(statusBarVisibilityChangeRequested(bool)),
@@ -764,7 +765,7 @@ WebActionMapper::WebActionMapper(QAction *root, QWebPage::WebAction webAction, Q
     if (!m_root)
         return;
     connect(m_root, SIGNAL(triggered()), this, SLOT(rootTriggered()));
-    connect(root, SIGNAL(destroyed(QObject *)), this, SLOT(rootDestroyed()));
+    connect(root, SIGNAL(destroyed(QObject*)), this, SLOT(rootDestroyed()));
     root->setEnabled(false);
 }
 
@@ -813,7 +814,7 @@ void WebActionMapper::childChanged()
 void WebActionMapper::updateCurrent(QWebPage *currentParent)
 {
     if (m_currentParent)
-        disconnect(m_currentParent, SIGNAL(destroyed(QObject *)),
+        disconnect(m_currentParent, SIGNAL(destroyed(QObject*)),
                    this, SLOT(currentDestroyed()));
 
     m_currentParent = currentParent;
@@ -827,7 +828,7 @@ void WebActionMapper::updateCurrent(QWebPage *currentParent)
     QAction *source = m_currentParent->action(m_webAction);
     m_root->setChecked(source->isChecked());
     m_root->setEnabled(source->isEnabled());
-    connect(m_currentParent, SIGNAL(destroyed(QObject *)),
+    connect(m_currentParent, SIGNAL(destroyed(QObject*)),
             this, SLOT(currentDestroyed()));
 }
 
