@@ -1,38 +1,38 @@
 /****************************************************************************
 **
-** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
-** All rights reserved.
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the qmake application of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** GNU Lesser General Public License Usage
-** This file may be used under the terms of the GNU Lesser General Public
-** License version 2.1 as published by the Free Software Foundation and
-** appearing in the file LICENSE.LGPL included in the packaging of this
-** file. Please review the following information to ensure the GNU Lesser
-** General Public License version 2.1 requirements will be met:
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
-** In addition, as a special exception, Nokia gives you certain additional
-** rights. These rights are described in the Nokia Qt LGPL Exception
+** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+**
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU General
-** Public License version 3.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of this
-** file. Please review the following information to ensure the GNU General
-** Public License version 3.0 requirements will be met:
-** http://www.gnu.org/copyleft/gpl.html.
-**
-** Other Usage
-** Alternatively, this file may be used in accordance with the terms and
-** conditions contained in a signed written agreement between you and Nokia.
-**
-**
-**
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3.0 as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU General Public License version 3.0 requirements will be
+** met: http://www.gnu.org/copyleft/gpl.html.
 **
 **
 ** $QT_END_LICENSE$
@@ -181,6 +181,7 @@ const char _Optimization[]                      = "Optimization";
 const char _OptimizeReferences[]                = "OptimizeReferences";
 const char _OutputDirectory[]                   = "OutputDirectory";
 const char _OutputFile[]                        = "OutputFile";
+const char _PlatformToolSet[]                   = "PlatformToolSet";
 const char _PrecompiledHeader[]                 = "PrecompiledHeader";
 const char _PrecompiledHeaderFile[]             = "PrecompiledHeaderFile";
 const char _PrecompiledHeaderOutputFile[]       = "PrecompiledHeaderOutputFile";
@@ -413,6 +414,7 @@ void VCXProjectWriter::write(XmlOutput &xml, VCProjectSingleConfig &tool)
     xml << import("Project", "$(VCTargetsPath)\\Microsoft.Cpp.Default.props");
 
     write(xml, tool.Configuration);
+    const QString condition = generateCondition(tool.Configuration);
 
     xml << import("Project", "$(VCTargetsPath)\\Microsoft.Cpp.props");
 
@@ -423,7 +425,7 @@ void VCXProjectWriter::write(XmlOutput &xml, VCProjectSingleConfig &tool)
 
     // PropertySheets
     xml << tag("ImportGroup")
-        << attrTag("Condition", QString("'$(Configuration)|$(Platform)'=='%1'").arg(tool.Configuration.Name))
+        << attrTag("Condition", condition)
         << attrTag("Label", "PropertySheets");
 
     xml << tag("Import")
@@ -432,7 +434,6 @@ void VCXProjectWriter::write(XmlOutput &xml, VCProjectSingleConfig &tool)
         << closetag()
         << closetag();
 
-
     // UserMacros
     xml << tag("PropertyGroup")
         << attrTag("Label", "UserMacros")
@@ -442,57 +443,57 @@ void VCXProjectWriter::write(XmlOutput &xml, VCProjectSingleConfig &tool)
 
     if ( !tool.Configuration.OutputDirectory.isEmpty() ) {
         xml<< tag("OutDir")
-            << attrTag("Condition", QString("'$(Configuration)|$(Platform)'=='%1'").arg(tool.Configuration.Name))
+            << attrTag("Condition", condition)
             << valueTag(tool.Configuration.OutputDirectory);
     }
     if ( !tool.Configuration.IntermediateDirectory.isEmpty() ) {
         xml<< tag("IntDir")
-            << attrTag("Condition", QString("'$(Configuration)|$(Platform)'=='%1'").arg(tool.Configuration.Name))
+            << attrTag("Condition", condition)
             << valueTag(tool.Configuration.IntermediateDirectory);
     }
     if ( !tool.Configuration.PrimaryOutput.isEmpty() ) {
         xml<< tag("TargetName")
-            << attrTag("Condition", QString("'$(Configuration)|$(Platform)'=='%1'").arg(tool.Configuration.Name))
+            << attrTag("Condition", condition)
             << valueTag(tool.Configuration.PrimaryOutput);
     }
 
     if ( tool.Configuration.linker.IgnoreImportLibrary != unset) {
         xml<< tag("IgnoreImportLibrary")
-            << attrTag("Condition", QString("'$(Configuration)|$(Platform)'=='%1'").arg(tool.Configuration.Name))
+            << attrTag("Condition", condition)
             << valueTagT(tool.Configuration.linker.IgnoreImportLibrary);
     }
 
     if ( tool.Configuration.linker.LinkIncremental != linkIncrementalDefault) {
         const triState ts = (tool.Configuration.linker.LinkIncremental == linkIncrementalYes ? _True : _False);
         xml<< tag("LinkIncremental")
-            << attrTag("Condition", QString("'$(Configuration)|$(Platform)'=='%1'").arg(tool.Configuration.Name))
+            << attrTag("Condition", condition)
             << valueTagT(ts);
     }
 
     if ( tool.Configuration.preBuild.ExcludedFromBuild != unset )
     {
         xml<< tag("PreBuildEventUseInBuild")
-            << attrTag("Condition", QString("'$(Configuration)|$(Platform)'=='%1'").arg(tool.Configuration.Name))
+            << attrTag("Condition", condition)
             << valueTagT(!tool.Configuration.preBuild.ExcludedFromBuild);
     }
 
     if ( tool.Configuration.preLink.ExcludedFromBuild != unset )
     {
         xml<< tag("PreLinkEventUseInBuild")
-            << attrTag("Condition", QString("'$(Configuration)|$(Platform)'=='%1'").arg(tool.Configuration.Name))
+            << attrTag("Condition", condition)
             << valueTagT(!tool.Configuration.preLink.ExcludedFromBuild);
     }
 
     if ( tool.Configuration.postBuild.ExcludedFromBuild != unset )
     {
         xml<< tag("PostBuildEventUseInBuild")
-            << attrTag("Condition", QString("'$(Configuration)|$(Platform)'=='%1'").arg(tool.Configuration.Name))
+            << attrTag("Condition", condition)
             << valueTagT(!tool.Configuration.postBuild.ExcludedFromBuild);
     }
     xml << closetag();
 
     xml << tag("ItemDefinitionGroup")
-        << attrTag("Condition", QString("'$(Configuration)|$(Platform)'=='%1'").arg(tool.Configuration.Name));
+        << attrTag("Condition", condition);
 
     // ClCompile
     write(xml, tool.Configuration.compiler);
@@ -559,6 +560,8 @@ void VCXProjectWriter::write(XmlOutput &xml, VCProjectSingleConfig &tool)
         outputFilter(tempProj, xml, xmlFilter, tempProj.ExtraCompilers.at(x));
     }
 
+    outputFilter(tempProj, xml, xmlFilter, "Root Files");
+
     xml << import("Project", "$(VCTargetsPath)\\Microsoft.Cpp.targets");
 
     xml << tag("ImportGroup")
@@ -613,7 +616,7 @@ void VCXProjectWriter::write(XmlOutput &xml, VCProject &tool)
     // PropertySheets
     for (int i = 0; i < tool.SingleProjects.count(); ++i) {
         xml << tag("ImportGroup")
-            << attrTag("Condition", QString("'$(Configuration)|$(Platform)'=='%1'").arg(tool.SingleProjects.at(i).Configuration.Name))
+            << attrTag("Condition", generateCondition(tool.SingleProjects.at(i).Configuration))
             << attrTag("Label", "PropertySheets");
 
         xml << tag("Import")
@@ -630,86 +633,97 @@ void VCXProjectWriter::write(XmlOutput &xml, VCProject &tool)
 
     xml << tag("PropertyGroup");
     for (int i = 0; i < tool.SingleProjects.count(); ++i) {
+        const VCConfiguration &config = tool.SingleProjects.at(i).Configuration;
+        const QString condition = generateCondition(config);
 
-        if ( !tool.SingleProjects.at(i).Configuration.OutputDirectory.isEmpty() ) {
+        if (!config.OutputDirectory.isEmpty()) {
             xml << tag("OutDir")
-                << attrTag("Condition", QString("'$(Configuration)|$(Platform)'=='%1'").arg(tool.SingleProjects.at(i).Configuration.Name))
-                << valueTag(tool.SingleProjects.at(i).Configuration.OutputDirectory);
+                << attrTag("Condition", condition)
+                << valueTag(config.OutputDirectory);
         }
-        if ( !tool.SingleProjects.at(i).Configuration.IntermediateDirectory.isEmpty() ) {
+        if (!config.IntermediateDirectory.isEmpty()) {
             xml << tag("IntDir")
-                << attrTag("Condition", QString("'$(Configuration)|$(Platform)'=='%1'").arg(tool.SingleProjects.at(i).Configuration.Name))
-                << valueTag(tool.SingleProjects.at(i).Configuration.IntermediateDirectory);
+                << attrTag("Condition", condition)
+                << valueTag(config.IntermediateDirectory);
         }
-        if ( !tool.SingleProjects.at(i).Configuration.PrimaryOutput.isEmpty() ) {
+        if (!config.PrimaryOutput.isEmpty()) {
             xml << tag("TargetName")
-                << attrTag("Condition", QString("'$(Configuration)|$(Platform)'=='%1'").arg(tool.SingleProjects.at(i).Configuration.Name))
-                << valueTag(tool.SingleProjects.at(i).Configuration.PrimaryOutput);
+                << attrTag("Condition", condition)
+                << valueTag(config.PrimaryOutput);
         }
 
-        if ( tool.SingleProjects.at(i).Configuration.linker.IgnoreImportLibrary != unset) {
+        if (config.linker.IgnoreImportLibrary != unset) {
             xml << tag("IgnoreImportLibrary")
-                << attrTag("Condition", QString("'$(Configuration)|$(Platform)'=='%1'").arg(tool.SingleProjects.at(i).Configuration.Name))
-                << valueTagT(tool.SingleProjects.at(i).Configuration.linker.IgnoreImportLibrary);
+                << attrTag("Condition", condition)
+                << valueTagT(config.linker.IgnoreImportLibrary);
         }
 
-        if ( tool.SingleProjects.at(i).Configuration.linker.LinkIncremental != linkIncrementalDefault) {
-            const triState ts = (tool.SingleProjects.at(i).Configuration.linker.LinkIncremental == linkIncrementalYes ? _True : _False);
+        if (config.linker.LinkIncremental != linkIncrementalDefault) {
+            const triState ts = (config.linker.LinkIncremental == linkIncrementalYes ? _True : _False);
             xml << tag("LinkIncremental")
-                << attrTag("Condition", QString("'$(Configuration)|$(Platform)'=='%1'").arg(tool.SingleProjects.at(i).Configuration.Name))
+                << attrTag("Condition", condition)
                 << valueTagT(ts);
         }
 
-        if ( tool.SingleProjects.at(i).Configuration.preBuild.ExcludedFromBuild != unset )
+        const triState generateManifest = config.linker.GenerateManifest;
+        if (generateManifest != unset) {
+            xml << tag("GenerateManifest")
+                << attrTag("Condition", condition)
+                << valueTagT(generateManifest);
+        }
+
+        if (config.preBuild.ExcludedFromBuild != unset)
         {
             xml << tag("PreBuildEventUseInBuild")
-                << attrTag("Condition", QString("'$(Configuration)|$(Platform)'=='%1'").arg(tool.SingleProjects.at(i).Configuration.Name))
-                << valueTagT(!tool.SingleProjects.at(i).Configuration.preBuild.ExcludedFromBuild);
+                << attrTag("Condition", condition)
+                << valueTagT(!config.preBuild.ExcludedFromBuild);
         }
 
-        if ( tool.SingleProjects.at(i).Configuration.preLink.ExcludedFromBuild != unset )
+        if (config.preLink.ExcludedFromBuild != unset)
         {
             xml << tag("PreLinkEventUseInBuild")
-                << attrTag("Condition", QString("'$(Configuration)|$(Platform)'=='%1'").arg(tool.SingleProjects.at(i).Configuration.Name))
-                << valueTagT(!tool.SingleProjects.at(i).Configuration.preLink.ExcludedFromBuild);
+                << attrTag("Condition", condition)
+                << valueTagT(!config.preLink.ExcludedFromBuild);
         }
 
-        if ( tool.SingleProjects.at(i).Configuration.postBuild.ExcludedFromBuild != unset )
+        if (config.postBuild.ExcludedFromBuild != unset)
         {
             xml << tag("PostBuildEventUseInBuild")
-                << attrTag("Condition", QString("'$(Configuration)|$(Platform)'=='%1'").arg(tool.SingleProjects.at(i).Configuration.Name))
-                << valueTagT(!tool.SingleProjects.at(i).Configuration.postBuild.ExcludedFromBuild);
+                << attrTag("Condition", condition)
+                << valueTagT(!config.postBuild.ExcludedFromBuild);
         }
     }
     xml << closetag();
 
     for (int i = 0; i < tool.SingleProjects.count(); ++i) {
+        const VCConfiguration &config = tool.SingleProjects.at(i).Configuration;
+
         xml << tag("ItemDefinitionGroup")
-            << attrTag("Condition", QString("'$(Configuration)|$(Platform)'=='%1'").arg(tool.SingleProjects.at(i).Configuration.Name));
+            << attrTag("Condition", generateCondition(config));
 
         // ClCompile
-        write(xml, tool.SingleProjects.at(i).Configuration.compiler);
+        write(xml, config.compiler);
 
         // Link
-        write(xml, tool.SingleProjects.at(i).Configuration.linker);
+        write(xml, config.linker);
 
         // Midl
-        write(xml, tool.SingleProjects.at(i).Configuration.idl);
+        write(xml, config.idl);
 
         // ResourceCompiler
-        write(xml, tool.SingleProjects.at(i).Configuration.resource);
+        write(xml, config.resource);
 
         // Post build event
-        if ( tool.SingleProjects.at(i).Configuration.postBuild.ExcludedFromBuild != unset )
-            write(xml, tool.SingleProjects.at(i).Configuration.postBuild);
+        if (config.postBuild.ExcludedFromBuild != unset)
+            write(xml, config.postBuild);
 
         // Pre build event
-        if ( tool.SingleProjects.at(i).Configuration.preBuild.ExcludedFromBuild != unset )
-            write(xml, tool.SingleProjects.at(i).Configuration.preBuild);
+        if (config.preBuild.ExcludedFromBuild != unset)
+            write(xml, config.preBuild);
 
         // Pre link event
-        if ( tool.SingleProjects.at(i).Configuration.preLink.ExcludedFromBuild != unset )
-            write(xml, tool.SingleProjects.at(i).Configuration.preLink);
+        if (config.preLink.ExcludedFromBuild != unset)
+            write(xml, config.preLink);
 
         xml << closetag();
     }
@@ -902,10 +916,14 @@ static inline QString toString(inlineExpansionOption option)
 static inline QString toString(optimizeOption option)
 {
     switch (option) {
+    case optimizeDisabled:
+        return "Disabled";
     case optimizeMinSpace:
         return "MinSpace";
     case optimizeMaxSpeed:
         return "MaxSpeed";
+    case optimizeFull:
+        return "Full";
     }
     return QString();
 }
@@ -1304,7 +1322,7 @@ void VCXProjectWriter::write(XmlOutput &xml, const VCCLCompilerTool &tool)
             << attrTagT(_StringPooling, tool.StringPooling)
             << attrTagS(_StructMemberAlignment, toString(tool.StructMemberAlignment))
             << attrTagT(_SuppressStartupBanner, tool.SuppressStartupBanner)
-//unused    << attrTagS(_TreatSpecificWarningsAsErrors, tool.TreatSpecificWarningsAsErrors)
+            << attrTagX(_TreatSpecificWarningsAsErrors, tool.TreatSpecificWarningsAsErrors, ";")
             << attrTagT(_TreatWarningAsError, tool.WarnAsError)
             << attrTagT(_TreatWChar_tAsBuiltInType, tool.TreatWChar_tAsBuiltInType)
             << attrTagT(_UndefineAllPreprocessorDefinitions, tool.UndefineAllPreprocessorDefinitions)
@@ -1445,33 +1463,33 @@ void VCXProjectWriter::write(XmlOutput &xml, const VCMIDLTool &tool)
 
 void VCXProjectWriter::write(XmlOutput &xml, const VCCustomBuildTool &tool)
 {
-    const QString &configName = tool.config->Name;
+    const QString condition = generateCondition(*tool.config);
 
     if ( !tool.AdditionalDependencies.isEmpty() )
     {
         xml << tag("AdditionalInputs")
-            << attrTag("Condition", QString("'$(Configuration)|$(Platform)'=='%1'").arg(configName))
+            << attrTag("Condition", condition)
             << valueTagDefX(tool.AdditionalDependencies, "AdditionalInputs", ";");
     }
 
     if( !tool.CommandLine.isEmpty() )
     {
         xml << tag("Command")
-            << attrTag("Condition", QString("'$(Configuration)|$(Platform)'=='%1'").arg(configName))
+            << attrTag("Condition", condition)
             << valueTag(tool.CommandLine.join(vcxCommandSeparator()));
     }
 
     if ( !tool.Description.isEmpty() )
     {
         xml << tag("Message")
-            << attrTag("Condition", QString("'$(Configuration)|$(Platform)'=='%1'").arg(configName))
+            << attrTag("Condition", condition)
             << valueTag(tool.Description);
     }
 
     if ( !tool.Outputs.isEmpty() )
     {
         xml << tag("Outputs")
-            << attrTag("Condition", QString("'$(Configuration)|$(Platform)'=='%1'").arg(configName))
+            << attrTag("Condition", condition)
             << valueTagDefX(tool.Outputs, "Outputs", ";");
     }
 }
@@ -1539,8 +1557,9 @@ void VCXProjectWriter::write(XmlOutput &xml, const VCDeploymentTool &tool)
 void VCXProjectWriter::write(XmlOutput &xml, const VCConfiguration &tool)
 {
         xml << tag("PropertyGroup")
-            << attrTag("Condition", QString("'$(Configuration)|$(Platform)'=='%1'").arg(tool.Name))
+            << attrTag("Condition", generateCondition(tool))
             << attrTag("Label", "Configuration")
+            << attrTagS(_PlatformToolSet, platformToolSetVersion(tool.CompilerVersion))
             << attrTagS(_OutputDirectory, tool.OutputDirectory)
             << attrTagT(_ATLMinimizesCRunTimeLibraryUsage, tool.ATLMinimizesCRunTimeLibraryUsage)
             << attrTagT(_BuildBrowserInformation, tool.BuildBrowserInformation)
@@ -1907,10 +1926,10 @@ bool VCXProjectWriter::outputFileConfig(VCFilter &filter, XmlOutput &xml, XmlOut
             }
         }
 
+        const QString condition = generateCondition(*filter.Config);
         if(!inBuild) {
-
             xml << tag("ExcludedFromBuild")
-                << attrTag("Condition", QString("'$(Configuration)|$(Platform)'=='%1'").arg(filter.Config->Name))
+                << attrTag("Condition", condition)
                 << valueTag("true");
         }
 
@@ -1918,17 +1937,17 @@ bool VCXProjectWriter::outputFileConfig(VCFilter &filter, XmlOutput &xml, XmlOut
 
             if ( !filter.CompilerTool.ForcedIncludeFiles.isEmpty() ) {
                 xml << tag("ForcedIncludeFiles")
-                    << attrTag("Condition", QString("'$(Configuration)|$(Platform)'=='%1'").arg(filter.Config->Name))
+                    << attrTag("Condition", condition)
                     << valueTagX(filter.CompilerTool.ForcedIncludeFiles);
             }
 
             if ( !filter.CompilerTool.PrecompiledHeaderThrough.isEmpty() ) {
 
                 xml << tag("PrecompiledHeaderFile")
-                    << attrTag("Condition", QString("'$(Configuration)|$(Platform)'=='%1'").arg(filter.Config->Name))
+                    << attrTag("Condition", condition)
                     << valueTag(filter.CompilerTool.PrecompiledHeaderThrough)
                     << tag("PrecompiledHeader")
-                    << attrTag("Condition", QString("'$(Configuration)|$(Platform)'=='%1'").arg(filter.Config->Name))
+                    << attrTag("Condition", condition)
                     << valueTag(toString(filter.CompilerTool.UsePrecompiledHeader));
             }
         }
@@ -1937,4 +1956,21 @@ bool VCXProjectWriter::outputFileConfig(VCFilter &filter, XmlOutput &xml, XmlOut
     return fileAdded;
 }
 
+QString VCXProjectWriter::generateCondition(const VCConfiguration &config)
+{
+    return QLatin1String("'$(Configuration)|$(Platform)'=='") + config.Name + QLatin1Char('\'');
+}
+
+QString VCXProjectWriter::platformToolSetVersion(const DotNET version)
+{
+    switch (version)
+    {
+    case NET2012:
+        return "v110";
+    case NET2013:
+        return "v120";
+    }
+    return QString();
+}
+
 QT_END_NAMESPACE
