@@ -1,17 +1,18 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
-** All rights reserved.
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the qmake application of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
@@ -21,8 +22,8 @@
 ** ensure the GNU Lesser General Public License version 2.1 requirements
 ** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
@@ -33,8 +34,7 @@
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
+**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -58,7 +58,10 @@ enum DotNET {
     NET2002 = 0x70,
     NET2003 = 0x71,
     NET2005 = 0x80,
-    NET2008 = 0x90
+    NET2008 = 0x90,
+    NET2010 = 0xa0,
+    NET2012 = 0xb0,
+    NET2013 = 0xc0
 };
 
 /*
@@ -85,6 +88,9 @@ enum triState {
     _False = 0,
     _True = 1
 };
+
+triState operator!(const triState &rhs);
+
 enum addressAwarenessType {
     addrAwareDefault,
     addrAwareNoLarge,
@@ -98,10 +104,10 @@ enum asmListingOption {
     asmListingAsmSrc
 };
 enum basicRuntimeCheckOption {
-    runtimeBasicCheckNone,
-    runtimeCheckStackFrame,
-    runtimeCheckUninitVariables,
-    runtimeBasicCheckAll
+    runtimeBasicCheckNone = 0,
+    runtimeCheckStackFrame = 1,
+    runtimeCheckUninitVariables = 2,
+    runtimeBasicCheckAll = runtimeCheckStackFrame | runtimeCheckUninitVariables
 };
 enum browseInfoOption {
     brInfoNone,
@@ -298,7 +304,8 @@ enum linkProgressOption {
 };
 enum machineTypeOption {
     machineNotSet,
-    machineX86
+    machineX86,
+    machineX64 = 17
 };
 enum midlCharOption {
     midlCharUnsigned,
@@ -361,6 +368,7 @@ enum optLinkTimeCodeGenType {
     optLTCGUpdate
 };
 enum pchOption {
+    pchUnset = -1,
     pchNone,
     pchCreateUsingSpecific,
     pchGenerateAuto,
@@ -465,7 +473,7 @@ enum warningLevelOption {
 class VCToolBase {
 protected:
     // Functions
-    VCToolBase(){};
+    VCToolBase(){}
     virtual ~VCToolBase(){}
     virtual bool parseOption(const char* option) = 0;
 public:
@@ -473,6 +481,7 @@ public:
         for (QStringList::ConstIterator it=options.begin(); (it!=options.end()); it++)
             parseOption((*it).toLatin1());
     }
+    static QStringList fixCommandLine(const QString &input);
 };
 
 class VCConfiguration;
@@ -518,6 +527,7 @@ public:
     QStringList             ForcedIncludeFiles;
     QStringList             ForcedUsingFiles;
     preprocessOption        GeneratePreprocessedFile;
+    triState                PreprocessSuppressLineNumbers;
     triState                GlobalOptimizations;
     triState                IgnoreStandardIncludePath;
     triState                ImproveFloatingPointConsistency;
@@ -525,6 +535,7 @@ public:
     triState                KeepComments;
     triState                MinimalRebuild;
     QString                 ObjectFile;
+    triState                OmitDefaultLibName;
     triState                OmitFramePointers;
     triState                OpenMP;
     optimizeOption          Optimization;
@@ -547,12 +558,29 @@ public:
     triState                UndefineAllPreprocessorDefinitions;
     QStringList             UndefinePreprocessorDefinitions;
     pchOption               UsePrecompiledHeader;
+    triState                UseUnicodeForAssemblerListing;
+    QStringList             TreatSpecificWarningsAsErrors;
     triState                WarnAsError;
     warningLevelOption      WarningLevel;
     triState                WholeProgramOptimization;
     useOfArchitecture       CompileForArchitecture;
     triState                InterworkCalls;
+
+    // VS2010
+    triState                EnablePREfast;
+    triState                DisplayFullPaths;
+    triState                MultiProcessorCompilation;
+    QString                 MultiProcessorCompilationProcessorCount;
+    triState                GenerateXMLDocumentationFiles;
+    QString                 XMLDocumentationFileName;
+    QString                 ErrorReporting;
+    triState                CreateHotpatchableImage;
+    QString                 PreprocessOutputPath;
+
     VCConfiguration*        config;
+
+private:
+    bool parseRuntimeCheckOption(char c, int *rtc);
 };
 
 class VCLinkerTool : public VCToolBase
@@ -569,6 +597,7 @@ public:
     QStringList             AdditionalOptions;
     QStringList             AddModuleNamesToAssembly;
     QString                 BaseAddress;
+    triState                DataExecutionPrevention;
     QStringList             DelayLoadDLLs;
     optFoldingType          EnableCOMDATFolding;
     QString                 EntryPointSymbol;
@@ -599,6 +628,7 @@ public:
     optRefType              OptimizeReferences;
     QString                 OutputFile;
     QString                 ProgramDatabaseFile;
+    triState                RandomizedBaseAddress;
     triState                RegisterOutput;
     triState                ResourceOnlyDLL;
     triState                SetChecksum;
@@ -613,10 +643,33 @@ public:
     triState                SwapRunFromNet;
     machineTypeOption       TargetMachine;
     termSvrAwarenessType    TerminalServerAware;
+    triState                TreatWarningsAsErrors;
     triState                TurnOffAssemblyGeneration;
     QString                 TypeLibraryFile;
     qlonglong               TypeLibraryResourceID;
     QString                 Version;
+
+    // VS2010
+    triState                GenerateManifest;
+    QStringList             AdditionalManifestDependencies;
+    QString                 ManifestFile;
+    triState                EnableUAC;
+    QString                 UACExecutionLevel;
+    triState                UACUIAccess;
+    qlonglong               SectionAlignment;
+    triState                PreventDllBinding;
+    triState                AllowIsolation;
+    triState                AssemblyDebug;
+    QStringList             AssemblyLinkResource;
+    QString                 CLRImageType;
+    QString                 CLRSupportLastError;
+    QString                 CLRThreadAttribute;
+    triState                CLRUnmanagedCodeCheck;
+    triState                DelaySign;
+    QString                 KeyContainer;
+    QString                 KeyFile;
+    QString                 LinkErrorReporting;
+
     VCConfiguration*        config;
 };
 
@@ -659,6 +712,18 @@ public:
     triState                ValidateParameters;
     triState                WarnAsError;
     midlWarningLevelOption  WarningLevel;
+
+    // VS 2010
+    triState                ApplicationConfigurationMode;
+    QString                 GenerateClientFiles;
+    QString                 ClientStubFile;
+    QString                 TypeLibFormat;
+    triState                ValidateAllParameters;
+    triState                SuppressCompilerWarnings;
+    QString                 GenerateServerFiles;
+    QString                 ServerStubFile;
+    qlonglong               LocaleID;
+
     VCConfiguration*        config;
 };
 
@@ -668,7 +733,7 @@ public:
     // Functions
     VCLibrarianTool();
     virtual ~VCLibrarianTool(){}
-    bool parseOption(const char*){ return false; };
+    bool parseOption(const char*){ return false; }
 
     // Variables
     QStringList             AdditionalDependencies;
@@ -689,7 +754,7 @@ public:
     // Functions
     VCCustomBuildTool();
     virtual ~VCCustomBuildTool(){}
-    bool parseOption(const char*){ return false; };
+    bool parseOption(const char*){ return false; }
 
     // Variables
     QStringList             AdditionalDependencies;
@@ -698,6 +763,8 @@ public:
     QStringList             Outputs;
     QString                 ToolName;
     QString                 ToolPath;
+
+    VCConfiguration*        config;
 };
 
 class VCResourceCompilerTool : public VCToolBase
@@ -706,7 +773,7 @@ public:
     // Functions
     VCResourceCompilerTool();
     virtual ~VCResourceCompilerTool(){}
-    bool parseOption(const char*){ return false; };
+    bool parseOption(const char*){ return false; }
 
     // Variables
     QStringList             AdditionalIncludeDirectories;
@@ -718,6 +785,7 @@ public:
     QString                 ResourceOutputFileName;
     linkProgressOption      ShowProgress;
     QString                 ToolPath;
+    triState                SuppressStartupBanner;
 };
 
 class VCDeploymentTool
@@ -738,15 +806,16 @@ class VCEventTool : public VCToolBase
 {
 protected:
     // Functions
-    VCEventTool() : ExcludedFromBuild(unset){};
+    VCEventTool(const QString &eventName);
     virtual ~VCEventTool(){}
-    bool parseOption(const char*){ return false; };
+    bool parseOption(const char*){ return false; }
 
 public:
     // Variables
-    QString                 CommandLine;
+    QStringList             CommandLine;
     QString                 Description;
     triState                ExcludedFromBuild;
+    QString                 EventName;
     QString                 ToolName;
     QString                 ToolPath;
 };
@@ -789,7 +858,8 @@ public:
     QString                 DeleteExtensionsOnClean;
     QString                 ImportLibrary;
     QString                 IntermediateDirectory;
-    QString                 Name;
+    QString                 Name;   // "ConfigurationName|PlatformName"
+    QString                 ConfigurationName;
     QString                 OutputDirectory;
     QString                 PrimaryOutput;
     QString                 ProgramDatabase;
@@ -846,14 +916,13 @@ class VCFilter
 public:
     // Functions
     VCFilter();
-    ~VCFilter(){};
+    ~VCFilter(){}
 
     void addFile(const QString& filename);
     void addFile(const VCFilterFile& fileInfo);
     void addFiles(const QStringList& fileList);
     bool addExtraCompiler(const VCFilterFile &info);
     void modifyPCHstage(QString str);
-    void outputFileConfig(XmlOutput &xml, const QString &filename);
 
     // Variables
     QString                 Name;
@@ -866,14 +935,11 @@ public:
 
     customBuildCheck	    CustomBuild;
 
-    bool		    useCustomBuildTool;
+    bool                    useCustomBuildTool;
     VCCustomBuildTool       CustomBuildTool;
 
     bool                    useCompilerTool;
     VCCLCompilerTool        CompilerTool;
-
-private:
-    friend XmlOutput &operator<<(XmlOutput &xml, VCFilter &tool);
 };
 
 typedef QList<VCFilter> VCFilterList;
@@ -891,7 +957,7 @@ public:
         Extras
     };
     // Functions
-    VCProjectSingleConfig(){};
+    VCProjectSingleConfig(){}
     ~VCProjectSingleConfig(){}
 
     // Variables
@@ -921,8 +987,6 @@ public:
     VCFilter               &filterForExtraCompiler(const QString &compilerName);
 };
 
-
-
 // Tree & Flat view of files --------------------------------------------------
 class VCFilter;
 class Node
@@ -1048,30 +1112,35 @@ public:
 
     // List of all extracompilers
     QStringList             ExtraCompilers;
-
-    // Functions
-    void                    outputFilter(XmlOutput &xml,
-//                                         VCProjectSingleConfig::FilterTypes type,
-                                         const QString &filtername);
-
-    void                    outputFileConfigs(XmlOutput &xml,
-//                                              VCProjectSingleConfig::FilterTypes type,
-                                              const VCFilterFile &info,
-                                              const QString &filtername);
 };
 
-XmlOutput &operator<<(XmlOutput &, const VCCLCompilerTool &);
-XmlOutput &operator<<(XmlOutput &, const VCLinkerTool &);
-XmlOutput &operator<<(XmlOutput &, const VCMIDLTool &);
-XmlOutput &operator<<(XmlOutput &, const VCCustomBuildTool &);
-XmlOutput &operator<<(XmlOutput &, const VCLibrarianTool &);
-XmlOutput &operator<<(XmlOutput &, const VCResourceCompilerTool &);
-XmlOutput &operator<<(XmlOutput &, const VCEventTool &);
-XmlOutput &operator<<(XmlOutput &, const VCDeploymentTool &);
-XmlOutput &operator<<(XmlOutput &, const VCConfiguration &);
-XmlOutput &operator<<(XmlOutput &, VCFilter &);
-XmlOutput &operator<<(XmlOutput &, const VCProjectSingleConfig &);
-XmlOutput &operator<<(XmlOutput &, VCProject &);
+class VCProjectWriter
+{
+public:
+    virtual ~VCProjectWriter() {}
+
+    virtual void write(XmlOutput &, VCProjectSingleConfig &);
+    virtual void write(XmlOutput &, VCProject &);
+
+    virtual void write(XmlOutput &, const VCCLCompilerTool &);
+    virtual void write(XmlOutput &, const VCLinkerTool &);
+    virtual void write(XmlOutput &, const VCMIDLTool &);
+    virtual void write(XmlOutput &, const VCCustomBuildTool &);
+    virtual void write(XmlOutput &, const VCLibrarianTool &);
+    virtual void write(XmlOutput &, const VCResourceCompilerTool &);
+    virtual void write(XmlOutput &, const VCEventTool &);
+    virtual void write(XmlOutput &, const VCDeploymentTool &);
+    virtual void write(XmlOutput &, const VCConfiguration &);
+    virtual void write(XmlOutput &, VCFilter &);
+
+private:
+    static void outputFilter(VCProject &project, XmlOutput &xml, const QString &filtername);
+    static void outputFileConfigs(VCProject &project, XmlOutput &xml, const VCFilterFile &info, const QString &filtername);
+    static void outputFileConfig(VCFilter &filter, XmlOutput &xml, const QString &filename);
+
+    friend class TreeNode;
+    friend class FlatNode;
+};
 
 QT_END_NAMESPACE
 
