@@ -1,17 +1,18 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
-** All rights reserved.
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
@@ -21,8 +22,8 @@
 ** ensure the GNU Lesser General Public License version 2.1 requirements
 ** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
@@ -33,8 +34,7 @@
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
+**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -46,6 +46,7 @@
 #include "qdebug.h"
 #include "qmap.h"
 #include "qdatetime.h"
+#include "qeasingcurve.h"
 #include "qlist.h"
 #include "qstring.h"
 #include "qstringlist.h"
@@ -135,9 +136,11 @@ static void construct(QVariant::Private *x, const void *copy)
         v_construct<QPointF>(x, copy);
         break;
 #endif
+#ifndef QT_BOOTSTRAPPED
     case QVariant::Url:
         v_construct<QUrl>(x, copy);
         break;
+#endif
     case QVariant::Locale:
         v_construct<QLocale>(x, copy);
         break;
@@ -145,6 +148,11 @@ static void construct(QVariant::Private *x, const void *copy)
     case QVariant::RegExp:
         v_construct<QRegExp>(x, copy);
         break;
+#endif
+#ifndef QT_BOOTSTRAPPED
+    case QVariant::EasingCurve:
+        v_construct<QEasingCurve>(x, copy);
+        break;
 #endif
     case QVariant::Int:
         x->data.i = copy ? *static_cast<const int *>(copy) : 0;
@@ -248,9 +256,11 @@ static void clear(QVariant::Private *d)
         v_clear<QRectF>(d);
         break;
 #endif
+#ifndef QT_BOOTSTRAPPED
     case QVariant::Url:
         v_clear<QUrl>(d);
         break;
+#endif
     case QVariant::Locale:
         v_clear<QLocale>(d);
         break;
@@ -258,6 +268,11 @@ static void clear(QVariant::Private *d)
     case QVariant::RegExp:
         v_clear<QRegExp>(d);
         break;
+#endif
+#ifndef QT_BOOTSTRAPPED
+    case QVariant::EasingCurve:
+        v_clear<QEasingCurve>(d);
+        break;
 #endif
     case QVariant::LongLong:
     case QVariant::ULongLong:
@@ -317,7 +332,10 @@ static bool isNull(const QVariant::Private *d)
     case QVariant::PointF:
         return v_cast<QPointF>(d)->isNull();
 #endif
+#ifndef QT_BOOTSTRAPPED
+    case QVariant::EasingCurve:
     case QVariant::Url:
+#endif
     case QVariant::Locale:
     case QVariant::RegExp:
     case QVariant::StringList:
@@ -405,8 +423,10 @@ static bool compare(const QVariant::Private *a, const QVariant::Private *b)
     case QVariant::PointF:
         return *v_cast<QPointF>(a) == *v_cast<QPointF>(b);
 #endif
+#ifndef QT_BOOTSTRAPPED
     case QVariant::Url:
         return *v_cast<QUrl>(a) == *v_cast<QUrl>(b);
+#endif
     case QVariant::Locale:
         return *v_cast<QLocale>(a) == *v_cast<QLocale>(b);
 #ifndef QT_NO_REGEXP
@@ -435,6 +455,10 @@ static bool compare(const QVariant::Private *a, const QVariant::Private *b)
         return *v_cast<QTime>(a) == *v_cast<QTime>(b);
     case QVariant::DateTime:
         return *v_cast<QDateTime>(a) == *v_cast<QDateTime>(b);
+#ifndef QT_BOOTSTRAPPED
+    case QVariant::EasingCurve:
+        return *v_cast<QEasingCurve>(a) == *v_cast<QEasingCurve>(b);
+#endif
     case QVariant::ByteArray:
         return *v_cast<QByteArray>(a) == *v_cast<QByteArray>(b);
     case QVariant::BitArray:
@@ -466,9 +490,13 @@ static bool compare(const QVariant::Private *a, const QVariant::Private *b)
      * QMetaType::VoidStar, QMetaType::QObjectStar and so forth, is that it wouldn't include
      * user defined pointer types. */
     const char *const typeName = QMetaType::typeName(a->type);
-    if (typeName[qstrlen(typeName) - 1] == '*')
+    uint typeNameLen = qstrlen(typeName);
+    if (typeNameLen > 0 && typeName[typeNameLen - 1] == '*')
         return *static_cast<void *const *>(a_ptr) == *static_cast<void *const *>(b_ptr);
 
+    if (a->is_null && b->is_null)
+        return true;
+
     return a_ptr == b_ptr;
 }
 
@@ -603,6 +631,7 @@ static bool convert(const QVariant::Private *d, QVariant::Type t, void *result,
         ok = &dummy;
 
     switch (uint(t)) {
+#ifndef QT_BOOTSTRAPPED
     case QVariant::Url:
         switch (d->type) {
         case QVariant::String:
@@ -612,6 +641,7 @@ static bool convert(const QVariant::Private *d, QVariant::Type t, void *result,
             return false;
         }
         break;
+#endif
     case QVariant::String: {
         QString *str = static_cast<QString *>(result);
         switch (d->type) {
@@ -661,9 +691,11 @@ static bool convert(const QVariant::Private *d, QVariant::Type t, void *result,
             if (v_cast<QStringList>(d)->count() == 1)
                 *str = v_cast<QStringList>(d)->at(0);
             break;
+#ifndef QT_BOOTSTRAPPED
         case QVariant::Url:
             *str = v_cast<QUrl>(d)->toString();
             break;
+#endif
         default:
             return false;
         }
@@ -925,11 +957,7 @@ static bool convert(const QVariant::Private *d, QVariant::Type t, void *result,
         case QMetaType::UChar:
         case QMetaType::UShort:
         case QMetaType::ULong:
-#if defined(Q_CC_MSVC) && !defined(Q_CC_MSVC_NET)
-            *f = (double)(qlonglong)qMetaTypeUNumber(d);
-#else
             *f = double(qMetaTypeUNumber(d));
-#endif
             break;
         default:
             *f = 0.0;
@@ -964,11 +992,7 @@ static bool convert(const QVariant::Private *d, QVariant::Type t, void *result,
         case QMetaType::UChar:
         case QMetaType::UShort:
         case QMetaType::ULong:
-#if defined(Q_CC_MSVC) && !defined(Q_CC_MSVC_NET)
-            *f = (float)(qlonglong)qMetaTypeUNumber(d);
-#else
             *f = float(qMetaTypeUNumber(d));
-#endif
             break;
         default:
             *f = 0.0f;
@@ -1062,7 +1086,7 @@ static void streamDebug(QDebug dbg, const QVariant &v)
         dbg.nospace() << v.toFloat();
         break;
     case QMetaType::QObjectStar:
-        dbg.nospace() << qVariantValue<QObject *>(v);
+        dbg.nospace() << qvariant_cast<QObject *>(v);
         break;
     case QVariant::Double:
         dbg.nospace() << v.toDouble();
@@ -1097,12 +1121,19 @@ static void streamDebug(QDebug dbg, const QVariant &v)
     case QVariant::DateTime:
         dbg.nospace() << v.toDateTime();
         break;
+#ifndef QT_BOOTSTRAPPED
+    case QVariant::EasingCurve:
+        dbg.nospace() << v.toEasingCurve();
+        break;
+#endif
     case QVariant::ByteArray:
         dbg.nospace() << v.toByteArray();
         break;
+#ifndef QT_BOOTSTRAPPED
     case QVariant::Url:
         dbg.nospace() << v.toUrl();
         break;
+#endif
 #ifndef QT_NO_GEOM_VARIANT
     case QVariant::Point:
         dbg.nospace() << v.toPoint();
@@ -1218,7 +1249,7 @@ const QVariant::Handler *QVariant::handler = &qt_kernel_variant_handler;
     conversion functions to data types defined in QtGui, such as
     QColor, QImage, and QPixmap. In other words, there is no \c
     toColor() function. Instead, you can use the QVariant::value() or
-    the qVariantValue() template function. For example:
+    the qvariant_cast() template function. For example:
 
     \snippet doc/src/snippets/code/src_corelib_kernel_qvariant.cpp 2
 
@@ -1265,6 +1296,7 @@ const QVariant::Handler *QVariant::handler = &qt_kernel_variant_handler;
     \value Date  a QDate
     \value DateTime  a QDateTime
     \value Double  a double
+    \value EasingCurve a QEasingCurve
     \value Font  a QFont
     \value Hash a QVariantHash
     \value Icon  a QIcon
@@ -1332,12 +1364,12 @@ const QVariant::Handler *QVariant::handler = &qt_kernel_variant_handler;
 
     Note that you have to pass the address of the variable you want stored.
 
-    Usually, you never have to use this constructor, use qVariantFromValue()
+    Usually, you never have to use this constructor, use QVariant::fromValue()
     instead to construct variants from the pointer types represented by
     \c QMetaType::VoidStar, \c QMetaType::QObjectStar and
     \c QMetaType::QWidgetStar.
 
-    \sa qVariantFromValue(), Type
+    \sa QVariant::fromValue(), Type
 */
 
 /*!
@@ -1482,6 +1514,13 @@ QVariant::QVariant(const char *val)
     Constructs a new variant with a date/time value, \a val.
 */
 
+/*!
+    \since 4.7
+  \fn QVariant::QVariant(const QEasingCurve &val)
+
+    Constructs a new variant with an easing curve value, \a val.
+*/
+
 /*!
   \fn QVariant::QVariant(const QByteArray &val)
 
@@ -1681,6 +1720,10 @@ QVariant::QVariant(const QTime &val)
 { d.is_null = false; d.type = Time; v_construct<QTime>(&d, val); }
 QVariant::QVariant(const QDateTime &val)
 { d.is_null = false; d.type = DateTime; v_construct<QDateTime>(&d, val); }
+#ifndef QT_BOOTSTRAPPED
+QVariant::QVariant(const QEasingCurve &val)
+{ d.is_null = false; d.type = EasingCurve; v_construct<QEasingCurve>(&d, val); }
+#endif
 QVariant::QVariant(const QList<QVariant> &list)
 { d.is_null = false; d.type = List; v_construct<QVariantList>(&d, list); }
 QVariant::QVariant(const QMap<QString, QVariant> &map)
@@ -1697,7 +1740,9 @@ QVariant::QVariant(const QRect &r) { d.is_null = false; d.type = Rect; v_constru
 QVariant::QVariant(const QSize &s) { d.is_null = false; d.type = Size; v_construct<QSize>(&d, s); }
 QVariant::QVariant(const QSizeF &s) { d.is_null = false; d.type = SizeF; v_construct<QSizeF>(&d, s); }
 #endif
+#ifndef QT_BOOTSTRAPPED
 QVariant::QVariant(const QUrl &u) { d.is_null = false; d.type = Url; v_construct<QUrl>(&d, u); }
+#endif
 QVariant::QVariant(const QLocale &l) { d.is_null = false; d.type = Locale; v_construct<QLocale>(&d, l); }
 #ifndef QT_NO_REGEXP
 QVariant::QVariant(const QRegExp &regExp) { d.is_null = false; d.type = RegExp; v_construct<QRegExp>(&d, regExp); }
@@ -1776,6 +1821,14 @@ QVariant& QVariant::operator=(const QVariant &variant)
     return *this;
 }
 
+/*!
+    \fn void QVariant::swap(QVariant &other)
+    \since 4.8
+
+    Swaps variant \a other with this variant. This operation is very
+    fast and never fails.
+*/
+
 /*!
     \fn void QVariant::detach()
 
@@ -1870,7 +1923,7 @@ QVariant::Type QVariant::nameToType(const char *name)
 }
 
 #ifndef QT_NO_DATASTREAM
-enum { MapFromThreeCount = 35 };
+enum { MapFromThreeCount = 36 };
 static const ushort map_from_three[MapFromThreeCount] =
 {
     QVariant::Invalid,
@@ -1907,7 +1960,8 @@ static const ushort map_from_three[MapFromThreeCount] =
     QVariant::KeySequence,
     QVariant::Pen,
     QVariant::LongLong,
-    QVariant::ULongLong
+    QVariant::ULongLong,
+    QVariant::EasingCurve
 };
 
 /*!
@@ -2164,6 +2218,22 @@ QDateTime QVariant::toDateTime() const
     return qVariantToHelper<QDateTime>(d, DateTime, handler);
 }
 
+/*!
+    \since 4.7
+    \fn QEasingCurve QVariant::toEasingCurve() const
+
+    Returns the variant as a QEasingCurve if the variant has type() \l
+    EasingCurve; otherwise returns a default easing curve.
+
+    \sa canConvert(), convert()
+*/
+#ifndef QT_BOOTSTRAPPED
+QEasingCurve QVariant::toEasingCurve() const
+{
+    return qVariantToHelper<QEasingCurve>(d, EasingCurve, handler);
+}
+#endif
+
 /*!
     \fn QByteArray QVariant::toByteArray() const
 
@@ -2285,6 +2355,7 @@ QPointF QVariant::toPointF() const
 
 #endif // QT_NO_GEOM_VARIANT
 
+#ifndef QT_BOOTSTRAPPED
 /*!
     \fn QUrl QVariant::toUrl() const
 
@@ -2297,6 +2368,7 @@ QUrl QVariant::toUrl() const
 {
     return qVariantToHelper<QUrl>(d, Url, handler);
 }
+#endif
 
 /*!
     \fn QLocale QVariant::toLocale() const
@@ -2605,8 +2677,9 @@ static const quint32 qCanConvertMatrix[QVariant::LastCoreType + 1] =
 
 /*QRegExp*/       0,
 
-/*QHash*/         0
+/*QHash*/         0,
 
+/*QEasingCurve*/  0
 };
 
 /*!
@@ -3017,10 +3090,6 @@ QDebug operator<<(QDebug dbg, const QVariant::Type p)
 
     \snippet doc/src/snippets/code/src_corelib_kernel_qvariant.cpp 4
 
-    \warning This function is not available with MSVC 6. Use
-    qVariantSetValue() instead if you need to support that version of
-    the compiler.
-
     \sa value(), fromValue(), canConvert()
  */
 
@@ -3038,10 +3107,6 @@ QDebug operator<<(QDebug dbg, const QVariant::Type p)
 
     \snippet doc/src/snippets/code/src_corelib_kernel_qvariant.cpp 5
 
-    \warning This function is not available with MSVC 6. Use
-    qVariantValue() or qvariant_cast() instead if you need to support
-    that version of the compiler.
-
     \sa setValue(), fromValue(), canConvert()
 */
 
@@ -3054,10 +3119,6 @@ QDebug operator<<(QDebug dbg, const QVariant::Type p)
 
     \snippet doc/src/snippets/code/src_corelib_kernel_qvariant.cpp 6
 
-    \warning This function is not available with MSVC 6. Use
-    qVariantCanConvert() instead if you need to support that version
-    of the compiler.
-
     \sa convert()
 */
 
@@ -3073,23 +3134,22 @@ QDebug operator<<(QDebug dbg, const QVariant::Type p)
     \note If you are working with custom types, you should use
     the Q_DECLARE_METATYPE() macro to register your custom type.
 
-    \warning This function is not available with MSVC 6. Use
-    qVariantFromValue() instead if you need to support that version
-    of the compiler.
-
     \sa setValue(), value()
 */
 
 /*!
     \fn QVariant qVariantFromValue(const T &value)
     \relates QVariant
+    \obsolete
 
     Returns a variant containing a copy of the given \a value
     with template type \c{T}.
 
-    This function is equivalent to QVariant::fromValue(\a value). It
-    is provided as a work-around for MSVC 6, which doesn't support
-    member template functions.
+    This function is equivalent to QVariant::fromValue(\a value).
+
+    \note This function was provided as a workaround for MSVC 6
+    which did not support member template functions. It is advised
+    to use the other form in new code.
 
     For example, a QObject pointer can be stored in a variant with the
     following code:
@@ -3101,13 +3161,16 @@ QDebug operator<<(QDebug dbg, const QVariant::Type p)
 
 /*! \fn void qVariantSetValue(QVariant &variant, const T &value)
     \relates QVariant
+    \obsolete
 
     Sets the contents of the given \a variant to a copy of the
     \a value with the specified template type \c{T}.
 
-    This function is equivalent to QVariant::setValue(\a value). It
-    is provided as a work-around for MSVC 6, which doesn't support
-    member template functions.
+    This function is equivalent to QVariant::setValue(\a value).
+
+    \note This function was provided as a workaround for MSVC 6
+    which did not support member template functions. It is advised
+    to use the other form in new code.
 
     \sa QVariant::setValue()
 */
@@ -3118,33 +3181,39 @@ QDebug operator<<(QDebug dbg, const QVariant::Type p)
 
     Returns the given \a value converted to the template type \c{T}.
 
-    This function is equivalent to qVariantValue().
+    This function is equivalent to QVariant::value().
 
-    \sa qVariantValue(), QVariant::value()
+    \sa QVariant::value()
 */
 
 /*! \fn T qVariantValue(const QVariant &value)
     \relates QVariant
+    \obsolete
 
     Returns the given \a value converted to the template type \c{T}.
 
     This function is equivalent to
-    \l{QVariant::value()}{QVariant::value}<T>(\a value). It is
-    provided as a work-around for MSVC 6, which doesn't support
-    member template functions.
+    \l{QVariant::value()}{QVariant::value}<T>(\a value).
+
+    \note This function was provided as a workaround for MSVC 6
+    which did not support member template functions. It is advised
+    to use the other form in new code.
 
     \sa QVariant::value(), qvariant_cast()
 */
 
 /*! \fn bool qVariantCanConvert(const QVariant &value)
     \relates QVariant
+    \obsolete
 
     Returns true if the given \a value can be converted to the
     template type specified; otherwise returns false.
 
-    This function is equivalent to QVariant::canConvert(\a value). It
-    is provided as a work-around for MSVC 6, which doesn't support
-    member template functions.
+    This function is equivalent to QVariant::canConvert(\a value).
+
+    \note This function was provided as a workaround for MSVC 6
+    which did not support member template functions. It is advised
+    to use the other form in new code.
 
     \sa QVariant::canConvert()
 */
