@@ -1,17 +1,18 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
-** All rights reserved.
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
@@ -21,8 +22,8 @@
 ** ensure the GNU Lesser General Public License version 2.1 requirements
 ** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
@@ -33,8 +34,7 @@
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
+**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -68,6 +68,10 @@
 #include "private/qt_s60_p.h"
 #endif
 
+#if defined(Q_WS_S60) || defined(Q_WS_MAEMO_5)
+#  define QT_SMALL_COLORDIALOG
+#endif
+
 QT_BEGIN_NAMESPACE
 
 //////////// QWellArray BEGIN
@@ -640,6 +644,7 @@ protected:
     void paintEvent(QPaintEvent*);
     void mouseMoveEvent(QMouseEvent *);
     void mousePressEvent(QMouseEvent *);
+    void resizeEvent(QResizeEvent *);
 
 private:
     int hue;
@@ -650,7 +655,7 @@ private:
     int satPt(const QPoint &pt);
     void setCol(const QPoint &pt);
 
-    QPixmap *pix;
+    QPixmap pix;
 };
 
 static int pWidth = 220;
@@ -786,13 +791,27 @@ void QColorLuminancePicker::setCol(int h, int s , int v)
 }
 
 QPoint QColorPicker::colPt()
-{ return QPoint((360-hue)*(pWidth-1)/360, (255-sat)*(pHeight-1)/255); }
+{
+    QRect r = contentsRect();
+    return QPoint((360 - hue) * (r.width() - 1) / 360, (255 - sat) * (r.height() - 1) / 255);
+}
+
 int QColorPicker::huePt(const QPoint &pt)
-{ return 360 - pt.x()*360/(pWidth-1); }
+{
+    QRect r = contentsRect();
+    return 360 - pt.x() * 360 / (r.width() - 1);
+}
+
 int QColorPicker::satPt(const QPoint &pt)
-{ return 255 - pt.y()*255/(pHeight-1) ; }
+{
+    QRect r = contentsRect();
+    return 255 - pt.y() * 255 / (r.height() - 1);
+}
+
 void QColorPicker::setCol(const QPoint &pt)
-{ setCol(huePt(pt), satPt(pt)); }
+{
+    setCol(huePt(pt), satPt(pt));
+}
 
 QColorPicker::QColorPicker(QWidget* parent)
     : QFrame(parent)
@@ -800,29 +819,12 @@ QColorPicker::QColorPicker(QWidget* parent)
     hue = 0; sat = 0;
     setCol(150, 255);
 
-    QImage img(pWidth, pHeight, QImage::Format_RGB32);
-    int x, y;
-    uint *pixel = (uint *) img.scanLine(0);
-    for (y = 0; y < pHeight; y++) {
-        const uint *end = pixel + pWidth;
-        x = 0;
-        while (pixel < end) {
-            QPoint p(x, y);
-            QColor c;
-            c.setHsv(huePt(p), satPt(p), 200);
-            *pixel = c.rgb();
-            ++pixel;
-            ++x;
-        }
-    }
-    pix = new QPixmap(QPixmap::fromImage(img));
     setAttribute(Qt::WA_NoSystemBackground);
     setSizePolicy(QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed) );
 }
 
 QColorPicker::~QColorPicker()
 {
-    delete pix;
 }
 
 QSize QColorPicker::sizeHint() const
@@ -865,7 +867,7 @@ void QColorPicker::paintEvent(QPaintEvent* )
     drawFrame(&p);
     QRect r = contentsRect();
 
-    p.drawPixmap(r.topLeft(), *pix);
+    p.drawPixmap(r.topLeft(), pix);
     QPoint pt = colPt() + r.topLeft();
     p.setPen(Qt::black);
 
@@ -874,6 +876,31 @@ void QColorPicker::paintEvent(QPaintEvent* )
 
 }
 
+void QColorPicker::resizeEvent(QResizeEvent *ev)
+{
+    QFrame::resizeEvent(ev);
+
+    int w = width() - frameWidth() * 2;
+    int h = height() - frameWidth() * 2;
+    QImage img(w, h, QImage::Format_RGB32);
+    int x, y;
+    uint *pixel = (uint *) img.scanLine(0);
+    for (y = 0; y < h; y++) {
+        const uint *end = pixel + w;
+        x = 0;
+        while (pixel < end) {
+            QPoint p(x, y);
+            QColor c;
+            c.setHsv(huePt(p), satPt(p), 200);
+            *pixel = c.rgb();
+            ++pixel;
+            ++x;
+        }
+    }
+    pix = QPixmap::fromImage(img);
+}
+
+
 class QColSpinBox : public QSpinBox
 {
 public:
@@ -1072,14 +1099,16 @@ QColorShower::QColorShower(QColorDialog *parent)
     gl->setMargin(gl->spacing());
     lab = new QColorShowLabel(this);
 
-#ifdef Q_WS_S60
-    QS60Data s60Data = QS60Data();
-    const bool nonTouchUI = !s60Data.hasTouchscreen;
+#ifdef QT_SMALL_COLORDIALOG
+#  ifdef Q_WS_S60
+    const bool nonTouchUI = !S60->hasTouchscreen;
+#  elif defined Q_WS_MAEMO_5
+    const bool nonTouchUI = false;
+#  endif
 #endif
 
-
 #ifndef Q_WS_WINCE
-#ifdef Q_WS_S60
+#ifdef QT_SMALL_COLORDIALOG
     lab->setMinimumHeight(60);
 #endif
     lab->setMinimumWidth(60);
@@ -1090,7 +1119,7 @@ QColorShower::QColorShower(QColorDialog *parent)
 // In S60, due to small screen and different screen layouts need to re-arrange the widgets.
 // For QVGA screens only the comboboxes and color label are visible.
 // For nHD screens only color and luminence pickers and color label are visible.
-#ifndef Q_WS_S60
+#if !defined(QT_SMALL_COLORDIALOG)
     gl->addWidget(lab, 0, 0, -1, 1);
 #else
     if (nonTouchUI)
@@ -1108,7 +1137,7 @@ QColorShower::QColorShower(QColorDialog *parent)
     lblHue->setBuddy(hEd);
 #endif
     lblHue->setAlignment(Qt::AlignRight|Qt::AlignVCenter);
-#ifndef Q_WS_S60
+#if !defined(QT_SMALL_COLORDIALOG)
     gl->addWidget(lblHue, 0, 1);
     gl->addWidget(hEd, 0, 2);
 #else
@@ -1127,7 +1156,7 @@ QColorShower::QColorShower(QColorDialog *parent)
     lblSat->setBuddy(sEd);
 #endif
     lblSat->setAlignment(Qt::AlignRight|Qt::AlignVCenter);
-#ifndef Q_WS_S60
+#if !defined(QT_SMALL_COLORDIALOG)
     gl->addWidget(lblSat, 1, 1);
     gl->addWidget(sEd, 1, 2);
 #else
@@ -1146,7 +1175,7 @@ QColorShower::QColorShower(QColorDialog *parent)
     lblVal->setBuddy(vEd);
 #endif
     lblVal->setAlignment(Qt::AlignRight|Qt::AlignVCenter);
-#ifndef Q_WS_S60
+#if !defined(QT_SMALL_COLORDIALOG)
     gl->addWidget(lblVal, 2, 1);
     gl->addWidget(vEd, 2, 2);
 #else
@@ -1165,7 +1194,7 @@ QColorShower::QColorShower(QColorDialog *parent)
     lblRed->setBuddy(rEd);
 #endif
     lblRed->setAlignment(Qt::AlignRight|Qt::AlignVCenter);
-#ifndef Q_WS_S60
+#if !defined(QT_SMALL_COLORDIALOG)
     gl->addWidget(lblRed, 0, 3);
     gl->addWidget(rEd, 0, 4);
 #else
@@ -1184,7 +1213,7 @@ QColorShower::QColorShower(QColorDialog *parent)
     lblGreen->setBuddy(gEd);
 #endif
     lblGreen->setAlignment(Qt::AlignRight|Qt::AlignVCenter);
-#ifndef Q_WS_S60
+#if !defined(QT_SMALL_COLORDIALOG)
     gl->addWidget(lblGreen, 1, 3);
     gl->addWidget(gEd, 1, 4);
 #else
@@ -1203,7 +1232,7 @@ QColorShower::QColorShower(QColorDialog *parent)
     lblBlue->setBuddy(bEd);
 #endif
     lblBlue->setAlignment(Qt::AlignRight|Qt::AlignVCenter);
-#ifndef Q_WS_S60
+#if !defined(QT_SMALL_COLORDIALOG)
     gl->addWidget(lblBlue, 2, 3);
     gl->addWidget(bEd, 2, 4);
 #else
@@ -1222,7 +1251,7 @@ QColorShower::QColorShower(QColorDialog *parent)
     alphaLab->setBuddy(alphaEd);
 #endif
     alphaLab->setAlignment(Qt::AlignRight|Qt::AlignVCenter);
-#ifndef Q_WS_S60
+#if !defined(QT_SMALL_COLORDIALOG)
     gl->addWidget(alphaLab, 3, 1, 1, 3);
     gl->addWidget(alphaEd, 3, 4);
 #else
@@ -1467,7 +1496,7 @@ void QColorDialogPrivate::init(const QColor &initial)
 
     leftLay = 0;
 
-#if (defined(Q_WS_WINCE) || defined(Q_WS_S60))
+#if defined(Q_WS_WINCE) || defined(QT_SMALL_COLORDIALOG)
     smallDisplay = true;
     const int lumSpace = 20;
 #else
@@ -1497,9 +1526,12 @@ void QColorDialogPrivate::init(const QColor &initial)
     }
 #endif
 
-#if defined(Q_WS_S60)
-    QS60Data s60Data = QS60Data();
-    const bool nonTouchUI = !s60Data.hasTouchscreen;
+#if defined(QT_SMALL_COLORDIALOG)
+#  if defined(Q_WS_S60)
+    const bool nonTouchUI = !S60->hasTouchscreen;
+#  elif defined(Q_WS_MAEMO_5)
+    const bool nonTouchUI = false;
+#  endif
 #endif
 
     if (!smallDisplay) {
@@ -1532,7 +1564,7 @@ void QColorDialogPrivate::init(const QColor &initial)
         leftLay->addWidget(addCusBt);
     } else {
         // better color picker size for small displays
-#ifdef Q_WS_S60
+#if defined(QT_SMALL_COLORDIALOG)
         QSize screenSize = QApplication::desktop()->availableGeometry(QCursor::pos()).size();
         pWidth = pHeight = qMin(screenSize.width(), screenSize.height());
         pHeight -= 20;
@@ -1558,7 +1590,7 @@ void QColorDialogPrivate::init(const QColor &initial)
 
     cp->setFrameStyle(QFrame::Panel + QFrame::Sunken);
 
-#if defined(Q_WS_S60)
+#if defined(QT_SMALL_COLORDIALOG)
     if (!nonTouchUI) {
         pickLay->addWidget(cp);
         cLay->addSpacing(lumSpace);
@@ -1572,7 +1604,7 @@ void QColorDialogPrivate::init(const QColor &initial)
     cLay->addSpacing(lumSpace);
 
     lp = new QColorLuminancePicker(q);
-#if defined(Q_WS_S60)
+#if defined(QT_SMALL_COLORDIALOG)
     QSize screenSize = QApplication::desktop()->availableGeometry(QCursor::pos()).size();
     const int minDimension = qMin(screenSize.height(), screenSize.width());
     //set picker to be finger-usable
@@ -1594,9 +1626,9 @@ void QColorDialogPrivate::init(const QColor &initial)
 
     cs = new QColorShower(q);
     QObject::connect(cs, SIGNAL(newCol(QRgb)), q, SLOT(_q_newColorTypedIn(QRgb)));
-    QObject::connect(cs, SIGNAL(currentColorChanged(const QColor&)),
-                     q, SIGNAL(currentColorChanged(const QColor&)));
-#if defined(Q_WS_S60)
+    QObject::connect(cs, SIGNAL(currentColorChanged(QColor)),
+                     q, SIGNAL(currentColorChanged(QColor)));
+#if defined(QT_SMALL_COLORDIALOG)
     if (!nonTouchUI)
         pWidth -= cp->size().width();
     topLay->addWidget(cs);
@@ -1670,9 +1702,6 @@ static const Qt::WindowFlags DefaultWindowFlags =
     during the execution of the program. Use setCustomColor() to set
     the custom colors, and use customColor() to get them.
 
-    Additional widgets that allow users to pick colors are available
-    as \l{Qt Solutions}.
-
     The \l{dialogs/standarddialogs}{Standard Dialogs} example shows
     how to use QColorDialog as well as other built-in Qt dialogs.
 
@@ -1815,8 +1844,8 @@ QColorDialog::ColorDialogOptions QColorDialog::options() const
 
     \value ShowAlphaChannel Allow the user to select the alpha component of a color.
     \value NoButtons Don't display \gui{OK} and \gui{Cancel} buttons. (Useful for "live dialogs".)
-    \value DontUseNativeDialog Use Qt's standard color dialog on the Mac instead of Apple's
-                               native color panel.
+    \value DontUseNativeDialog Use Qt's standard color dialog on the Mac instead of the operating system
+                               native color dialog.
 
     \sa options, setOption(), testOption(), windowModality()
 */
@@ -1873,6 +1902,7 @@ void QColorDialog::setVisible(bool visible)
     } else {
         if (d->delegate) {
             d->closeCocoaColorPanel();
+            QColorDialogPrivate::sharedColorPanelAvailable = true;
             setAttribute(Qt::WA_DontShowOnScreen, false);
         }
     }
@@ -1904,7 +1934,7 @@ void QColorDialog::setVisible(bool visible)
 void QColorDialog::open(QObject *receiver, const char *member)
 {
     Q_D(QColorDialog);
-    connect(this, SIGNAL(colorSelected(const QColor&)), receiver, member);
+    connect(this, SIGNAL(colorSelected(QColor)), receiver, member);
     d->receiverToDisconnectOnClose = receiver;
     d->memberToDisconnectOnClose = member;
     QDialog::open();
@@ -1920,6 +1950,12 @@ void QColorDialog::open(QObject *receiver, const char *member)
     \sa QDialog::open()
 */
 
+/*
+    For Symbian color dialogs
+*/
+#ifdef Q_WS_S60
+extern QColor qtSymbianGetColor(const QColor &initial);
+#endif
 /*!
     \since 4.5
 
@@ -1929,10 +1965,19 @@ void QColorDialog::open(QObject *receiver, const char *member)
     QColor::isValid()) color if the user cancels the dialog.
 
     The \a options argument allows you to customize the dialog.
+
+    On Symbian, this static function will use the native color dialog and not a QColorDialog.
+    On Symbian the parameters \a title and \a parent has no relevance and the
+    \a options parameter is only used to define if the native color dialog is
+    used or not.
 */
 QColor QColorDialog::getColor(const QColor &initial, QWidget *parent, const QString &title,
                               ColorDialogOptions options)
 {
+#ifdef Q_WS_S60
+    if (!(options & DontUseNativeDialog))
+        return qtSymbianGetColor(initial);
+#endif
     QColorDialog dlg(parent);
     if (!title.isEmpty())
         dlg.setWindowTitle(title);
@@ -1947,10 +1992,16 @@ QColor QColorDialog::getColor(const QColor &initial, QWidget *parent, const QStr
     returns that color. The color is initially set to \a initial. The
     dialog is a child of \a parent. It returns an invalid (see
     QColor::isValid()) color if the user cancels the dialog.
+
+    On Symbian, this static function will use the native
+    color dialog and not a QColorDialog.
 */
 
 QColor QColorDialog::getColor(const QColor &initial, QWidget *parent)
 {
+#ifdef Q_WS_S60
+    return qtSymbianGetColor(initial);
+#endif
     return getColor(initial, parent, QString(), ColorDialogOptions(0));
 }
 
@@ -2036,7 +2087,7 @@ void QColorDialog::done(int result)
         d->selectedQColor = QColor();
     }
     if (d->receiverToDisconnectOnClose) {
-        disconnect(this, SIGNAL(colorSelected(const QColor&)),
+        disconnect(this, SIGNAL(colorSelected(QColor)),
                    d->receiverToDisconnectOnClose, d->memberToDisconnectOnClose);
         d->receiverToDisconnectOnClose = 0;
     }
