@@ -1,17 +1,18 @@
 /****************************************************************************
 **
-** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
-** All rights reserved.
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
@@ -21,8 +22,8 @@
 ** ensure the GNU Lesser General Public License version 2.1 requirements
 ** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
@@ -33,8 +34,7 @@
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
+**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -161,7 +161,6 @@ void QDockWidgetTitleButton::paintEvent(QPaintEvent *)
 {
     QPainter p(this);
 
-    QRect r = rect();
     QStyleOptionToolButton opt;
     opt.init(this);
     opt.state |= QStyle::State_AutoRaise;
@@ -691,6 +690,7 @@ void QDockWidgetPrivate::initDrag(const QPoint &pos, bool nca)
     if (state != 0)
         return;
 
+    Q_Q(QDockWidget);
     QMainWindow *win = qobject_cast<QMainWindow*>(parent);
     Q_ASSERT(win != 0);
     QMainWindowLayout *layout = qt_mainwindow_layout(win);
@@ -699,12 +699,23 @@ void QDockWidgetPrivate::initDrag(const QPoint &pos, bool nca)
         return;
 
     state = new QDockWidgetPrivate::DragState;
-    state->pressPos = pos;
     state->dragging = false;
     state->widgetItem = 0;
     state->ownWidgetItem = false;
     state->nca = nca;
     state->ctrlDrag = false;
+
+    if (!q->isFloating()) {
+        // When dragging the widget out of the docking area,
+        // use the middle of title area as pressPos
+        QDockWidgetLayout *dwlayout = qobject_cast<QDockWidgetLayout*>(q->layout());
+        Q_ASSERT(dwlayout != 0);
+        int width = undockedGeometry.isNull() ? q->width() : undockedGeometry.width();
+        state->pressPos.setY(dwlayout->titleArea().height() / 2);
+        state->pressPos.setX(width / 2);
+    } else {
+        state->pressPos = pos;
+    }
 }
 
 void QDockWidgetPrivate::startDrag()
@@ -988,11 +999,17 @@ void QDockWidgetPrivate::moveEvent(QMoveEvent *event)
 void QDockWidgetPrivate::unplug(const QRect &rect)
 {
     Q_Q(QDockWidget);
-    QRect r = rect;
-    r.moveTopLeft(q->mapToGlobal(QPoint(0, 0)));
-    QDockWidgetLayout *dwLayout = qobject_cast<QDockWidgetLayout*>(layout);
-    if (dwLayout->nativeWindowDeco(true))
-        r.adjust(0, dwLayout->titleHeight(), 0, 0);
+    QRect r;
+    if (!undockedGeometry.isNull()) {
+        r = undockedGeometry;
+    } else {
+        r = rect;
+        r.moveTopLeft(q->mapToGlobal(QPoint(0, 0)));
+        QDockWidgetLayout *dwLayout = qobject_cast<QDockWidgetLayout*>(layout);
+        if (dwLayout->nativeWindowDeco(true))
+            r.adjust(0, dwLayout->titleHeight(), 0, 0);
+    }
+
     setWindowState(true, true, r);
 }
 
@@ -1264,6 +1281,9 @@ void QDockWidget::setFloating(bool floating)
         d->endDrag(true);
 
     QRect r = d->undockedGeometry;
+    // Keep position when undocking for the first time.
+    if (floating && isVisible() && !r.isValid())
+        r = QRect(mapToGlobal(QPoint(0, 0)), size());
 
     d->setWindowState(floating, false, floating ? r : QRect());
 
@@ -1389,9 +1409,12 @@ bool QDockWidget::event(QEvent *event)
         d->toggleViewAction->setChecked(false);
         emit visibilityChanged(false);
         break;
-    case QEvent::Show:
+    case QEvent::Show: {
         d->toggleViewAction->setChecked(true);
-        emit visibilityChanged(geometry().right() >= 0 && geometry().bottom() >= 0);
+        const QPoint parentTopLeft = isWindow() ?
+            QApplication::desktop()->availableGeometry(this).topLeft() : QPoint(0, 0);
+        emit visibilityChanged(geometry().right() >= parentTopLeft.x() && geometry().bottom() >= parentTopLeft.y());
+}
         break;
 #endif
     case QEvent::ApplicationLayoutDirectionChange:
@@ -1531,8 +1554,11 @@ QAction * QDockWidget::toggleViewAction() const
 
 /*!
     \since 4.3
+
     Sets an arbitrary \a widget as the dock widget's title bar. If \a widget
-    is 0, the title bar widget is removed, but not deleted.
+    is 0, any custom title bar widget previously set on the dock widget is
+    removed, but not deleted, and the default title bar will be used
+    instead.
 
     If a title bar widget is set, QDockWidget will not use native window
     decorations when it is floated.
@@ -1540,23 +1566,27 @@ QAction * QDockWidget::toggleViewAction() const
     Here are some tips for implementing custom title bars:
 
     \list
-    \i Mouse events that are not explicitly handled by the title bar widget
+    \o Mouse events that are not explicitly handled by the title bar widget
        must be ignored by calling QMouseEvent::ignore(). These events then
        propagate to the QDockWidget parent, which handles them in the usual
        manner, moving when the title bar is dragged, docking and undocking
        when it is double-clicked, etc.
 
-    \i When DockWidgetVerticalTitleBar is set on QDockWidget, the title
+    \o When DockWidgetVerticalTitleBar is set on QDockWidget, the title
        bar widget is repositioned accordingly. In resizeEvent(), the title
        bar should check what orientation it should assume:
        \snippet doc/src/snippets/code/src_gui_widgets_qdockwidget.cpp 0
 
-    \i The title bar widget must have a valid QWidget::sizeHint() and
+    \o The title bar widget must have a valid QWidget::sizeHint() and
        QWidget::minimumSizeHint(). These functions should take into account
        the current orientation of the title bar.
+
+    \o It is not possible to remove a title bar from a dock widget. However,
+       a similar effect can be achieved by setting a default constructed
+       QWidget as the title bar widget.
     \endlist
 
-    Using qobject_cast as shown above, the title bar widget has full access
+    Using qobject_cast() as shown above, the title bar widget has full access
     to its parent QDockWidget. Hence it can perform such operations as docking
     and hiding in response to user actions.
 
