@@ -1,38 +1,38 @@
 /****************************************************************************
 **
-** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
-** All rights reserved.
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the QML Shaders plugin of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** GNU Lesser General Public License Usage
-** This file may be used under the terms of the GNU Lesser General Public
-** License version 2.1 as published by the Free Software Foundation and
-** appearing in the file LICENSE.LGPL included in the packaging of this
-** file. Please review the following information to ensure the GNU Lesser
-** General Public License version 2.1 requirements will be met:
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
-** In addition, as a special exception, Nokia gives you certain additional
-** rights. These rights are described in the Nokia Qt LGPL Exception
+** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+**
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU General
-** Public License version 3.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of this
-** file. Please review the following information to ensure the GNU General
-** Public License version 3.0 requirements will be met:
-** http://www.gnu.org/copyleft/gpl.html.
-**
-** Other Usage
-** Alternatively, this file may be used in accordance with the terms and
-** conditions contained in a signed written agreement between you and Nokia.
-**
-**
-**
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3.0 as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU General Public License version 3.0 requirements will be
+** met: http://www.gnu.org/copyleft/gpl.html.
 **
 **
 ** $QT_END_LICENSE$
@@ -199,10 +199,6 @@ Rectangle {
 
 */
 
-#ifdef Q_OS_SYMBIAN
-#define OBSERVE_GL_CONTEXT_LOSS 1
-#endif
-
 ShaderEffectItem::ShaderEffectItem(QDeclarativeItem *parent)
     : QDeclarativeItem(parent)
     , m_program(0)
@@ -219,21 +215,15 @@ ShaderEffectItem::ShaderEffectItem(QDeclarativeItem *parent)
     , m_hasShaderPrograms(false)
     , m_mirrored(false)
     , m_defaultVertexShader(true)
-    , m_contextObserver(0)
 {
     setFlag(QGraphicsItem::ItemHasNoContents, false);
     connect(this, SIGNAL(visibleChanged()), this, SLOT(handleVisibilityChange()));
     m_active = isVisible();
-
-#ifndef OBSERVE_GL_CONTEXT_LOSS
-    m_program = new QGLShaderProgram(this);
-#endif
 }
 
 ShaderEffectItem::~ShaderEffectItem()
 {
     reset();
-    delete m_contextObserver;
 }
 
 
@@ -413,30 +403,15 @@ void ShaderEffectItem::renderEffect(QPainter *painter, const QMatrix4x4 &matrix)
     if (!painter || !painter->device())
         return;
 
-#ifdef OBSERVE_GL_CONTEXT_LOSS
-    QGLContext *context = const_cast <QGLContext*> (QGLContext::currentContext());
-    if (!m_program || !m_contextObserver || !m_contextObserver->isValid()) {
-        // Context has changed, re-create QGLShaderProgram
-        if (context) {
-            delete m_program;
-            m_program = 0;
-
-            delete m_contextObserver;
-            m_contextObserver = 0;
-
+    if (!m_program || !m_program->programId()) {
+        // Deleted due to deactivation, to save GPU memory,
+        // or invalidated due to GL context change.
+        delete m_program;
+        if (QGLContext::currentContext())
             m_program = new QGLShaderProgram(this);
-            m_contextObserver = new QGLFramebufferObject(QSize(2,2));
-
-            if (!m_contextObserver || !m_program) {
-                delete m_program;
-                m_program = 0;
-                delete m_contextObserver;
-                m_contextObserver = 0;
-                qWarning() << "ShaderEffectItem::renderEffect - Creating QGLShaderProgram or QGLFrameBufferObject failed!";
-            }
-        }
+        if (!m_program)
+            qWarning() << "ShaderEffectItem::renderEffect - Creating QGLShaderProgram failed!";
     }
-#endif
 
     if (!m_program)
         return;
@@ -683,14 +658,10 @@ void ShaderEffectItem::setActive(bool enable)
     }
 
     // QGLShaderProgram is deleted when not active (to minimize GPU memory usage).
-#ifdef OBSERVE_GL_CONTEXT_LOSS
     if (!m_active && m_program) {
         delete m_program;
         m_program = 0;
-        delete m_contextObserver;
-        m_contextObserver = 0;
     }
-#endif
 
     emit activeChanged();
     markDirty();
