@@ -1,38 +1,38 @@
 /****************************************************************************
 **
-** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
-** All rights reserved.
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the QtMultimedia module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** GNU Lesser General Public License Usage
-** This file may be used under the terms of the GNU Lesser General Public
-** License version 2.1 as published by the Free Software Foundation and
-** appearing in the file LICENSE.LGPL included in the packaging of this
-** file. Please review the following information to ensure the GNU Lesser
-** General Public License version 2.1 requirements will be met:
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
-** In addition, as a special exception, Nokia gives you certain additional
-** rights. These rights are described in the Nokia Qt LGPL Exception
+** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+**
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU General
-** Public License version 3.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of this
-** file. Please review the following information to ensure the GNU General
-** Public License version 3.0 requirements will be met:
-** http://www.gnu.org/copyleft/gpl.html.
-**
-** Other Usage
-** Alternatively, this file may be used in accordance with the terms and
-** conditions contained in a signed written agreement between you and Nokia.
-**
-**
-**
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3.0 as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU General Public License version 3.0 requirements will be
+** met: http://www.gnu.org/copyleft/gpl.html.
 **
 **
 ** $QT_END_LICENSE$
@@ -51,6 +51,7 @@
 //
 
 #include "qaudiooutput_win32_p.h"
+#include <QtEndian>
 
 #ifndef SPEAKER_FRONT_LEFT
     #define SPEAKER_FRONT_LEFT            0x00000001
@@ -130,7 +131,7 @@ QAudioOutputPrivate::~QAudioOutputPrivate()
 }
 
 void CALLBACK QAudioOutputPrivate::waveOutProc( HWAVEOUT hWaveOut, UINT uMsg,
-        DWORD dwInstance, DWORD dwParam1, DWORD dwParam2 )
+        DWORD_PTR dwInstance, DWORD_PTR dwParam1, DWORD_PTR dwParam2 )
 {
     Q_UNUSED(dwParam1)
     Q_UNUSED(dwParam2)
@@ -262,7 +263,7 @@ bool QAudioOutputPrivate::open()
     } else if (settings.sampleSize() <= 0) {
         qWarning("QAudioOutput: open error, invalid sample size (%d).",
                  settings.sampleSize());
-    } else if (settings.frequency() < 8000 || settings.frequency() > 48000) {
+    } else if (settings.frequency() < 8000 || settings.frequency() > 96000) {
         qWarning("QAudioOutput: open error, frequency out of range (%d).", settings.frequency());
     } else if (buffer_size == 0) {
         // Default buffer size, 200ms, default period size is 40ms
@@ -455,6 +456,30 @@ qint64 QAudioOutputPrivate::write( const char *data, qint64 len )
     char* p = (char*)data;
     int l = (int)len;
 
+    QByteArray reverse;
+    if (settings.byteOrder() == QAudioFormat::BigEndian) {
+
+        switch (settings.sampleSize()) {
+            case 8:
+                // No need to convert
+                break;
+
+            case 16:
+                reverse.resize(l);
+                for (qint64 i = 0; i < (l >> 1); i++)
+                    *((qint16*)reverse.data() + i) = qFromBigEndian(*((qint16*)data + i));
+                p = reverse.data();
+                break;
+
+            case 32:
+                reverse.resize(l);
+                for (qint64 i = 0; i < (l >> 2); i++)
+                    *((qint32*)reverse.data() + i) = qFromBigEndian(*((qint32*)data + i));
+                p = reverse.data();
+                break;
+        }
+    }
+
     WAVEHDR* current;
     int remain;
     current = &waveBlocks[waveCurrentBlock];
