@@ -1,17 +1,18 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
-** All rights reserved.
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the QtMultimedia module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
@@ -21,8 +22,8 @@
 ** ensure the GNU Lesser General Public License version 2.1 requirements
 ** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
@@ -33,8 +34,7 @@
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
+**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -136,7 +136,6 @@ public:
     \class QVideoSurfaceFormat
     \brief The QVideoSurfaceFormat class specifies the stream format of a video presentation
     surface.
-    \preliminary
     \since 4.6
 
     A video surface presents a stream of video frames.  The surface's format describes the type of
@@ -253,7 +252,7 @@ QVideoSurfaceFormat::~QVideoSurfaceFormat()
 
 bool QVideoSurfaceFormat::isValid() const
 {
-    return d->pixelFormat == QVideoFrame::Format_Invalid && d->frameSize.isValid();
+    return d->pixelFormat != QVideoFrame::Format_Invalid && d->frameSize.isValid();
 }
 
 /*!
@@ -503,11 +502,11 @@ QList<QByteArray> QVideoSurfaceFormat::propertyNames() const
 QVariant QVideoSurfaceFormat::property(const char *name) const
 {
     if (qstrcmp(name, "handleType") == 0) {
-        return qVariantFromValue(d->handleType);
+        return QVariant::fromValue(d->handleType);
     } else if (qstrcmp(name, "pixelFormat") == 0) {
-        return qVariantFromValue(d->pixelFormat);
+        return QVariant::fromValue(d->pixelFormat);
     } else if (qstrcmp(name, "handleType") == 0) {
-        return qVariantFromValue(d->handleType);
+        return QVariant::fromValue(d->handleType);
     } else if (qstrcmp(name, "frameSize") == 0) {
         return d->frameSize;
     } else if (qstrcmp(name, "frameWidth") == 0) {
@@ -517,15 +516,15 @@ QVariant QVideoSurfaceFormat::property(const char *name) const
     } else if (qstrcmp(name, "viewport") == 0) {
         return d->viewport;
     } else if (qstrcmp(name, "scanLineDirection") == 0) {
-        return qVariantFromValue(d->scanLineDirection);
+        return QVariant::fromValue(d->scanLineDirection);
     } else if (qstrcmp(name, "frameRate") == 0) {
-        return qVariantFromValue(d->frameRate);
+        return QVariant::fromValue(d->frameRate);
     } else if (qstrcmp(name, "pixelAspectRatio") == 0) {
-        return qVariantFromValue(d->pixelAspectRatio);
+        return QVariant::fromValue(d->pixelAspectRatio);
     } else if (qstrcmp(name, "sizeHint") == 0) {
         return sizeHint();
     } else if (qstrcmp(name, "yCbCrColorSpace") == 0) {
-        return qVariantFromValue(d->ycbcrColorSpace);
+        return QVariant::fromValue(d->ycbcrColorSpace);
     } else {
         int id = 0;
         for (; id < d->propertyNames.count() && d->propertyNames.at(id) != name; ++id) {}
@@ -547,7 +546,7 @@ void QVideoSurfaceFormat::setProperty(const char *name, const QVariant &value)
     } else if (qstrcmp(name, "pixelFormat") == 0) {
         // read only.
     } else if (qstrcmp(name, "frameSize") == 0) {
-        if (qVariantCanConvert<QSize>(value)) {
+        if (value.canConvert<QSize>()) {
             d->frameSize = qvariant_cast<QSize>(value);
             d->viewport = QRect(QPoint(0, 0), d->frameSize);
         }
@@ -556,21 +555,21 @@ void QVideoSurfaceFormat::setProperty(const char *name, const QVariant &value)
     } else if (qstrcmp(name, "frameHeight") == 0) {
         // read only.
     } else if (qstrcmp(name, "viewport") == 0) {
-        if (qVariantCanConvert<QRect>(value))
+        if (value.canConvert<QRect>())
             d->viewport = qvariant_cast<QRect>(value);
     } else if (qstrcmp(name, "scanLineDirection") == 0) {
-        if (qVariantCanConvert<Direction>(value))
+        if (value.canConvert<Direction>())
             d->scanLineDirection = qvariant_cast<Direction>(value);
     } else if (qstrcmp(name, "frameRate") == 0) {
-        if (qVariantCanConvert<qreal>(value))
+        if (value.canConvert<qreal>())
             d->frameRate = qvariant_cast<qreal>(value);
     } else if (qstrcmp(name, "pixelAspectRatio") == 0) {
-        if (qVariantCanConvert<QSize>(value))
+        if (value.canConvert<QSize>())
             d->pixelAspectRatio = qvariant_cast<QSize>(value);
     } else if (qstrcmp(name, "sizeHint") == 0) {
         // read only.
     } else if (qstrcmp(name, "yCbCrColorSpace") == 0) {
-          if (qVariantCanConvert<YCbCrColorSpace>(value))
+          if (value.canConvert<YCbCrColorSpace>())
               d->ycbcrColorSpace = qvariant_cast<YCbCrColorSpace>(value);
     } else {
         int id = 0;
