@@ -1,17 +1,18 @@
 /****************************************************************************
 **
-** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
-** All rights reserved.
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the QtOpenGL module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
@@ -21,8 +22,8 @@
 ** ensure the GNU Lesser General Public License version 2.1 requirements
 ** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
@@ -33,8 +34,7 @@
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
+**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -49,16 +49,18 @@
 #include <private/qgl_p.h>
 #include <private/qdrawhelper_p.h>
 #include <private/qimage_p.h>
+#include <private/qfont_p.h>
 
 #include <private/qpaintengineex_opengl2_p.h>
 
 #include <qdesktopwidget.h>
 #include <qfile.h>
 #include <qimagereader.h>
+#include <qbuffer.h>
 
 QT_BEGIN_NAMESPACE
 
-extern QGLWidget* qt_gl_share_widget();
+Q_OPENGL_EXPORT extern const QGLContext* qt_gl_share_context();
 
 /*!
     \class QGLFramebufferObjectPool
@@ -261,14 +263,14 @@ QGLPixmapData::QGLPixmapData(PixelType type)
 
 QGLPixmapData::~QGLPixmapData()
 {
-    QGLWidget *shareWidget = qt_gl_share_widget();
-    if (!shareWidget)
+    const QGLContext *shareContext = qt_gl_share_context();
+    if (!shareContext)
         return;
 
     delete m_engine;
 
     if (m_texture.id) {
-        QGLShareContextScope ctx(shareWidget->context());
+        QGLShareContextScope ctx(shareContext);
         glDeleteTextures(1, &m_texture.id);
     }
 }
@@ -288,7 +290,7 @@ bool QGLPixmapData::isValidContext(const QGLContext *ctx) const
     if (ctx == m_ctx)
         return true;
 
-    const QGLContext *share_ctx = qt_gl_share_widget()->context();
+    const QGLContext *share_ctx = qt_gl_share_context();
     return ctx == share_ctx || QGLContext::areSharing(ctx, share_ctx);
 }
 
@@ -308,7 +310,7 @@ void QGLPixmapData::resize(int width, int height)
     d = pixelType() == QPixmapData::PixmapType ? 32 : 1;
 
     if (m_texture.id) {
-        QGLShareContextScope ctx(qt_gl_share_widget()->context());
+        QGLShareContextScope ctx(qt_gl_share_context());
         glDeleteTextures(1, &m_texture.id);
         m_texture.id = 0;
     }
@@ -325,7 +327,7 @@ void QGLPixmapData::ensureCreated() const
 
     m_dirty = false;
 
-    QGLShareContextScope ctx(qt_gl_share_widget()->context());
+    QGLShareContextScope ctx(qt_gl_share_context());
     m_ctx = ctx;
 
     const GLenum internal_format = m_hasAlpha ? GL_RGBA : GL_RGB;
@@ -368,9 +370,102 @@ void QGLPixmapData::ensureCreated() const
 
 void QGLPixmapData::fromImage(const QImage &image,
                               Qt::ImageConversionFlags flags)
+{
+    QImage img = image;
+    createPixmapForImage(img, flags, false);
+}
+
+void QGLPixmapData::fromImageReader(QImageReader *imageReader,
+                                 Qt::ImageConversionFlags flags)
+{
+    QImage image = imageReader->read();
+    if (image.isNull())
+        return;
+
+    createPixmapForImage(image, flags, true);
+}
+
+bool QGLPixmapData::fromFile(const QString &filename, const char *format,
+                             Qt::ImageConversionFlags flags)
+{
+    if (pixelType() == QPixmapData::BitmapType)
+        return QPixmapData::fromFile(filename, format, flags);
+    QFile file(filename);
+    if (file.open(QIODevice::ReadOnly)) {
+        QByteArray data = file.peek(64);
+        bool alpha;
+        if (m_texture.canBindCompressedTexture
+                (data.constData(), data.size(), format, &alpha)) {
+            resize(0, 0);
+            data = file.readAll();
+            file.close();
+            QGLShareContextScope ctx(qt_gl_share_context());
+            QSize size = m_texture.bindCompressedTexture
+                (data.constData(), data.size(), format);
+            if (!size.isEmpty()) {
+                w = size.width();
+                h = size.height();
+                is_null = false;
+                d = 32;
+                m_hasAlpha = alpha;
+                m_source = QImage();
+                m_dirty = isValid();
+                return true;
+            }
+            return false;
+        }
+    }
+
+    QImage image = QImageReader(filename, format).read();
+    if (image.isNull())
+        return false;
+
+    createPixmapForImage(image, flags, true);
+
+    return !isNull();
+}
+
+bool QGLPixmapData::fromData(const uchar *buffer, uint len, const char *format,
+                             Qt::ImageConversionFlags flags)
+{
+    bool alpha;
+    const char *buf = reinterpret_cast<const char *>(buffer);
+    if (m_texture.canBindCompressedTexture(buf, int(len), format, &alpha)) {
+        resize(0, 0);
+        QGLShareContextScope ctx(qt_gl_share_context());
+        QSize size = m_texture.bindCompressedTexture(buf, int(len), format);
+        if (!size.isEmpty()) {
+            w = size.width();
+            h = size.height();
+            is_null = false;
+            d = 32;
+            m_hasAlpha = alpha;
+            m_source = QImage();
+            m_dirty = isValid();
+            return true;
+        }
+    }
+
+    QByteArray a = QByteArray::fromRawData(reinterpret_cast<const char *>(buffer), len);
+    QBuffer b(&a);
+    b.open(QIODevice::ReadOnly);
+    QImage image = QImageReader(&b, format).read();
+    if (image.isNull())
+        return false;
+
+    createPixmapForImage(image, flags, true);
+
+    return !isNull();
+}
+
+/*!
+    out-of-place conversion (inPlace == false) will always detach()
+ */
+void QGLPixmapData::createPixmapForImage(QImage &image, Qt::ImageConversionFlags flags, bool inPlace)
 {
     if (image.size() == QSize(w, h))
         setSerialNumber(++qt_gl_pixmap_serial);
+
     resize(image.width(), image.height());
 
     if (pixelType() == BitmapType) {
@@ -386,7 +481,15 @@ void QGLPixmapData::fromImage(const QImage &image,
                 || const_cast<QImage &>(image).data_ptr()->checkForAlphaPixels()))
             format = QImage::Format_ARGB32_Premultiplied;;
 
-        m_source = image.convertToFormat(format);
+        if (inPlace && image.data_ptr()->convertInPlace(format, flags)) {
+            m_source = image;
+        } else {
+            m_source = image.convertToFormat(format);
+
+            // convertToFormat won't detach the image if format stays the same.
+            if (image.format() == format)
+                m_source.detach();
+        }
     }
 
     m_dirty = true;
@@ -399,69 +502,12 @@ void QGLPixmapData::fromImage(const QImage &image,
     d = m_source.depth();
 
     if (m_texture.id) {
-        QGLShareContextScope ctx(qt_gl_share_widget()->context());
+        QGLShareContextScope ctx(qt_gl_share_context());
         glDeleteTextures(1, &m_texture.id);
         m_texture.id = 0;
     }
 }
 
-bool QGLPixmapData::fromFile(const QString &filename, const char *format,
-                             Qt::ImageConversionFlags flags)
-{
-    if (pixelType() == QPixmapData::BitmapType)
-        return QPixmapData::fromFile(filename, format, flags);
-    QFile file(filename);
-    if (!file.open(QIODevice::ReadOnly))
-        return false;
-    QByteArray data = file.peek(64);
-    bool alpha;
-    if (m_texture.canBindCompressedTexture
-            (data.constData(), data.size(), format, &alpha)) {
-        resize(0, 0);
-        data = file.readAll();
-        file.close();
-        QGLShareContextScope ctx(qt_gl_share_widget()->context());
-        QSize size = m_texture.bindCompressedTexture
-            (data.constData(), data.size(), format);
-        if (!size.isEmpty()) {
-            w = size.width();
-            h = size.height();
-            is_null = false;
-            d = 32;
-            m_hasAlpha = alpha;
-            m_source = QImage();
-            m_dirty = isValid();
-            return true;
-        }
-        return false;
-    }
-    fromImage(QImageReader(&file, format).read(), flags);
-    return !isNull();
-}
-
-bool QGLPixmapData::fromData(const uchar *buffer, uint len, const char *format,
-                             Qt::ImageConversionFlags flags)
-{
-    bool alpha;
-    const char *buf = reinterpret_cast<const char *>(buffer);
-    if (m_texture.canBindCompressedTexture(buf, int(len), format, &alpha)) {
-        resize(0, 0);
-        QGLShareContextScope ctx(qt_gl_share_widget()->context());
-        QSize size = m_texture.bindCompressedTexture(buf, int(len), format);
-        if (!size.isEmpty()) {
-            w = size.width();
-            h = size.height();
-            is_null = false;
-            d = 32;
-            m_hasAlpha = alpha;
-            m_source = QImage();
-            m_dirty = isValid();
-            return true;
-        }
-    }
-    return QPixmapData::fromData(buffer, len, format, flags);
-}
-
 bool QGLPixmapData::scroll(int dx, int dy, const QRect &rect)
 {
     Q_UNUSED(dx);
@@ -479,7 +525,7 @@ void QGLPixmapData::copy(const QPixmapData *data, const QRect &rect)
 
     const QGLPixmapData *other = static_cast<const QGLPixmapData *>(data);
     if (other->m_renderFbo) {
-        QGLShareContextScope ctx(qt_gl_share_widget()->context());
+        QGLShareContextScope ctx(qt_gl_share_context());
 
         resize(rect.width(), rect.height());
         m_hasAlpha = other->m_hasAlpha;
@@ -586,6 +632,12 @@ QImage QGLPixmapData::toImage() const
     if (m_renderFbo) {
         copyBackFromRenderFbo(true);
     } else if (!m_source.isNull()) {
+        QImageData *data = const_cast<QImage &>(m_source).data_ptr();
+        if (data->paintEngine && data->paintEngine->isActive()
+            && data->paintEngine->paintDevice() == &m_source)
+        {
+            return m_source.copy();
+        }
         return m_source;
     } else if (m_dirty || m_hasFillColor) {
         return fillImage(m_fillColor);
@@ -593,7 +645,7 @@ QImage QGLPixmapData::toImage() const
         ensureCreated();
     }
 
-    QGLShareContextScope ctx(qt_gl_share_widget()->context());
+    QGLShareContextScope ctx(qt_gl_share_context());
     glBindTexture(GL_TEXTURE_2D, m_texture.id);
     return qt_gl_read_texture(QSize(w, h), true, true);
 }
@@ -617,7 +669,7 @@ void QGLPixmapData::copyBackFromRenderFbo(bool keepCurrentFboBound) const
 
     m_hasFillColor = false;
 
-    const QGLContext *share_ctx = qt_gl_share_widget()->context();
+    const QGLContext *share_ctx = qt_gl_share_context();
     QGLShareContextScope ctx(share_ctx);
 
     ensureCreated();
@@ -672,8 +724,8 @@ QPaintEngine* QGLPixmapData::paintEngine() const
         extern QGLWidget* qt_gl_share_widget();
 
         if (!QGLContext::currentContext())
-            qt_gl_share_widget()->makeCurrent();
-        QGLShareContextScope ctx(qt_gl_share_widget()->context());
+            const_cast<QGLContext *>(qt_gl_share_context())->makeCurrent();
+        QGLShareContextScope ctx(qt_gl_share_context());
 
         QGLFramebufferObjectFormat format;
         format.setAttachment(QGLFramebufferObject::CombinedDepthStencil);
@@ -739,9 +791,6 @@ QGLTexture* QGLPixmapData::texture() const
     return &m_texture;
 }
 
-extern int qt_defaultDpiX();
-extern int qt_defaultDpiY();
-
 int QGLPixmapData::metric(QPaintDevice::PaintDeviceMetric metric) const
 {
     if (w == 0)
