@@ -1,38 +1,38 @@
 /****************************************************************************
 **
-** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
-** All rights reserved.
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the QtOpenGL module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** GNU Lesser General Public License Usage
-** This file may be used under the terms of the GNU Lesser General Public
-** License version 2.1 as published by the Free Software Foundation and
-** appearing in the file LICENSE.LGPL included in the packaging of this
-** file. Please review the following information to ensure the GNU Lesser
-** General Public License version 2.1 requirements will be met:
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
-** In addition, as a special exception, Nokia gives you certain additional
-** rights. These rights are described in the Nokia Qt LGPL Exception
+** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+**
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU General
-** Public License version 3.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of this
-** file. Please review the following information to ensure the GNU General
-** Public License version 3.0 requirements will be met:
-** http://www.gnu.org/copyleft/gpl.html.
-**
-** Other Usage
-** Alternatively, this file may be used in accordance with the terms and
-** conditions contained in a signed written agreement between you and Nokia.
-**
-**
-**
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3.0 as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU General Public License version 3.0 requirements will be
+** met: http://www.gnu.org/copyleft/gpl.html.
 **
 **
 ** $QT_END_LICENSE$
@@ -184,28 +184,29 @@ QGLGraphicsSystem::QGLGraphicsSystem(bool useX11GL)
 class QGLGlobalShareWidget
 {
 public:
-    QGLGlobalShareWidget() : widget(0), initializing(false) {
+    QGLGlobalShareWidget() : widget(0), init(false) {
         created = true;
     }
 
     QGLWidget *shareWidget() {
-        if (!initializing && !widget && !cleanedUp) {
-            initializing = true;
+        if (!init && !widget && !cleanedUp) {
+            init = true;
             widget = new QGLWidget(QGLFormat(QGL::SingleBuffer | QGL::NoDepthBuffer | QGL::NoStencilBuffer));
 #ifdef Q_OS_SYMBIAN
             if (!widget->context()->isValid()) {
                 delete widget;
                 widget = 0;
-                initializing = false;
+                init = false;
                 return 0;
             }
 #endif
+
             widget->resize(1, 1);
 
             // We don't need this internal widget to appear in QApplication::topLevelWidgets()
             if (QWidgetPrivate::allWidgets)
                 QWidgetPrivate::allWidgets->remove(widget);
-            initializing = false;
+            init = false;
         }
         return widget;
     }
@@ -232,12 +233,17 @@ public:
         cleanedUp = false;
     }
 
+    bool initializing()
+    {
+        return init;
+    }
+
     static bool cleanedUp;
     static bool created;
 
 private:
     QGLWidget *widget;
-    bool initializing;
+    bool init;
 };
 
 bool QGLGlobalShareWidget::cleanedUp = false;
@@ -268,6 +274,13 @@ void qt_destroy_gl_share_widget()
         _qt_gl_share_widget()->destroy();
 }
 
+bool qt_initializing_gl_share_widget()
+{
+    if (QGLGlobalShareWidget::created)
+        return _qt_gl_share_widget()->initializing();
+    return false;
+}
+
 const QGLContext *qt_gl_share_context()
 {
     QGLWidget *widget = qt_gl_share_widget();
@@ -308,12 +321,6 @@ struct QGLWindowSurfacePrivate
 QGLFormat QGLWindowSurface::surfaceFormat;
 QGLWindowSurface::SwapMode QGLWindowSurface::swapBehavior = QGLWindowSurface::AutomaticSwap;
 
-void QGLWindowSurfaceGLPaintDevice::endPaint()
-{
-    glFlush();
-    QGLPaintDevice::endPaint();
-}
-
 QSize QGLWindowSurfaceGLPaintDevice::size() const
 {
     return d->size;
@@ -680,6 +687,11 @@ void QGLWindowSurface::flush(QWidget *widget, const QRegion &rgn, const QPoint &
             }
 
             QGLContext *ctx = reinterpret_cast<QGLContext *>(parent->d_func()->extraData()->glContext);
+#ifdef Q_OS_SYMBIAN
+            if (!ctx)
+                return;
+#endif
+
             if (widget != window()) {
                 if (initializeOffscreenTexture(window()->size()))
                     qWarning() << "QGLWindowSurface: Flushing to native child widget, may lead to significant performance loss";
