@@ -1,17 +1,18 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
-** All rights reserved.
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the QtOpenVG module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
@@ -21,8 +22,8 @@
 ** ensure the GNU Lesser General Public License version 2.1 requirements
 ** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
@@ -33,8 +34,7 @@
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
+**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -42,6 +42,7 @@
 #include "qwindowsurface_vgegl_p.h"
 #include "qpaintengine_vg_p.h"
 #include "qpixmapdata_vg_p.h"
+#include "qvgimagepool_p.h"
 #include "qvg_p.h"
 
 #if !defined(QT_NO_EGL)
@@ -70,18 +71,13 @@ VGImageFormat qt_vg_config_to_vg_format(QEglContext *context)
 
 QImage::Format qt_vg_config_to_image_format(QEglContext *context)
 {
-    EGLint red = 0;
-    EGLint green = 0;
-    EGLint blue = 0;
-    EGLint alpha = 0;
-    context->configAttrib(EGL_RED_SIZE, &red);
-    context->configAttrib(EGL_GREEN_SIZE, &green);
-    context->configAttrib(EGL_BLUE_SIZE, &blue);
-    context->configAttrib(EGL_ALPHA_SIZE, &alpha);
+    EGLint red = context->configAttrib(EGL_RED_SIZE);
+    EGLint green = context->configAttrib(EGL_GREEN_SIZE);
+    EGLint blue = context->configAttrib(EGL_BLUE_SIZE);
+    EGLint alpha = context->configAttrib(EGL_ALPHA_SIZE);
     QImage::Format argbFormat;
 #ifdef EGL_VG_ALPHA_FORMAT_PRE_BIT
-    EGLint type = 0;
-    context->configAttrib(EGL_SURFACE_TYPE, &type);
+    EGLint type = context->configAttrib(EGL_SURFACE_TYPE);
     if ((type & EGL_VG_ALPHA_FORMAT_PRE_BIT) != 0)
         argbFormat = QImage::Format_ARGB32_Premultiplied;
     else
@@ -111,15 +107,19 @@ public:
 
     QEglContext *context;
     int refCount;
+    int widgetRefCount;
     QVGPaintEngine *engine;
     EGLSurface surface;
+    QVGPixmapData *firstPixmap;
 };
 
 QVGSharedContext::QVGSharedContext()
     : context(0)
     , refCount(0)
+    , widgetRefCount(0)
     , engine(0)
     , surface(EGL_NO_SURFACE)
+    , firstPixmap(0)
 {
 }
 
@@ -154,6 +154,29 @@ void qt_vg_destroy_paint_engine(QVGPaintEngine *engine)
     Q_UNUSED(engine);
 }
 
+void qt_vg_register_pixmap(QVGPixmapData *pd)
+{
+    QVGSharedContext *shared = sharedContext();
+    pd->next = shared->firstPixmap;
+    pd->prev = 0;
+    if (shared->firstPixmap)
+        shared->firstPixmap->prev = pd;
+    shared->firstPixmap = pd;
+}
+
+void qt_vg_unregister_pixmap(QVGPixmapData *pd)
+{
+    if (pd->next)
+        pd->next->prev = pd->prev;
+    if (pd->prev) {
+        pd->prev->next = pd->next;
+    } else {
+        QVGSharedContext *shared = sharedContext();
+        if (shared)
+           shared->firstPixmap = pd->next;
+    }
+}
+
 #else
 
 QVGPaintEngine *qt_vg_create_paint_engine(void)
@@ -166,17 +189,23 @@ void qt_vg_destroy_paint_engine(QVGPaintEngine *engine)
     delete engine;
 }
 
+void qt_vg_register_pixmap(QVGPixmapData *pd)
+{
+    Q_UNUSED(pd);
+}
+
+void qt_vg_unregister_pixmap(QVGPixmapData *pd)
+{
+    Q_UNUSED(pd);
+}
+
 #endif
 
 #ifdef EGL_VG_ALPHA_FORMAT_PRE_BIT
 
 static bool isPremultipliedContext(const QEglContext *context)
 {
-    EGLint value = 0;
-    if (context->configAttrib(EGL_SURFACE_TYPE, &value))
-        return (value & EGL_VG_ALPHA_FORMAT_PRE_BIT) != 0;
-    else
-        return false;
+    return context->configAttrib(EGL_SURFACE_TYPE) & EGL_VG_ALPHA_FORMAT_PRE_BIT;
 }
 
 #endif
@@ -188,17 +217,49 @@ static QEglContext *createContext(QPaintDevice *device)
     // Create the context object and open the display.
     context = new QEglContext();
     context->setApi(QEgl::OpenVG);
-    if (!context->openDisplay(device)) {
-        delete context;
-        return 0;
-    }
 
     // Set the swap interval for the display.
     QByteArray interval = qgetenv("QT_VG_SWAP_INTERVAL");
     if (!interval.isEmpty())
-        eglSwapInterval(context->display(), interval.toInt());
+        eglSwapInterval(QEgl::display(), interval.toInt());
     else
-        eglSwapInterval(context->display(), 1);
+        eglSwapInterval(QEgl::display(), 1);
+
+#ifdef EGL_RENDERABLE_TYPE
+    // Has the user specified an explicit EGL configuration to use?
+    QByteArray configId = qgetenv("QT_VG_EGL_CONFIG");
+    if (!configId.isEmpty()) {
+        EGLint cfgId = configId.toInt();
+        EGLint properties[] = {
+            EGL_CONFIG_ID, cfgId,
+            EGL_NONE
+        };
+        EGLint matching = 0;
+        EGLConfig cfg;
+        if (eglChooseConfig
+                    (QEgl::display(), properties, &cfg, 1, &matching) &&
+                matching > 0) {
+            // Check that the selected configuration actually supports OpenVG
+            // and then create the context with it.
+            EGLint id = 0;
+            EGLint type = 0;
+            eglGetConfigAttrib
+                (QEgl::display(), cfg, EGL_CONFIG_ID, &id);
+            eglGetConfigAttrib
+                (QEgl::display(), cfg, EGL_RENDERABLE_TYPE, &type);
+            if (cfgId == id && (type & EGL_OPENVG_BIT) != 0) {
+                context->setConfig(cfg);
+                if (!context->createContext()) {
+                    delete context;
+                    return 0;
+                }
+                return context;
+            } else {
+                qWarning("QT_VG_EGL_CONFIG: %d is not a valid OpenVG configuration", int(cfgId));
+            }
+        }
+    }
+#endif
 
     // Choose an appropriate configuration for rendering into the device.
     QEglProperties configProps;
@@ -206,24 +267,22 @@ static QEglContext *createContext(QPaintDevice *device)
     int redSize = configProps.value(EGL_RED_SIZE);
     if (redSize == EGL_DONT_CARE || redSize == 0)
         configProps.setPixelFormat(QImage::Format_ARGB32);  // XXX
-#ifndef QVG_SCISSOR_CLIP
-    // If we are using the mask to clip, then explicitly request a mask.
     configProps.setValue(EGL_ALPHA_MASK_SIZE, 1);
-#endif
 #ifdef EGL_VG_ALPHA_FORMAT_PRE_BIT
-    configProps.setValue(EGL_SURFACE_TYPE, EGL_WINDOW_BIT | EGL_PBUFFER_BIT |
-                         EGL_VG_ALPHA_FORMAT_PRE_BIT);
+    configProps.setValue(EGL_SURFACE_TYPE, EGL_WINDOW_BIT
+                         | EGL_SWAP_BEHAVIOR_PRESERVED_BIT
+                         | EGL_VG_ALPHA_FORMAT_PRE_BIT);
     configProps.setRenderableType(QEgl::OpenVG);
     if (!context->chooseConfig(configProps)) {
         // Try again without the "pre" bit.
-        configProps.setValue(EGL_SURFACE_TYPE, EGL_WINDOW_BIT | EGL_PBUFFER_BIT);
+        configProps.setValue(EGL_SURFACE_TYPE, EGL_WINDOW_BIT | EGL_SWAP_BEHAVIOR_PRESERVED_BIT);
         if (!context->chooseConfig(configProps)) {
             delete context;
             return 0;
         }
     }
 #else
-    configProps.setValue(EGL_SURFACE_TYPE, EGL_WINDOW_BIT | EGL_PBUFFER_BIT);
+    configProps.setValue(EGL_SURFACE_TYPE, EGL_WINDOW_BIT | EGL_SWAP_BEHAVIOR_PRESERVED_BIT);
     configProps.setRenderableType(QEgl::OpenVG);
     if (!context->chooseConfig(configProps)) {
         delete context;
@@ -242,9 +301,11 @@ static QEglContext *createContext(QPaintDevice *device)
 
 #if !defined(QVG_NO_SINGLE_CONTEXT)
 
-QEglContext *qt_vg_create_context(QPaintDevice *device)
+QEglContext *qt_vg_create_context(QPaintDevice *device, int devType)
 {
     QVGSharedContext *shared = sharedContext();
+    if (devType == QInternal::Widget)
+        ++(shared->widgetRefCount);
     if (shared->context) {
         ++(shared->refCount);
         return shared->context;
@@ -255,23 +316,68 @@ QEglContext *qt_vg_create_context(QPaintDevice *device)
     }
 }
 
-void qt_vg_destroy_context(QEglContext *context)
+static void qt_vg_destroy_shared_context(QVGSharedContext *shared)
+{
+    shared->context->makeCurrent(qt_vg_shared_surface());
+    delete shared->engine;
+    shared->engine = 0;
+    shared->context->doneCurrent();
+    if (shared->surface != EGL_NO_SURFACE) {
+        eglDestroySurface(QEgl::display(), shared->surface);
+        shared->surface = EGL_NO_SURFACE;
+    }
+    delete shared->context;
+    shared->context = 0;
+}
+
+void qt_vg_hibernate_pixmaps(QVGSharedContext *shared)
+{
+    // Artificially increase the reference count to prevent the
+    // context from being destroyed until after we have finished
+    // the hibernation process.
+    ++(shared->refCount);
+
+    // We need a context current to hibernate the VGImage objects.
+    shared->context->makeCurrent(qt_vg_shared_surface());
+
+    // Scan all QVGPixmapData objects in the system and hibernate them.
+    QVGPixmapData *pd = shared->firstPixmap;
+    while (pd != 0) {
+        pd->hibernate();
+        pd = pd->next;
+    }
+
+    // Hibernate any remaining VGImage's in the image pool.
+    QVGImagePool::instance()->hibernate();
+
+    // Don't need the current context any more.
+    shared->context->lazyDoneCurrent();
+
+    // Decrease the reference count and destroy the context if necessary.
+    if (--(shared->refCount) <= 0)
+        qt_vg_destroy_shared_context(shared);
+}
+
+void qt_vg_destroy_context(QEglContext *context, int devType)
 {
     QVGSharedContext *shared = sharedContext();
     if (shared->context != context) {
         // This is not the shared context.  Shouldn't happen!
         delete context;
-    } else if (--(shared->refCount) <= 0) {
-        shared->context->makeCurrent(qt_vg_shared_surface());
-        delete shared->engine;
-        shared->engine = 0;
-        shared->context->doneCurrent();
-        if (shared->surface != EGL_NO_SURFACE) {
-            eglDestroySurface(shared->context->display(), shared->surface);
-            shared->surface = EGL_NO_SURFACE;
-        }
-        delete shared->context;
-        shared->context = 0;
+        return;
+    }
+    if (devType == QInternal::Widget)
+        --(shared->widgetRefCount);
+    if (--(shared->refCount) <= 0) {
+        qt_vg_destroy_shared_context(shared);
+    } else if (shared->widgetRefCount <= 0 && devType == QInternal::Widget) {
+        // All of the widget window surfaces have been destroyed
+        // but we still have VG pixmaps active.  Ask them to hibernate
+        // to free up GPU resources until a widget is shown again.
+        // This may eventually cause the EGLContext to be destroyed
+        // because nothing in the system needs a context, which will
+        // free up even more GPU resources.
+        qt_vg_hibernate_pixmaps(shared);
     }
 }
 
@@ -295,20 +401,22 @@ EGLSurface qt_vg_shared_surface(void)
             attribs[4] = EGL_NONE;
         }
         shared->surface = eglCreatePbufferSurface
-            (shared->context->display(), shared->context->config(), attribs);
+            (QEgl::display(), shared->context->config(), attribs);
     }
     return shared->surface;
 }
 
 #else
 
-QEglContext *qt_vg_create_context(QPaintDevice *device)
+QEglContext *qt_vg_create_context(QPaintDevice *device, int devType)
 {
+    Q_UNUSED(devType);
     return createContext(device);
 }
 
-void qt_vg_destroy_context(QEglContext *context)
+void qt_vg_destroy_context(QEglContext *context, int devType)
 {
+    Q_UNUSED(devType);
     delete context;
 }
 
@@ -398,7 +506,7 @@ QVGEGLWindowSurfaceVGImage::~QVGEGLWindowSurfaceVGImage()
         }
         if (windowSurface != EGL_NO_SURFACE)
             context->destroySurface(windowSurface);
-        qt_vg_destroy_context(context);
+        qt_vg_destroy_context(context, QInternal::Widget);
     }
 }
 
@@ -417,7 +525,7 @@ QEglContext *QVGEGLWindowSurfaceVGImage::ensureContext(QWidget *widget)
     if (!context) {
         // Create a new EGL context.  We create the surface in beginPaint().
         size = newSize;
-        context = qt_vg_create_context(widget);
+        context = qt_vg_create_context(widget, QInternal::Widget);
         if (!context)
             return 0;
         isPaintingActive = false;
@@ -436,7 +544,7 @@ void QVGEGLWindowSurfaceVGImage::beginPaint(QWidget *widget)
             context->makeCurrent(mainSurface());
             recreateBackBuffer = false;
             if (backBufferSurface != EGL_NO_SURFACE) {
-                eglDestroySurface(context->display(), backBufferSurface);
+                eglDestroySurface(QEgl::display(), backBufferSurface);
                 backBufferSurface = EGL_NO_SURFACE;
             }
             if (backBuffer != VG_INVALID_HANDLE) {
@@ -449,7 +557,7 @@ void QVGEGLWindowSurfaceVGImage::beginPaint(QWidget *widget)
             if (backBuffer != VG_INVALID_HANDLE) {
                 // Create an EGL surface for rendering into the VGImage.
                 backBufferSurface = eglCreatePbufferFromClientBuffer
-                    (context->display(), EGL_OPENVG_IMAGE,
+                    (QEgl::display(), EGL_OPENVG_IMAGE,
                      (EGLClientBuffer)(backBuffer),
                      context->config(), NULL);
                 if (backBufferSurface == EGL_NO_SURFACE) {
@@ -512,7 +620,7 @@ QVGEGLWindowSurfaceDirect::~QVGEGLWindowSurfaceDirect()
     if (context) {
         if (windowSurface != EGL_NO_SURFACE)
             context->destroySurface(windowSurface);
-        qt_vg_destroy_context(context);
+        qt_vg_destroy_context(context, QInternal::Widget);
     }
 }
 
@@ -540,6 +648,7 @@ QEglContext *QVGEGLWindowSurfaceDirect::ensureContext(QWidget *widget)
 #endif
         windowSurface = context->createSurface(widget, &surfaceProps);
         isPaintingActive = false;
+        needToSwap = true;
     }
 #else
     if (context && size != newSize) {
@@ -551,7 +660,7 @@ QEglContext *QVGEGLWindowSurfaceDirect::ensureContext(QWidget *widget)
         qt_vg_destroy_paint_engine(engine);
         engine = 0;
         context->destroySurface(windowSurface);
-        qt_vg_destroy_context(context);
+        qt_vg_destroy_context(context, QInternal::Widget);
         context = 0;
         windowSurface = EGL_NO_SURFACE;
     }
@@ -560,7 +669,7 @@ QEglContext *QVGEGLWindowSurfaceDirect::ensureContext(QWidget *widget)
     if (!context) {
         // Create a new EGL context and bind it to the widget surface.
         size = newSize;
-        context = qt_vg_create_context(widget);
+        context = qt_vg_create_context(widget, QInternal::Widget);
         if (!context)
             return 0;
         // We want a direct to window rendering surface if possible.
@@ -577,7 +686,9 @@ QEglContext *QVGEGLWindowSurfaceDirect::ensureContext(QWidget *widget)
 #endif
         EGLSurface surface = context->createSurface(widget, &surfaceProps);
         if (surface == EGL_NO_SURFACE) {
-            qt_vg_destroy_context(context);
+            qt_vg_destroy_paint_engine(engine);
+            engine = 0;
+            qt_vg_destroy_context(context, QInternal::Widget);
             context = 0;
             return 0;
         }
@@ -585,26 +696,26 @@ QEglContext *QVGEGLWindowSurfaceDirect::ensureContext(QWidget *widget)
 #if defined(QVG_DIRECT_TO_WINDOW)
         // Did we get a direct to window rendering surface?
         EGLint buffer = 0;
-        if (eglQueryContext(context->display(), context->context(),
+        if (eglQueryContext(QEgl::display(), context->context(),
                             EGL_RENDER_BUFFER, &buffer) &&
                 buffer == EGL_SINGLE_BUFFER) {
             needToSwap = false;
         }
-#endif
-#if !defined(QVG_NO_PRESERVED_SWAP)
-        // Try to force the surface back buffer to preserve its contents.
-        if (needToSwap) {
-            eglGetError();  // Clear error state first.
-            eglSurfaceAttrib(context->display(), surface,
-                             EGL_SWAP_BEHAVIOR, EGL_BUFFER_PRESERVED);
-            if (eglGetError() != EGL_SUCCESS) {
-                qWarning("QVG: could not enable preserved swap");
-            }
-        }
 #endif
         windowSurface = surface;
         isPaintingActive = false;
     }
+
+#if !defined(QVG_NO_PRESERVED_SWAP)
+    // Try to force the surface back buffer to preserve its contents.
+    if (needToSwap) {
+        bool succeeded = eglSurfaceAttrib(QEgl::display(), windowSurface,
+                EGL_SWAP_BEHAVIOR, EGL_BUFFER_PRESERVED);
+        if (!succeeded && eglGetError() != EGL_SUCCESS) {
+            qWarning("QVG: could not enable preserved swap");
+        }
+    }
+#endif
     return context;
 }
 
@@ -637,6 +748,38 @@ void QVGEGLWindowSurfaceDirect::endPaint
     }
 }
 
+bool QVGEGLWindowSurfaceDirect::supportsStaticContents() const
+{
+#if defined(QVG_BUFFER_SCROLLING) && !defined(QVG_NO_PRESERVED_SWAP)
+    return true;
+#else
+    return QVGEGLWindowSurfacePrivate::supportsStaticContents();
+#endif
+}
+
+bool QVGEGLWindowSurfaceDirect::scroll(QWidget *widget, const QRegion& area, int dx, int dy)
+{
+#ifdef QVG_BUFFER_SCROLLING
+    QEglContext *context = ensureContext(widget);
+    if (context) {
+        context->makeCurrent(windowSurface);
+        QRect scrollRect = area.boundingRect();
+        int sx = scrollRect.x();
+        int sy = size.height() - scrollRect.y() - scrollRect.height();
+        vgSeti(VG_SCISSORING, VG_FALSE);
+        vgCopyPixels(sx + dx, sy - dy, sx, sy, scrollRect.width(), scrollRect.height());
+        context->lazyDoneCurrent();
+        return true;
+    }
+#else
+    Q_UNUSED(widget);
+    Q_UNUSED(area);
+    Q_UNUSED(dx);
+    Q_UNUSED(dy);
+#endif
+    return false;
+}
+
 QT_END_NAMESPACE
 
 #endif
