@@ -1,17 +1,18 @@
 /****************************************************************************
 **
-** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
-** All rights reserved.
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the plugins of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
@@ -21,8 +22,8 @@
 ** ensure the GNU Lesser General Public License version 2.1 requirements
 ** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
@@ -33,8 +34,7 @@
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
+**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -84,8 +84,8 @@ SymbianNetworkConfigurationPrivate::~SymbianNetworkConfigurationPrivate()
 }
 
 SymbianEngine::SymbianEngine(QObject *parent)
-:   QBearerEngine(parent), CActive(CActive::EPriorityHigh), iFirstUpdate(true), iInitOk(true),
-    iUpdatePending(false)
+:   QBearerEngine(parent), CActive(CActive::EPriorityHigh), iFirstUpdate(true), ipCommsDB(0),
+    iInitOk(true), iUpdatePending(false), ipAccessPointsAvailabilityScanner(0)
 {
 }
 
@@ -104,11 +104,11 @@ void SymbianEngine::initialize()
         return;
     }
 
-    TRAP_IGNORE(iConnectionMonitor.ConnectL());
-#ifdef SNAP_FUNCTIONALITY_AVAILABLE
-    TRAP_IGNORE(iConnectionMonitor.SetUintAttribute(EBearerIdAll, 0, KBearerGroupThreshold, 1));
-#endif
-    TRAP_IGNORE(iConnectionMonitor.NotifyEventL(*this));
+    TRAP(error, StartConnectionMonitorNotifyL());
+    if (error != KErrNone) {
+        iInitOk = false;
+        return;
+    }
 
 #ifdef SNAP_FUNCTIONALITY_AVAILABLE
     TRAP(error, iCmManager.OpenL());
@@ -135,30 +135,41 @@ void SymbianEngine::initialize()
 
     updateConfigurations();
     updateStatesToSnaps();
-    updateAvailableAccessPoints(); // On first time updates synchronously (without WLAN scans)
+    updateAvailableAccessPoints(); // On first time updates (without WLAN scans)
     // Start monitoring IAP and/or SNAP changes in Symbian CommsDB
     startCommsDatabaseNotifications();
-    iFirstUpdate = false;
+}
+
+void SymbianEngine::StartConnectionMonitorNotifyL()
+{
+    iConnectionMonitor.ConnectL();
+    CleanupClosePushL(iConnectionMonitor);
+#ifdef SNAP_FUNCTIONALITY_AVAILABLE
+    User::LeaveIfError(iConnectionMonitor.SetUintAttribute(EBearerIdAll, 0, KBearerGroupThreshold, 1));
+#endif
+    iConnectionMonitor.NotifyEventL(*this);
+    CleanupStack::Pop();
 }
 
 SymbianEngine::~SymbianEngine()
 {
     Cancel();
 
-    iConnectionMonitor.CancelNotifications();
-    iConnectionMonitor.Close();
-    
-#ifdef SNAP_FUNCTIONALITY_AVAILABLE    
-    iCmManager.Close();
-#endif
-    
+    //The scanner may be using the connection monitor so it needs to be
+    //deleted first while the handle is still valid.
     delete ipAccessPointsAvailabilityScanner;
 
-    // CCommsDatabase destructor uses cleanup stack. Since QNetworkConfigurationManager
+    iConnectionMonitor.CancelNotifications();
+    iConnectionMonitor.Close();
+
+    // CCommsDatabase destructor and RCmManager.Close() use cleanup stack. Since QNetworkConfigurationManager
     // is a global static, but the time we are here, E32Main() has been exited already and
     // the thread's default cleanup stack has been deleted. Without this line, a
     // 'E32USER-CBase 69' -panic will occur.
     CTrapCleanup* cleanup = CTrapCleanup::New();
+#ifdef SNAP_FUNCTIONALITY_AVAILABLE
+    iCmManager.Close();
+#endif
     delete ipCommsDB;
     delete cleanup;
 }
@@ -247,7 +258,7 @@ void SymbianEngine::updateConfigurationsL()
 #ifdef SNAP_FUNCTIONALITY_AVAILABLE
     // S60 version is >= Series60 3rd Edition Feature Pack 2
     TInt error = KErrNone;
-    
+
     // Loop through all IAPs
     RArray<TUint32> connectionMethods; // IAPs
     CleanupClosePushL(connectionMethods);
@@ -281,7 +292,7 @@ void SymbianEngine::updateConfigurationsL()
         CleanupStack::PopAndDestroy(&connectionMethod);
     }
     CleanupStack::PopAndDestroy(&connectionMethods);
-    
+
     // Loop through all SNAPs
     RArray<TUint32> destinations;
     CleanupClosePushL(destinations);
@@ -318,7 +329,27 @@ void SymbianEngine::updateConfigurationsL()
             cpPriv->connectionId = 0;
             cpPriv->state = QNetworkConfiguration::Defined;
             cpPriv->type = QNetworkConfiguration::ServiceNetwork;
-            cpPriv->purpose = QNetworkConfiguration::UnknownPurpose;
+
+            // determine purpose of this SNAP
+            TUint32 purpose = CMManager::ESnapPurposeUnknown;
+            TRAP_IGNORE(purpose = destination.MetadataL(CMManager::ESnapMetadataPurpose));
+            switch (purpose) {
+            case CMManager::ESnapPurposeInternet:
+                cpPriv->purpose = QNetworkConfiguration::PublicPurpose;
+                break;
+            case CMManager::ESnapPurposeIntranet:
+                cpPriv->purpose = QNetworkConfiguration::PrivatePurpose;
+                break;
+            case CMManager::ESnapPurposeMMS:
+            case CMManager::ESnapPurposeOperator:
+                cpPriv->purpose = QNetworkConfiguration::ServiceSpecificPurpose;
+                break;
+            case CMManager::ESnapPurposeUnknown:
+            default:
+                cpPriv->purpose = QNetworkConfiguration::UnknownPurpose;
+                break;
+            }
+
             cpPriv->roamingSupported = false;
 
             QNetworkConfigurationPrivatePointer ptr(cpPriv);
@@ -451,7 +482,7 @@ void SymbianEngine::updateConfigurationsL()
                     break;
                 }
             }
-        }    
+        }
     }
 
     foreach (const QString &oldIface, knownSnapConfigs) {
@@ -471,9 +502,39 @@ void SymbianEngine::updateConfigurationsL()
 
 #ifdef SNAP_FUNCTIONALITY_AVAILABLE
     updateStatesToSnaps();
+    updatePurposeToIaps();
 #endif
 }
 
+//copy purpose from SNAP to child IAPs, unless child is contained in more than one SNAP with conflicting purposes.
+void SymbianEngine::updatePurposeToIaps()
+{
+    QMutexLocker lock(&mutex);
+    QHash<QString,int> purposes;
+    foreach (QNetworkConfigurationPrivatePointer snap, snapConfigurations.values()) {
+        QMutexLocker snaplock(&snap->mutex);
+        foreach (QNetworkConfigurationPrivatePointer iap, snap->serviceNetworkMembers.values()) {
+            QMutexLocker iaplock(&iap->mutex);
+            QString id = iap->id;
+            if (purposes.contains(id) && purposes.value(id) != snap->purpose)
+                purposes[id] = -1; //conflict detected
+            else
+                purposes[id] = snap->purpose;
+        }
+    }
+
+    for (QHash<QString,int>::const_iterator it = purposes.constBegin(); it != purposes.constEnd(); ++it) {
+        if (accessPointConfigurations.contains(it.key())) {
+            QNetworkConfigurationPrivatePointer iap = accessPointConfigurations.value(it.key());
+            QMutexLocker iaplock(&iap->mutex);
+            int purpose = it.value();
+            if (purpose == -1) //resolve conflicts as unknown
+                purpose = QNetworkConfiguration::UnknownPurpose;
+            iap->purpose = (QNetworkConfiguration::Purpose)purpose;
+        }
+    }
+}
+
 #ifdef SNAP_FUNCTIONALITY_AVAILABLE
 SymbianNetworkConfigurationPrivate *SymbianEngine::configFromConnectionMethodL(
         RCmConnectionMethod& connectionMethod)
@@ -481,21 +542,46 @@ SymbianNetworkConfigurationPrivate *SymbianEngine::configFromConnectionMethodL(
     SymbianNetworkConfigurationPrivate *cpPriv = new SymbianNetworkConfigurationPrivate;
     TUint32 iapId = connectionMethod.GetIntAttributeL(CMManager::ECmIapId);
     QString ident = QT_BEARERMGMT_CONFIGURATION_IAP_PREFIX+QString::number(qHash(iapId));
-    
+
     HBufC *pName = connectionMethod.GetStringAttributeL(CMManager::ECmName);
     CleanupStack::PushL(pName);
     QT_TRYCATCH_LEAVING(cpPriv->name = QString::fromUtf16(pName->Ptr(),pName->Length()));
     CleanupStack::PopAndDestroy(pName);
     pName = NULL;
-    
+
     TUint32 bearerId = connectionMethod.GetIntAttributeL(CMManager::ECmCommsDBBearerType);
     switch (bearerId) {
     case KCommDbBearerCSD:
         cpPriv->bearerType = QNetworkConfiguration::Bearer2G;
         break;
     case KCommDbBearerWcdma:
-        cpPriv->bearerType = QNetworkConfiguration::BearerWCDMA;
-        break;
+        {
+            //This is ambiguous, check the network status to find out what the expected connection will be
+            TUint mode;
+            TRequestStatus status;
+            iConnectionMonitor.GetUintAttribute(0, 0, KMobilePhoneNetworkMode, mode, status);
+            User::WaitForRequest(status);
+            if (status != KErrNone)
+                cpPriv->bearerType = QNetworkConfiguration::BearerUnknown;
+            else switch (mode) {
+            case EConnMonNetworkModeCdma2000:
+                cpPriv->bearerType = QNetworkConfiguration::BearerCDMA2000;
+                break;
+            case EConnMonNetworkModeWcdma:
+            case EConnMonNetworkModeTdcdma:
+                cpPriv->bearerType = QNetworkConfiguration::BearerWCDMA; //includes HSDPA, as this API can't detect it
+                break;
+            case EConnMonNetworkModeGsm:
+            case EConnMonNetworkModeAmps:
+            case EConnMonNetworkModeCdma95:
+                cpPriv->bearerType = QNetworkConfiguration::Bearer2G; //includes GPRS and EDGE, Qt API treats them both as 2G
+                break;
+            default:
+                cpPriv->bearerType = QNetworkConfiguration::BearerUnknown;
+                break;
+            }
+            break;
+        }
     case KCommDbBearerLAN:
         cpPriv->bearerType = QNetworkConfiguration::BearerEthernet;
         break;
@@ -512,7 +598,7 @@ SymbianNetworkConfigurationPrivate *SymbianEngine::configFromConnectionMethodL(
         cpPriv->bearerType = QNetworkConfiguration::BearerUnknown;
         break;
     }
-    
+
     TInt error = KErrNone;
     TUint32 bearerType = connectionMethod.GetIntAttributeL(CMManager::ECmBearerType);
     switch (bearerType) {
@@ -536,13 +622,13 @@ SymbianNetworkConfigurationPrivate *SymbianEngine::configFromConnectionMethodL(
         CleanupStack::PopAndDestroy(pName);
         pName = NULL;
     }
- 
+
     cpPriv->state = QNetworkConfiguration::Defined;
     TBool isConnected = connectionMethod.GetBoolAttributeL(CMManager::ECmConnected);
     if (isConnected) {
         cpPriv->state = QNetworkConfiguration::Active;
     }
-    
+
     cpPriv->isValid = true;
     cpPriv->id = ident;
     cpPriv->numericId = iapId;
@@ -558,7 +644,7 @@ bool SymbianEngine::readNetworkConfigurationValuesFromCommsDb(
 {
     TRAPD(error, readNetworkConfigurationValuesFromCommsDbL(aApId,apNetworkConfiguration));
     if (error != KErrNone) {
-        return false;        
+        return false;
     }
     return true;
 }
@@ -566,22 +652,22 @@ bool SymbianEngine::readNetworkConfigurationValuesFromCommsDb(
 void SymbianEngine::readNetworkConfigurationValuesFromCommsDbL(
         TUint32 aApId, SymbianNetworkConfigurationPrivate *apNetworkConfiguration)
 {
-    CApDataHandler* pDataHandler = CApDataHandler::NewLC(*ipCommsDB); 
-    CApAccessPointItem* pAPItem = CApAccessPointItem::NewLC(); 
+    CApDataHandler* pDataHandler = CApDataHandler::NewLC(*ipCommsDB);
+    CApAccessPointItem* pAPItem = CApAccessPointItem::NewLC();
     TBuf<KCommsDbSvrMaxColumnNameLength> name;
-    
+
     CApUtils* pApUtils = CApUtils::NewLC(*ipCommsDB);
     TUint32 apId = pApUtils->WapIdFromIapIdL(aApId);
-    
+
     pDataHandler->AccessPointDataL(apId,*pAPItem);
     pAPItem->ReadTextL(EApIapName, name);
     if (name.Compare(_L("Easy WLAN")) == 0) {
         // "Easy WLAN" won't be accepted to the Configurations list
         User::Leave(KErrNotFound);
     }
-    
+
     QString ident = QT_BEARERMGMT_CONFIGURATION_IAP_PREFIX+QString::number(qHash(aApId));
-    
+
     QT_TRYCATCH_LEAVING(apNetworkConfiguration->name = QString::fromUtf16(name.Ptr(),name.Length()));
     apNetworkConfiguration->isValid = true;
     apNetworkConfiguration->id = ident;
@@ -592,7 +678,7 @@ void SymbianEngine::readNetworkConfigurationValuesFromCommsDbL(
     apNetworkConfiguration->purpose = QNetworkConfiguration::UnknownPurpose;
     apNetworkConfiguration->roamingSupported = false;
     switch (pAPItem->BearerTypeL()) {
-    case EApBearerTypeCSD:      
+    case EApBearerTypeCSD:
         apNetworkConfiguration->bearerType = QNetworkConfiguration::Bearer2G;
         break;
     case EApBearerTypeGPRS:
@@ -617,7 +703,7 @@ void SymbianEngine::readNetworkConfigurationValuesFromCommsDbL(
         apNetworkConfiguration->bearerType = QNetworkConfiguration::BearerUnknown;
         break;
     }
-    
+
     CleanupStack::PopAndDestroy(pApUtils);
     CleanupStack::PopAndDestroy(pAPItem);
     CleanupStack::PopAndDestroy(pDataHandler);
@@ -656,7 +742,7 @@ QNetworkConfigurationPrivatePointer SymbianEngine::defaultConfigurationL()
         ptr = accessPointConfigurations.value(iface);
     }
 #endif
-    
+
     if (ptr) {
         QMutexLocker configLocker(&ptr->mutex);
         if (ptr->isValid)
@@ -676,7 +762,7 @@ void SymbianEngine::updateActiveAccessPoints()
     TUint connectionCount;
     iConnectionMonitor.GetConnectionCount(connectionCount, status);
     User::WaitForRequest(status);
-    
+
     // Go through all connections and set state of related IAPs to Active.
     // Status needs to be checked carefully, because ConnMon lists also e.g.
     // WLAN connections that are being currently tried --> we don't want to
@@ -755,7 +841,7 @@ void SymbianEngine::accessPointScanningReady(TBool scanSuccessful, TConnMonIapIn
     iUpdateGoingOn = false;
     if (scanSuccessful) {
         QList<QString> unavailableConfigs = accessPointConfigurations.keys();
-        
+
         // Set state of returned IAPs to Discovered
         // if state is not already Active
         for(TUint i=0; i<iapInfo.iCount; i++) {
@@ -772,7 +858,7 @@ void SymbianEngine::accessPointScanningReady(TBool scanSuccessful, TConnMonIapIn
                 }
             }
         }
-        
+
         // Make sure that state of rest of the IAPs won't be Active
         foreach (const QString &iface, unavailableConfigs) {
             QNetworkConfigurationPrivatePointer ptr = accessPointConfigurations.value(iface);
@@ -784,12 +870,18 @@ void SymbianEngine::accessPointScanningReady(TBool scanSuccessful, TConnMonIapIn
     }
 
     updateStatesToSnaps();
-    
+
     if (!iFirstUpdate) {
         startCommsDatabaseNotifications();
         mutex.unlock();
         emit updateCompleted();
         mutex.lock();
+    } else {
+        iFirstUpdate = false;
+        if (iScanInQueue) {
+            iScanInQueue = EFalse;
+            updateAvailableAccessPoints();
+        }
     }
 }
 
@@ -822,6 +914,7 @@ void SymbianEngine::updateStatesToSnaps()
                 discovered = true;
             }
         }
+        snapConfigLocker.unlock();
         if (active) {
             changeConfigurationStateTo(ptr, QNetworkConfiguration::Active);
         } else if (discovered) {
@@ -829,7 +922,7 @@ void SymbianEngine::updateStatesToSnaps()
         } else {
             changeConfigurationStateTo(ptr, QNetworkConfiguration::Defined);
         }
-    }    
+    }
 }
 
 #ifdef SNAP_FUNCTIONALITY_AVAILABLE
@@ -976,7 +1069,7 @@ void SymbianEngine::RunL()
     QMutexLocker locker(&mutex);
 
     if (iStatus != KErrCancel) {
-        // By default, start relistening notifications. Stop only if interesting event occured.
+        // By default, start relistening notifications. Stop only if interesting event occurred.
         iWaitingCommsDatabaseNotifications = true;
         RDbNotifier::TEvent event = STATIC_CAST(RDbNotifier::TEvent, iStatus.Int());
         switch (event) {
@@ -1047,7 +1140,7 @@ void SymbianEngine::EventL(const CConnMonEventBase& aEvent)
         if (connectionStatus == KConfigDaemonStartingRegistration) {
             TUint connectionId = realEvent->ConnectionId();
             TUint subConnectionCount = 0;
-            TUint apId;            
+            TUint apId;
             TRequestStatus status;
             iConnectionMonitor.GetUintAttribute(connectionId, subConnectionCount, KIAPId, apId, status);
             User::WaitForRequest(status);
@@ -1073,7 +1166,7 @@ void SymbianEngine::EventL(const CConnMonEventBase& aEvent)
             // Connection has been successfully opened
             TUint connectionId = realEvent->ConnectionId();
             TUint subConnectionCount = 0;
-            TUint apId;            
+            TUint apId;
             TRequestStatus status;
             iConnectionMonitor.GetUintAttribute(connectionId, subConnectionCount, KIAPId, apId, status);
             User::WaitForRequest(status);
@@ -1104,6 +1197,13 @@ void SymbianEngine::EventL(const CConnMonEventBase& aEvent)
                     }
                 );
             }
+
+            //update bearer type for 2G/3G connections
+            TInt bearer;
+            iConnectionMonitor.GetIntAttribute(connectionId, 0, KBearerInfo, bearer, status);
+            User::WaitForRequest(status);
+            if (status == KErrNone)
+                updateMobileBearerToConfigs(TConnMonBearerInfo(bearer));
         } else if (connectionStatus == KConfigDaemonStartingDeregistration) {
             TUint connectionId = realEvent->ConnectionId();
             QNetworkConfigurationPrivatePointer ptr = dataByConnectionId(connectionId);
@@ -1129,7 +1229,7 @@ void SymbianEngine::EventL(const CConnMonEventBase& aEvent)
                                                    connectionId, QNetworkSession::Disconnected);
                 );
             }
-            
+
             bool online = false;
             foreach (const QString &iface, accessPointConfigurations.keys()) {
                 QNetworkConfigurationPrivatePointer ptr = accessPointConfigurations.value(iface);
@@ -1145,7 +1245,7 @@ void SymbianEngine::EventL(const CConnMonEventBase& aEvent)
             }
         }
         }
-        break;    
+        break;
 
     case EConnMonIapAvailabilityChange:
         {
@@ -1159,7 +1259,7 @@ void SymbianEngine::EventL(const CConnMonEventBase& aEvent)
 
             QNetworkConfigurationPrivatePointer ptr = accessPointConfigurations.value(ident);
             if (ptr) {
-                // Configuration is either Discovered or Active 
+                // Configuration is either Discovered or Active
                 QT_TRYCATCH_LEAVING(changeConfigurationStateAtMinTo(ptr, QNetworkConfiguration::Discovered));
                 unDiscoveredConfigs.removeOne(ident);
             }
@@ -1345,7 +1445,7 @@ AccessPointsAvailabilityScanner::AccessPointsAvailabilityScanner(SymbianEngine&
                                                                RConnectionMonitor& connectionMonitor)
     : CActive(CActive::EPriorityHigh), iOwner(owner), iConnectionMonitor(connectionMonitor)
 {
-    CActiveScheduler::Add(this);  
+    CActiveScheduler::Add(this);
 }
 
 AccessPointsAvailabilityScanner::~AccessPointsAvailabilityScanner()
@@ -1356,24 +1456,31 @@ AccessPointsAvailabilityScanner::~AccessPointsAvailabilityScanner()
 void AccessPointsAvailabilityScanner::DoCancel()
 {
     iConnectionMonitor.CancelAsyncRequest(EConnMonGetPckgAttribute);
+    iScanActive = EFalse;
+    iOwner.iScanInQueue = EFalse;
 }
 
 void AccessPointsAvailabilityScanner::StartScanning()
 {
-    if (iOwner.iFirstUpdate) {
-        // On first update (the mgr is being instantiated) update only those bearers who
-        // don't need time-consuming scans (WLAN).
-        // Note: EBearerIdWCDMA covers also GPRS bearer
-        iConnectionMonitor.GetPckgAttribute(EBearerIdWCDMA, 0, KIapAvailability, iIapBuf, iStatus);
-        User::WaitForRequest(iStatus);
-        if (iStatus.Int() == KErrNone) {
-            iOwner.accessPointScanningReady(true,iIapBuf());
+    if (!iScanActive) {
+        iScanActive = ETrue;
+        if (iOwner.iFirstUpdate) {
+            // On first update (the mgr is being instantiated) update only those bearers who
+            // don't need time-consuming scans (WLAN).
+            // Note: EBearerIdWCDMA covers also GPRS bearer
+            iConnectionMonitor.GetPckgAttribute(EBearerIdWCDMA, 0, KIapAvailability, iIapBuf, iStatus);
+        } else {
+            iConnectionMonitor.GetPckgAttribute(EBearerIdAll, 0, KIapAvailability, iIapBuf, iStatus);
         }
-    } else {
-        iConnectionMonitor.GetPckgAttribute(EBearerIdAll, 0, KIapAvailability, iIapBuf, iStatus);
+
         if (!IsActive()) {
             SetActive();
         }
+    } else {
+        // Queue scan for getting WLAN info after first request returns
+        if (iOwner.iFirstUpdate) {
+            iOwner.iScanInQueue = ETrue;
+        }
     }
 }
 
@@ -1381,6 +1488,7 @@ void AccessPointsAvailabilityScanner::RunL()
 {
     QMutexLocker locker(&iOwner.mutex);
 
+    iScanActive = EFalse;
     if (iStatus.Int() != KErrNone) {
         iIapBuf().iCount = 0;
         QT_TRYCATCH_LEAVING(iOwner.accessPointScanningReady(false,iIapBuf()));
