@@ -1,17 +1,18 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
-** All rights reserved.
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the QtSql module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
@@ -21,8 +22,8 @@
 ** ensure the GNU Lesser General Public License version 2.1 requirements
 ** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
@@ -33,8 +34,7 @@
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
+**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -1459,27 +1459,22 @@ bool QIBaseDriver::open(const QString & db,
     pass.truncate(255);
 
     QByteArray ba;
-    ba.resize(usr.length() + pass.length() + enc.length() + role.length() + 6);
-    int i = -1;
-    ba[++i] = isc_dpb_version1;
-    ba[++i] = isc_dpb_user_name;
-    ba[++i] = usr.length();
-    memcpy(ba.data() + ++i, usr.data(), usr.length());
-    i += usr.length();
-    ba[i] = isc_dpb_password;
-    ba[++i] = pass.length();
-    memcpy(ba.data() + ++i, pass.data(), pass.length());
-    i += pass.length();
-    ba[i] = isc_dpb_lc_ctype;
-    ba[++i] = enc.length();
-    memcpy(ba.data() + ++i, enc.data(), enc.length());
-    i += enc.length();
+    ba.reserve(usr.length() + pass.length() + enc.length() + role.length() + 9);
+    ba.append(char(isc_dpb_version1));
+    ba.append(char(isc_dpb_user_name));
+    ba.append(char(usr.length()));
+    ba.append(usr.data(), usr.length());
+    ba.append(char(isc_dpb_password));
+    ba.append(char(pass.length()));
+    ba.append(pass.data(), pass.length());
+    ba.append(char(isc_dpb_lc_ctype));
+    ba.append(char(enc.length()));
+    ba.append(enc.data(), enc.length());
 
     if (!role.isEmpty()) {
-        ba[i] = isc_dpb_sql_role_name;
-        ba[++i] = role.length();
-        memcpy(ba.data() + ++i, role.data(), role.length());
-        i += role.length();
+        ba.append(char(isc_dpb_sql_role_name));
+        ba.append(char(role.length()));
+        ba.append(role.data(), role.length());
     }
 
     QString ldb;
@@ -1487,7 +1482,7 @@ bool QIBaseDriver::open(const QString & db,
         ldb += host + QLatin1Char(':');
     ldb += db;
     isc_attach_database(d->status, 0, const_cast<char *>(ldb.toLocal8Bit().constData()),
-                        &d->ibase, i, ba.data());
+                        &d->ibase, ba.size(), ba.data());
     if (d->isError(QT_TRANSLATE_NOOP("QIBaseDriver", "Error opening database"),
                    QSqlError::ConnectionError)) {
         setOpenError(true);
