@@ -1,17 +1,18 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
-** All rights reserved.
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the QtSql module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
@@ -21,8 +22,8 @@
 ** ensure the GNU Lesser General Public License version 2.1 requirements
 ** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
@@ -33,8 +34,7 @@
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
+**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -320,6 +320,16 @@ static bool qIsBlob(int t)
            || t == MYSQL_TYPE_LONG_BLOB;
 }
 
+static bool qIsInteger(int t)
+{
+    return t == MYSQL_TYPE_TINY
+           || t == MYSQL_TYPE_SHORT
+           || t == MYSQL_TYPE_LONG
+           || t == MYSQL_TYPE_LONGLONG
+           || t == MYSQL_TYPE_INT24;
+}
+
+
 void QMYSQLResultPrivate::bindBlobs()
 {
     int i;
@@ -368,6 +378,13 @@ bool QMYSQLResultPrivate::bindInValues()
             fieldInfo->length = 0;
             hasBlobs = true;
         } else {
+            // fieldInfo->length specifies the display width, which may be too
+            // small to hold valid integer values (see
+            // http://dev.mysql.com/doc/refman/5.0/en/numeric-types.html ), so
+            // always use the MAX_BIGINT_WIDTH for integer types
+            if (qIsInteger(fieldInfo->type)) {
+                fieldInfo->length = MAX_BIGINT_WIDTH;
+            }
             fieldInfo->type = MYSQL_TYPE_STRING;
         }
         bind = &inBinds[i];
@@ -403,10 +420,10 @@ QVariant QMYSQLResult::handle() const
 {
 #if MYSQL_VERSION_ID >= 40108
     if(d->preparedQuery)
-        return d->meta ? qVariantFromValue(d->meta) : qVariantFromValue(d->stmt);
+        return d->meta ? QVariant::fromValue(d->meta) : qVariantFromValue(d->stmt);
     else
 #endif
-        return qVariantFromValue(d->result);
+        return QVariant::fromValue(d->result);
 }
 
 void QMYSQLResult::cleanup()
@@ -509,15 +526,24 @@ bool QMYSQLResult::fetchNext()
         return false;
     if (d->preparedQuery) {
 #if MYSQL_VERSION_ID >= 40108
-        if (mysql_stmt_fetch(d->stmt))
+        int nRC = mysql_stmt_fetch(d->stmt);
+        if (nRC) {
+#ifdef MYSQL_DATA_TRUNCATED
+            if (nRC == 1 || nRC == MYSQL_DATA_TRUNCATED)
+#else
+            if (nRC == 1)
+#endif // MYSQL_DATA_TRUNCATED
+                setLastError(qMakeStmtError(QCoreApplication::translate("QMYSQLResult",
+                                    "Unable to fetch data"), QSqlError::StatementError, d->stmt));
             return false;
+        }
 #else
         return false;
 #endif
     } else {
-    d->row = mysql_fetch_row(d->result);
-    if (!d->row)
-        return false;
+        d->row = mysql_fetch_row(d->result);
+        if (!d->row)
+            return false;
     }
     setAt(at() + 1);
     return true;
@@ -965,11 +991,16 @@ bool QMYSQLResult::exec()
                     break; }
                 case QVariant::UInt:
                 case QVariant::Int:
-                case QVariant::Bool:
                     currBind->buffer_type = MYSQL_TYPE_LONG;
                     currBind->buffer = data;
                     currBind->buffer_length = sizeof(int);
                     currBind->is_unsigned = (val.type() != QVariant::Int);
+                break;
+                case QVariant::Bool:
+                    currBind->buffer_type = MYSQL_TYPE_TINY;
+                    currBind->buffer = data;
+                    currBind->buffer_length = sizeof(bool);
+                    currBind->is_unsigned = false;
                     break;
                 case QVariant::Double:
                     currBind->buffer_type = MYSQL_TYPE_DOUBLE;
@@ -1348,12 +1379,16 @@ QStringList QMYSQLDriver::tables(QSql::TableType type) const
     } else {
         QSqlQuery q(createResult());
         if(type & QSql::Tables) {
-            q.exec(QLatin1String("select table_name from information_schema.tables where table_type = 'BASE TABLE'"));
+            QString sql = QLatin1String("select table_name from information_schema.tables where table_schema = '") + QLatin1String(d->mysql->db) + QLatin1String("' and table_type = 'BASE TABLE'");
+            q.exec(sql);
+            
             while(q.next())
                 tl.append(q.value(0).toString());
         }
         if(type & QSql::Views) {
-            q.exec(QLatin1String("select table_name from information_schema.tables where table_type = 'VIEW'"));
+            QString sql = QLatin1String("select table_name from information_schema.tables where table_schema = '") + QLatin1String(d->mysql->db) + QLatin1String("' and table_type = 'VIEW'");
+            q.exec(sql);
+            
             while(q.next())
                 tl.append(q.value(0).toString());
         }
@@ -1365,7 +1400,6 @@ QStringList QMYSQLDriver::tables(QSql::TableType type) const
 QSqlIndex QMYSQLDriver::primaryIndex(const QString& tablename) const
 {
     QSqlIndex idx;
-    bool prepQ;
     if (!isOpen())
         return idx;
 
@@ -1407,7 +1441,7 @@ QSqlRecord QMYSQLDriver::record(const QString& tablename) const
 
 QVariant QMYSQLDriver::handle() const
 {
-    return qVariantFromValue(d->mysql);
+    return QVariant::fromValue(d->mysql);
 }
 
 bool QMYSQLDriver::beginTransaction()
