@@ -1,17 +1,18 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
-** All rights reserved.
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the QtSql module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
@@ -21,8 +22,8 @@
 ** ensure the GNU Lesser General Public License version 2.1 requirements
 ** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
@@ -33,8 +34,7 @@
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
+**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -136,7 +136,7 @@ void QSqlQueryModelPrivate::initColOffsets(int size)
     data before showing it to the user, and how to create a
     read-write model based on QSqlQueryModel.
 
-    If the database doesn't return the amount of selected rows in
+    If the database doesn't return the number of selected rows in
     a query, the model will fetch rows incrementally.
     See fetchMore() for more information.
 
@@ -175,7 +175,7 @@ QSqlQueryModel::~QSqlQueryModel()
     This only affects databases that don't report back the size of a query
     (see QSqlDriver::hasFeature()).
 
-    To force fetching of the entire database, you can use the following:
+    To force fetching of the entire result set, you can use the following:
 
     \snippet doc/src/snippets/code/src_sql_models_qsqlquerymodel.cpp 0
 
@@ -212,8 +212,8 @@ bool QSqlQueryModel::canFetchMore(const QModelIndex &parent) const
     \since 4.1
 
     If the database supports returning the size of a query
-    (see QSqlDriver::hasFeature()), the amount of rows of the current
-    query is returned. Otherwise, returns the amount of rows
+    (see QSqlDriver::hasFeature()), the number of rows of the current
+    query is returned. Otherwise, returns the number of rows
     currently cached on the client.
 
     \a parent should always be an invalid QModelIndex.
@@ -279,7 +279,11 @@ QVariant QSqlQueryModel::headerData(int section, Qt::Orientation orientation, in
             val = d->headers.value(section).value(Qt::EditRole);
         if (val.isValid())
             return val;
-        if (role == Qt::DisplayRole && d->rec.count() > section)
+
+        // See if it's an inserted column (iiq.column() != -1)
+        QModelIndex dItem = indexInQuery(createIndex(0, section));
+
+        if (role == Qt::DisplayRole && d->rec.count() > section && dItem.column() != -1)
             return d->rec.fieldName(section);
     }
     return QAbstractItemModel::headerData(section, orientation, role);
@@ -306,6 +310,8 @@ void QSqlQueryModel::queryChange()
     lastError() can be used to retrieve verbose information if there
     was an error setting the query.
 
+    \note Calling setQuery() will remove any inserted columns.
+
     \sa query(), QSqlQuery::isActive(), QSqlQuery::setForwardOnly(), lastError()
 */
 void QSqlQueryModel::setQuery(const QSqlQuery &query)
@@ -370,7 +376,8 @@ void QSqlQueryModel::setQuery(const QSqlQuery &query)
 /*! \overload
 
     Executes the query \a query for the given database connection \a
-    db. If no database is specified, the default connection is used.
+    db. If no database (or an invalid database) is specified, the
+    default connection is used.
 
     lastError() can be used to retrieve verbose information if there
     was an error setting the query.
