@@ -1,17 +1,18 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
-** All rights reserved.
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the QtTest module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
@@ -21,8 +22,8 @@
 ** ensure the GNU Lesser General Public License version 2.1 requirements
 ** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
@@ -33,8 +34,7 @@
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
+**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -54,7 +54,6 @@
 #include <QtCore/qdir.h>
 #include <QtCore/qprocess.h>
 #include <QtCore/qdebug.h>
-#include <QtCore/qlibraryinfo.h>
 
 #include "QtTest/private/qtestlog_p.h"
 #include "QtTest/private/qtesttable_p.h"
@@ -299,10 +298,14 @@ QT_BEGIN_NAMESPACE
 
     \relates QTest
 
-    Implements a main() function that instantiates a QApplication object and
+    Implements a main() function that instantiates an application object and
     the \a TestClass, and executes all tests in the order they were defined.
     Use this macro to build stand-alone executables.
 
+    If \c QT_GUI_LIB is defined, the application object will be a QApplication,
+    otherwise it will be a QCoreApplication.  If qmake is used and the configuration
+    includes \c{QT += gui}, then \c QT_GUI_LIB will be defined automatically.
+
     \bold {Note:} On platforms that have keypad navigation enabled by default (eg: Symbian),
     this macro will forcfully disable it to simplify the usage of key events when writing
     autotests. If you wish to write a test case that uses keypad navigation, you should
@@ -756,6 +759,9 @@ QT_BEGIN_NAMESPACE
     QTest::touchEvent to create a QTouchEventSequence instance. Add touch events to
     the sequence by calling press(), move(), release() and stationary(), and let the
     instance run out of scope to commit the sequence to the event system.
+
+    Example:
+    \snippet doc/src/snippets/code/src_qtestlib_qtestcase.cpp 25
 */
 
 /*!
@@ -827,25 +833,49 @@ namespace QTest
 {
     static QObject *currentTestObject = 0;
 
-    static struct TestFunction {
-        TestFunction():function(0), data(0) {}
-        ~TestFunction() { delete [] data; }
-        int function;
-        char *data;
-    } *testFuncs;
-
+    class TestFunction {
+    public:
+        TestFunction() : function_(-1), data_(0) {}
+        void set(int function, char *data) { function_ = function; data_ = data; }
+        char *data() const { return data_; }
+        int function() const { return function_; }
+        ~TestFunction() { delete[] data_; }
+    private:
+        int function_;
+        char *data_;
+    };
     /**
-     * Contains the count of test functions that was supplied
-     * on the command line, if any. Hence, if lastTestFuncIdx is
-     * more than zero, those functions should be run instead of
+     * Contains the list of test functions that was supplied
+     * on the command line, if any. Hence, if not empty,
+     * those functions should be run instead of
      * all appearing in the test case.
      */
-    static int lastTestFuncIdx = -1;
+    static TestFunction * testFuncs = 0;
+    static int testFuncCount = 0;
+
+    /** Don't leak testFuncs on exit even on error */
+    static struct TestFuncCleanup
+    {
+        void cleanup()
+        {
+            delete[] testFuncs;
+            testFuncCount = 0;
+            testFuncs = 0;
+        }
+
+        ~TestFuncCleanup() { cleanup(); }
+    } testFuncCleaner;
 
     static int keyDelay = -1;
     static int mouseDelay = -1;
     static int eventDelay = -1;
+    static bool randomOrder = false;
     static int keyVerbose = -1;
+    static unsigned int seed = 0;
+    static bool seedSet = false;
+#if defined(Q_OS_UNIX) && !defined(Q_OS_SYMBIAN)
+    static bool noCrashHandler = false;
+#endif
 
 void filter_unprintable(char *str)
 {
@@ -874,6 +904,19 @@ int qt_snprintf(char *str, int size, const char *format, ...)
     return res;
 }
 
+/*! \internal
+    Invoke a method of the object without generating warning if the method does not exist
+ */
+static void invokeMethod(QObject *obj, const char *methodName)
+{
+    const QMetaObject *metaObject = obj->metaObject();
+    int funcIndex = metaObject->indexOfMethod(methodName);
+    if (funcIndex >= 0) {
+        QMetaMethod method = metaObject->method(funcIndex);
+        method.invoke(obj, Qt::DirectConnection);
+    }
+}
+
 bool Q_TESTLIB_EXPORT defaultKeyVerbose()
 {
     if (keyVerbose == -1) {
@@ -915,6 +958,43 @@ int Q_TESTLIB_EXPORT defaultKeyDelay()
     return keyDelay;
 }
 
+void seedRandom()
+{
+    static bool randomSeeded = false;
+    if (!randomSeeded) {
+        if (!QTest::seedSet) {
+            QElapsedTimer timer;
+            timer.start();
+            QTest::seed = timer.msecsSinceReference();
+        }
+        qsrand(QTest::seed);
+        randomSeeded = true;
+    }
+}
+
+int qTestRandomSeed()
+{
+    Q_ASSERT(QTest::seedSet);
+    return QTest::seed;
+}
+
+template<typename T>
+void swap(T * array, int pos, int otherPos)
+{
+    T tmp = array[pos];
+    array[pos] = array[otherPos];
+    array[otherPos] = tmp;
+}
+
+template<typename T>
+static void randomizeList(T * array, int size)
+{
+    for (int i = 0; i != size; i++) {
+        int pos = qrand() % size;
+        swap(array, pos, i);
+    }
+}
+
 static bool isValidSlot(const QMetaMethod &sl)
 {
     if (sl.access() != QMetaMethod::Private || !sl.parameterTypes().isEmpty()
@@ -934,6 +1014,11 @@ static bool isValidSlot(const QMetaMethod &sl)
     return true;
 }
 
+Q_TESTLIB_EXPORT bool printAvailableFunctions = false;
+Q_TESTLIB_EXPORT bool printAvailableTags = false;
+Q_TESTLIB_EXPORT QStringList testFunctions;
+Q_TESTLIB_EXPORT QStringList testTags;
+
 static void qPrintTestSlots()
 {
     for (int i = 0; i < QTest::currentTestObject->metaObject()->methodCount(); ++i) {
@@ -943,6 +1028,65 @@ static void qPrintTestSlots()
     }
 }
 
+static void qPrintDataTags()
+{
+    // Get global data tags:
+    QTestTable::globalTestTable();
+    invokeMethod(QTest::currentTestObject, "initTestCase_data()");
+    const QTestTable *gTable = QTestTable::globalTestTable();
+
+    const QMetaObject *currTestMetaObj = QTest::currentTestObject->metaObject();
+
+    // Process test functions:
+    for (int i = 0; i < currTestMetaObj->methodCount(); ++i) {
+        QMetaMethod tf = currTestMetaObj->method(i);
+        if (isValidSlot(tf)) {
+            // Retrieve local tags:
+            QStringList localTags;
+            QTestTable table;
+            char member[512];
+            char *slot = qstrdup(tf.signature());
+            slot[strlen(slot) - 2] = '\0';
+            QTest::qt_snprintf(member, 512, "%s_data()", slot);
+            invokeMethod(QTest::currentTestObject, member);
+            for (int j = 0; j < table.dataCount(); ++j)
+                localTags << QLatin1String(table.testData(j)->dataTag());
+
+            // Print all tag combinations:
+            if (gTable->dataCount() == 0) {
+                if (localTags.count() == 0) {
+                    // No tags at all, so just print the test function:
+                    printf("%s %s\n", currTestMetaObj->className(), slot);
+                } else {
+                    // Only local tags, so print each of them:
+                    for (int k = 0; k < localTags.size(); ++k)
+                        printf(
+                            "%s %s %s\n",
+                            currTestMetaObj->className(), slot, localTags.at(k).toLatin1().data());
+                }
+            } else {
+                for (int j = 0; j < gTable->dataCount(); ++j) {
+                    if (localTags.count() == 0) {
+                        // Only global tags, so print the current one:
+                        printf(
+                            "%s %s __global__ %s\n",
+                            currTestMetaObj->className(), slot, gTable->testData(j)->dataTag());
+                    } else {
+                        // Local and global tags, so print each of the local ones and
+                        // the current global one:
+                        for (int k = 0; k < localTags.size(); ++k)
+                            printf(
+                                "%s %s %s __global__ %s\n", currTestMetaObj->className(), slot,
+                                localTags.at(k).toLatin1().data(), gTable->testData(j)->dataTag());
+                    }
+                }
+            }
+
+            delete[] slot;
+        }
+    }
+}
+
 static int qToInt(char *str)
 {
     char *pEnd;
@@ -954,28 +1098,34 @@ static int qToInt(char *str)
     return l;
 }
 
-static void qParseArgs(int argc, char *argv[])
+Q_TESTLIB_EXPORT void qtest_qParseArgs(int argc, char *argv[], bool qml)
 {
-    lastTestFuncIdx = -1;
-
     const char *testOptions =
          " options:\n"
          " -functions : Returns a list of current testfunctions\n"
+         " -datatags  : Returns a list of current data tags.\n"
+         "              A global data tag is preceded by ' __global__ '.\n"
          " -xunitxml  : Outputs results as XML XUnit document\n"
          " -xml       : Outputs results as XML document\n"
          " -lightxml  : Outputs results as stream of XML tags\n"
-         " -flush     : Flushes the resutls\n"
+         " -flush     : Flushes the results\n"
          " -o filename: Writes all output into a file\n"
          " -silent    : Only outputs warnings and failures\n"
          " -v1        : Print enter messages for each testfunction\n"
          " -v2        : Also print out each QVERIFY/QCOMPARE/QTEST\n"
          " -vs        : Print every signal emitted\n"
+         " -random    : Run testcases within each test in random order\n"
+         " -seed n    : Positive integer to be used as seed for -random. If not specified,\n"
+         "              the current time will be used as seed.\n"
          " -eventdelay ms    : Set default delay for mouse and keyboard simulation to ms milliseconds\n"
          " -keydelay ms      : Set default delay for keyboard simulation to ms milliseconds\n"
          " -mousedelay ms    : Set default delay for mouse simulation to ms milliseconds\n"
          " -keyevent-verbose : Turn on verbose messages for keyboard simulation\n"
          " -maxwarnings n    : Sets the maximum amount of messages to output.\n"
          "                     0 means unlimited, default: 2000\n"
+#if defined(Q_OS_UNIX) && !defined(Q_OS_SYMBIAN)
+         " -nocrashhandler   : Disables the crash handler\n"
+#endif
          "\n"
          " Benchmark related options:\n"
 #ifdef QTESTLIB_USE_VALGRIND
@@ -989,9 +1139,6 @@ static void qParseArgs(int argc, char *argv[])
         " -iterations  n  : Sets the number of accumulation iterations.\n"
         " -median  n      : Sets the number of median iterations.\n"
         " -vb             : Print out verbose benchmarking information.\n"
-#if !defined(QT_NO_PROCESS) && !defined(QT_NO_SETTINGS)
-        " -chart          : Create chart based on the benchmark result.\n"
-#endif
          "\n"
         " -help      : This help\n";
 
@@ -1003,8 +1150,18 @@ static void qParseArgs(int argc, char *argv[])
                    "%s", argv[0], testOptions);
             exit(0);
         } else if (strcmp(argv[i], "-functions") == 0) {
-            qPrintTestSlots();
-            exit(0);
+            if (qml) {
+                QTest::printAvailableFunctions = true;
+            } else {
+                qPrintTestSlots();
+                exit(0);
+            }
+        } else if (strcmp(argv[i], "-datatags") == 0) {
+            QTest::printAvailableTags = true;
+            if (!qml) {
+                qPrintDataTags();
+                exit(0);
+            }
         } else if(strcmp(argv[i], "-xunitxml") == 0){
             QTestLog::setLogMode(QTestLog::XunitXML);
         } else if (strcmp(argv[i], "-xml") == 0) {
@@ -1056,6 +1213,10 @@ static void qParseArgs(int argc, char *argv[])
             } else {
                 QTestLog::setMaxWarnings(qToInt(argv[++i]));
             }
+#if defined(Q_OS_UNIX) && !defined(Q_OS_SYMBIAN)
+        } else if (strcmp(argv[i], "-nocrashhandler") == 0) {
+            QTest::noCrashHandler = true;
+#endif
         } else if (strcmp(argv[i], "-keyevent-verbose") == 0) {
             QTest::keyVerbose = 1;
 #ifdef QTESTLIB_USE_VALGRIND
@@ -1081,6 +1242,22 @@ static void qParseArgs(int argc, char *argv[])
 #endif
         } else if (strcmp(argv[i], "-eventcounter") == 0) {
             QBenchmarkGlobalData::current->setMode(QBenchmarkGlobalData::EventCounter);
+        } else if (strcmp(argv[i], "-random") == 0) {
+            QTest::randomOrder = true;
+        } else if (strcmp(argv[i], "-seed") == 0) {
+            bool argumentOk = false;
+            if (i + 1 < argc) {
+                char * endpt = 0;
+                long longSeed = strtol(argv[++i], &endpt, 10);
+                argumentOk = (*endpt == '\0' && longSeed >= 0);
+                QTest::seed = longSeed;
+            }
+            if (!argumentOk) {
+                printf("-seed needs an extra positive integer parameter to specify the seed\n");
+                exit(1);
+            } else {
+                QTest::seedSet = true;
+            }
         } else if (strcmp(argv[i], "-minimumvalue") == 0) {
             if (i + 1 >= argc) {
                 printf("-minimumvalue needs an extra parameter to indicate the minimum time(ms)\n");
@@ -1105,18 +1282,52 @@ static void qParseArgs(int argc, char *argv[])
 
         } else if (strcmp(argv[i], "-vb") == 0) {
             QBenchmarkGlobalData::current->verboseOutput = true;
-#if !defined(QT_NO_PROCESS) && !defined(QT_NO_SETTINGS)
         } else if (strcmp(argv[i], "-chart") == 0) {
-            QBenchmarkGlobalData::current->createChart = true;
-            QTestLog::setLogMode(QTestLog::XML);
-            QTestLog::redirectOutput("results.xml");
-#endif
+            fprintf(stderr, "Warning: `-chart' option is not available\n");
         } else if (strcmp(argv[i], "-qws") == 0) {
             // do nothing
+        } else if (strcmp(argv[i], "-graphicssystem") == 0) {
+            // do nothing
+            if (i + 1 >= argc) {
+                printf("-graphicssystem needs an extra parameter specifying the graphics system\n");
+                exit(1);
+            } else {
+                ++i;
+            }
         } else if (argv[i][0] == '-') {
             printf("Unknown option: '%s'\n\n%s", argv[i], testOptions);
             exit(1);
+        } else if (qml) {
+            // We can't check the availability of test functions until
+            // we load the QML files.  So just store the data for now.
+            int colon = -1;
+            int offset;
+            for(offset = 0; *(argv[i]+offset); ++offset) {
+                if (*(argv[i]+offset) == ':') {
+                    if (*(argv[i]+offset+1) == ':') {
+                        // "::" is used as a test name separator.
+                        // e.g. "ClickTests::test_click:row1".
+                        ++offset;
+                    } else {
+                        colon = offset;
+                        break;
+                    }
+                }
+            }
+            if (colon == -1) {
+                QTest::testFunctions += QString::fromLatin1(argv[i]);
+                QTest::testTags += QString();
+            } else {
+                QTest::testFunctions +=
+                    QString::fromLatin1(argv[i], colon);
+                QTest::testTags +=
+                    QString::fromLatin1(argv[i] + colon + 1);
+            }
         } else {
+			if (!QTest::testFuncs) {
+		        QTest::testFuncs = new QTest::TestFunction[512];
+			}
+
             int colon = -1;
             char buf[512], *data=0;
             int off;
@@ -1138,17 +1349,16 @@ static void qParseArgs(int argc, char *argv[])
                 qPrintTestSlots();
                 exit(1);
             }
-            ++QTest::lastTestFuncIdx;
-            if (!QTest::testFuncs) {
-                struct Cleanup { ~Cleanup() { delete[] QTest::testFuncs; } };
-                static Cleanup cleanup;
-                QTest::testFuncs = new TestFunction[512];
-            }
-            QTest::testFuncs[QTest::lastTestFuncIdx].function = idx;
-            QTest::testFuncs[QTest::lastTestFuncIdx].data = data;
-            QTEST_ASSERT(QTest::lastTestFuncIdx < 512);
+            testFuncs[testFuncCount].set(idx, data);
+			testFuncCount++;
+			QTEST_ASSERT(QTest::testFuncCount < 512);
         }
     }
+
+    if (QTest::seedSet && !QTest::randomOrder) {
+        printf("-seed requires -random\n");
+        exit(1);
+    }
 }
 
 QBenchmarkResult qMedian(const QList<QBenchmarkResult> &container)
@@ -1195,7 +1405,7 @@ static void qInvokeTestMethodDataEntry(char *slot)
         bool invokeOk;
         do {
             QTestResult::setCurrentTestLocation(QTestResult::InitFunc);
-            QMetaObject::invokeMethod(QTest::currentTestObject, "init");
+            invokeMethod(QTest::currentTestObject, "init()");
             if (QTestResult::skipCurrentTest())
                 break;
 
@@ -1215,7 +1425,7 @@ static void qInvokeTestMethodDataEntry(char *slot)
                 QTestResult::addFailure("Unable to execute slot", __FILE__, __LINE__);
 
             QTestResult::setCurrentTestLocation(QTestResult::CleanupFunc);
-            QMetaObject::invokeMethod(QTest::currentTestObject, "cleanup");
+            invokeMethod(QTest::currentTestObject, "cleanup()");
             QTestResult::setCurrentTestLocation(QTestResult::NoWhere);
 
             // If this test method has a benchmark, repeat until all measurements are
@@ -1282,8 +1492,9 @@ static bool qInvokeTestMethod(const char *slotName, const char *data=0)
 
         if (curGlobalDataIndex == 0) {
             QTestResult::setCurrentTestLocation(QTestResult::DataFunc);
-            QTest::qt_snprintf(member, 512, "%s_data", slot);
-            QMetaObject::invokeMethod(QTest::currentTestObject, member, Qt::DirectConnection);
+            QTest::qt_snprintf(member, 512, "%s_data()", slot);
+            invokeMethod(QTest::currentTestObject, member);
+
             // if we encounter a SkipAll in the _data slot, we skip the whole
             // testfunction, no matter how much global data exists
             if (QTestResult::skipCurrentTest()) {
@@ -1298,11 +1509,23 @@ static bool qInvokeTestMethod(const char *slotName, const char *data=0)
             const int dataCount = table.dataCount();
             QTestResult::setSkipCurrentTest(false);
 
+            // Data tag requested but none available?
+            if (data && !dataCount) {
+                // Let empty data tag through.
+                if (!*data)
+                    data = 0;
+                else {
+                    printf("Unknown testdata for function %s: '%s'\n", slotName, data);
+                    printf("Function has no testdata.\n");
+                    return false;
+                }
+            }
+
             /* For each entry in the data table, do: */
             do {
                 if (!data || !qstrcmp(data, table.testData(curDataIndex)->dataTag())) {
                     foundFunction = true;
-                    QTestDataSetter s(table.isEmpty() ? static_cast<QTestData *>(0)
+                    QTestDataSetter s(curDataIndex >= dataCount ? static_cast<QTestData *>(0)
                                                       : table.testData(curDataIndex));
 
                     qInvokeTestMethodDataEntry(slot);
@@ -1430,17 +1653,19 @@ static void qInvokeTestMethods(QObject *testObject)
 {
     const QMetaObject *metaObject = testObject->metaObject();
     QTEST_ASSERT(metaObject);
-
-    QTestLog::startLogging();
-
+    if (QTest::randomOrder) {
+        QTestLog::startLogging(QTest::seed);
+    } else {
+        QTestLog::startLogging();
+    }
     QTestResult::setCurrentTestFunction("initTestCase");
     QTestResult::setCurrentTestLocation(QTestResult::DataFunc);
     QTestTable::globalTestTable();
-    QMetaObject::invokeMethod(testObject, "initTestCase_data", Qt::DirectConnection);
+    invokeMethod(testObject, "initTestCase_data()");
 
     if (!QTestResult::skipCurrentTest() && !QTest::currentTestFailed()) {
         QTestResult::setCurrentTestLocation(QTestResult::InitFunc);
-        QMetaObject::invokeMethod(testObject, "initTestCase");
+        invokeMethod(testObject, "initTestCase()");
 
         // finishedCurrentTestFunction() resets QTestResult::testFailed(), so use a local copy.
         const bool previousFailed = QTestResult::testFailed();
@@ -1448,27 +1673,37 @@ static void qInvokeTestMethods(QObject *testObject)
 
         if(!QTestResult::skipCurrentTest() && !previousFailed) {
 
-            if (lastTestFuncIdx >= 0) {
-                for (int i = 0; i <= lastTestFuncIdx; ++i) {
-                    if (!qInvokeTestMethod(metaObject->method(testFuncs[i].function).signature(),
-                                           testFuncs[i].data))
+            if (QTest::testFuncs) {
+                if (QTest::randomOrder)
+                    randomizeList(QTest::testFuncs, QTest::testFuncCount);
+                for (int i = 0; i != QTest::testFuncCount; i++) {
+                    if (!qInvokeTestMethod(metaObject->method(QTest::testFuncs[i].function()).signature(),
+                                                              QTest::testFuncs[i].data())) {
                         break;
+                    }
                 }
+                testFuncCleaner.cleanup();
             } else {
                 int methodCount = metaObject->methodCount();
-                for (int i = 0; i < methodCount; ++i) {
-                    QMetaMethod slotMethod = metaObject->method(i);
-                    if (!isValidSlot(slotMethod))
+                QMetaMethod *testMethods = new QMetaMethod[methodCount];
+                for (int i = 0; i != methodCount; i++)
+                    testMethods[i] = metaObject->method(i);
+                if (QTest::randomOrder)
+                    randomizeList(testMethods, methodCount);
+                for (int i = 0; i != methodCount; i++) {
+                    if (!isValidSlot(testMethods[i]))
                         continue;
-                    if (!qInvokeTestMethod(slotMethod.signature()))
+                    if (!qInvokeTestMethod(testMethods[i].signature()))
                         break;
                 }
+                delete[] testMethods;
+                testMethods = 0;
             }
         }
 
         QTestResult::setSkipCurrentTest(false);
         QTestResult::setCurrentTestFunction("cleanupTestCase");
-        QMetaObject::invokeMethod(testObject, "cleanupTestCase");
+        invokeMethod(testObject, "cleanupTestCase()");
     }
     QTestResult::finishedCurrentTestFunction();
     QTestResult::setCurrentTestFunction(0);
@@ -1492,6 +1727,14 @@ private:
 void FatalSignalHandler::signal(int signum)
 {
     qFatal("Received signal %d", signum);
+#if defined(Q_OS_INTEGRITY)
+    {
+        struct sigaction act;
+        memset(&act, 0, sizeof(struct sigaction));
+        act.sa_handler = SIG_DFL;
+        sigaction(signum, &act, NULL);
+    }
+#endif
 }
 
 FatalSignalHandler::FatalSignalHandler()
@@ -1506,8 +1749,9 @@ FatalSignalHandler::FatalSignalHandler()
     act.sa_handler = FatalSignalHandler::signal;
 
     // Remove the handler after it is invoked.
+#if !defined(Q_OS_INTEGRITY)
     act.sa_flags = SA_RESETHAND;
-
+#endif
     // Block all fatal signals in our signal handler so we don't try to close
     // the testlog twice.
     sigemptyset(&act.sa_mask);
@@ -1521,7 +1765,11 @@ FatalSignalHandler::FatalSignalHandler()
 #ifndef Q_WS_QWS
         // Don't overwrite any non-default handlers
         // however, we need to replace the default QWS handlers
-        if (oldact.sa_flags & SA_SIGINFO || oldact.sa_handler != SIG_DFL) {
+        if (
+#ifdef SA_SIGINFO
+            oldact.sa_flags & SA_SIGINFO ||
+#endif
+            oldact.sa_handler != SIG_DFL) {
             sigaction(fatalSignals[i], &oldact, 0);
         } else
 #endif
@@ -1648,7 +1896,13 @@ int QTest::qExec(QObject *testObject, int argc, char **argv)
     QTEST_ASSERT(metaObject);
 
     QTestResult::setCurrentTestObject(metaObject->className());
-    qParseArgs(argc, argv);
+    if (argc > 0)
+        QTestResult::setCurrentAppName(argv[0]);
+
+    qtest_qParseArgs(argc, argv, false);
+    if (QTest::randomOrder) {
+        seedRandom();
+    }
 #ifdef QTESTLIB_USE_VALGRIND
     if (QBenchmarkGlobalData::current->mode() == QBenchmarkGlobalData::CallgrindParentProcess) {
         const QStringList origAppArgs(QCoreApplication::arguments());
@@ -1661,7 +1915,9 @@ int QTest::qExec(QObject *testObject, int argc, char **argv)
 #endif
     {
 #if defined(Q_OS_UNIX) && !defined(Q_OS_SYMBIAN)
-        FatalSignalHandler handler;
+        QScopedPointer<FatalSignalHandler> handler;
+        if (!noCrashHandler)
+            handler.reset(new FatalSignalHandler);
 #endif
         qInvokeTestMethods(testObject);
     }
@@ -1680,6 +1936,8 @@ int QTest::qExec(QObject *testObject, int argc, char **argv)
              IOPMAssertionRelease(powerID);
          }
 #endif
+         currentTestObject = 0;
+
          // Rethrow exception to make debugging easier.
          throw;
          return 1;
@@ -1693,26 +1951,6 @@ int QTest::qExec(QObject *testObject, int argc, char **argv)
      }
 #endif
 
-
-#if !defined(QT_NO_PROCESS) && !defined(QT_NO_SETTINGS)
-    if (QBenchmarkGlobalData::current->createChart) {
-        QString chartLocation = QLibraryInfo::location(QLibraryInfo::BinariesPath);
-#ifdef Q_OS_WIN
-        chartLocation += QLatin1String("/../tools/qtestlib/chart/release/chart.exe");
-#else
-        chartLocation += QLatin1String("/../tools/qtestlib/chart/chart");
-#endif
-        if (QFile::exists(chartLocation)) {
-            QProcess p;
-            p.setProcessChannelMode(QProcess::ForwardedChannels);
-            p.start(chartLocation, QStringList() << QLatin1String("results.xml"));
-            p.waitForFinished(-1);
-        } else {
-            qDebug() << QLatin1String("Could not find the chart tool in ") + chartLocation + QLatin1String(", please make sure it is compiled.");
-        }
-    }
-#endif
-
 #if defined(QTEST_NOEXITCODE)
     return 0;
 #else
@@ -1903,6 +2141,14 @@ QTestData &QTest::newRow(const char *dataTag)
     \sa QTest::newRow(), QFETCH(), QMetaType
 */
 
+/*!
+    Returns the name of the binary that is currently executed.
+*/
+const char *QTest::currentAppName()
+{
+    return QTestResult::currentAppName();
+}
+
 /*!
     Returns the name of the test function that is currently executed.
 
