@@ -1,17 +1,18 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
-** All rights reserved.
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the Qt Assistant of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
@@ -21,8 +22,8 @@
 ** ensure the GNU Lesser General Public License version 2.1 requirements
 ** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
@@ -33,8 +34,7 @@
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
+**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -90,6 +90,7 @@ bool QHelpDBReader::init()
         return false;
 
     QSqlDatabase db = QSqlDatabase::addDatabase(QLatin1String("QSQLITE"), m_uniqueId);
+    db.setConnectOptions(QLatin1String("QSQLITE_OPEN_READONLY"));
     db.setDatabaseName(m_dbName);
     if (!db.open()) {
         /*: The placeholders are: %1 - The name of the database which cannot be opened
@@ -205,7 +206,7 @@ QByteArray QHelpDBReader::fileData(const QString &virtualFolder,
         "NamespaceTable d WHERE a.Id=b.FileId AND (b.Name=? OR b.Name=?) AND b.FolderId=c.Id "
         "AND c.Name=? AND c.NamespaceId=d.Id AND d.Name=?"));
     m_query->bindValue(0, filePath);
-    m_query->bindValue(1, QLatin1String("./") + filePath);
+    m_query->bindValue(1, QString(QLatin1String("./") + filePath));
     m_query->bindValue(2, virtualFolder);
     m_query->bindValue(3, m_namespace);
     m_query->exec();
@@ -515,7 +516,7 @@ QVariant QHelpDBReader::metaData(const QString &name) const
 QString QHelpDBReader::mergeList(const QStringList &list) const
 {
     QString str;
-    foreach (QString s, list)
+    foreach (const QString &s, list)
         str.append(QLatin1Char('\'') + quote(s) + QLatin1String("\', "));
     if (str.endsWith(QLatin1String(", ")))
         str = str.left(str.length()-2);
@@ -567,14 +568,14 @@ bool QHelpDBReader::createAttributesCache(const QStringList &attributes,
 
     bool needUpdate = !m_viewAttributes.count();
 
-    foreach (QString s, attributes)
+    foreach (const QString &s, attributes)
         m_viewAttributes.remove(s);
 
     if (m_viewAttributes.count() || needUpdate) {
         m_viewAttributes.clear();
         m_indicesCache = indexIds;        
     }
-    foreach (QString s, attributes)
+    foreach (const QString &s, attributes)
         m_viewAttributes.insert(s);
     m_useAttributesCache = true;
     return true;
