@@ -1,17 +1,18 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
-** All rights reserved.
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the Qt Assistant of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
@@ -21,8 +22,8 @@
 ** ensure the GNU Lesser General Public License version 2.1 requirements
 ** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
@@ -33,29 +34,36 @@
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
+**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
+#include "../shared/collectionconfiguration.h"
 #include "../shared/helpgenerator.h"
 
-#include <QtCore/QDir>
-#include <QtCore/QMap>
-#include <QtCore/QFileInfo>
-#include <QtCore/QCoreApplication>
-#include <QtCore/QDateTime>
-#include <QtCore/QBuffer>
-
 #include <private/qhelpgenerator_p.h>
 #include <private/qhelpprojectdata_p.h>
-#include <QtHelp/QHelpEngineCore>
 
+#include <QtCore/QCoreApplication>
+#include <QtCore/QDir>
+#include <QtCore/QMap>
+#include <QtCore/QFileInfo>
+#include <QtCore/QDateTime>
+#include <QtCore/QBuffer>
+#include <QtCore/QTranslator>
+#include <QtCore/QLocale>
+#include <QtCore/QLibraryInfo>
+#include <QtHelp/QHelpEngineCore>
 #include <QtXml/QXmlStreamReader>
 
+
 QT_USE_NAMESPACE
 
+class QCG {
+    Q_DECLARE_TR_FUNCTIONS(QCollectionGenerator)
+};
+
 class CollectionConfigReader : public QXmlStreamReader
 {
 public:
@@ -87,6 +95,11 @@ public:
     QStringList filesToRegister() const { return m_filesToRegister; }
 
     QString cacheDirectory() const { return m_cacheDirectory; }
+    bool cacheDirRelativeToCollection() const { return m_cacheDirRelativeToCollection; }
+
+    bool fullTextSearchFallbackEnabled() const {
+        return m_enableFullTextSearchFallback;
+    }
 
 private:
     void raiseErrorWithLine();
@@ -115,12 +128,13 @@ private:
     QMap<QString, QString> m_filesToGenerate;
     QStringList m_filesToRegister;
     QString m_cacheDirectory;
+    bool m_cacheDirRelativeToCollection;
+    bool m_enableFullTextSearchFallback;
 };
 
 void CollectionConfigReader::raiseErrorWithLine()
 {
-    raiseError(QObject::tr("Unknown token at line %1.")
-                    .arg(lineNumber()));
+    raiseError(QCG::tr("Unknown token at line %1.").arg(lineNumber()));
 }
 
 void CollectionConfigReader::readData(const QByteArray &contents)
@@ -130,6 +144,7 @@ void CollectionConfigReader::readData(const QByteArray &contents)
     m_enableAddressBar = true;
     m_hideAddressBar = true;
     m_enableDocumentationManager = true;
+    m_enableFullTextSearchFallback = false;
 
     addData(contents);
     while (!atEnd()) {
@@ -139,8 +154,9 @@ void CollectionConfigReader::readData(const QByteArray &contents)
                 && attributes().value(QLatin1String("version")) == QLatin1String("1.0"))
                 readConfig();
             else
-                raiseError(QObject::tr("Unknown token at line %1. Expected \"QtHelpCollectionProject\"!")
-                .arg(lineNumber()));
+                raiseError(QCG::tr("Unknown token at line %1. "
+                                   "Expected \"QtHelpCollectionProject\".")
+                           .arg(lineNumber()));
         }
     }
 }
@@ -162,7 +178,7 @@ void CollectionConfigReader::readConfig()
         }
     }
     if (!ok && !hasError())
-        raiseError(QLatin1String("Missing end tags."));
+        raiseError(QCG::tr("Missing end tags."));
 }
 
 void CollectionConfigReader::readAssistantSettings()
@@ -198,7 +214,13 @@ void CollectionConfigReader::readAssistantSettings()
             } else if (name() == QLatin1String("aboutDialog")) {
                 readAboutDialog();
             } else if (name() == "cacheDirectory") {
+                m_cacheDirRelativeToCollection =
+                    attributes().value(QLatin1String("base"))
+                    == QLatin1String("collection");
                 m_cacheDirectory = readElementText();
+            } else if (name() == QLatin1String("enableFullTextSearchFallback")) {
+                if (readElementText() == QLatin1String("true"))
+                    m_enableFullTextSearchFallback = true;
             } else {
                 raiseErrorWithLine();
             }
@@ -301,7 +323,7 @@ void CollectionConfigReader::readFiles()
         }
     }
     if (input.isEmpty() || output.isEmpty()) {
-        raiseError(QLatin1String("Missing input or output file for help file generation!"));
+        raiseError(QCG::tr("Missing input or output file for help file generation."));
         return;
     }
     m_filesToGenerate.insert(input, output);
@@ -322,6 +344,14 @@ void CollectionConfigReader::readRegister()
     }
 }
 
+namespace {
+    QString absoluteFileName(const QString &basePath, const QString &fileName)
+    {
+        return QFileInfo(fileName).isAbsolute() ?
+            fileName : basePath + QDir::separator() + fileName;
+    }
+}
+
 int main(int argc, char *argv[])
 {
     QString error;
@@ -332,6 +362,22 @@ int main(int argc, char *argv[])
     bool showHelp = false;
     bool showVersion = false;
 
+    QCoreApplication app(argc, argv);
+#ifndef Q_OS_WIN32
+    QTranslator translator;
+    QTranslator qtTranslator;
+    QTranslator qt_helpTranslator;
+    QString sysLocale = QLocale::system().name();
+    QString resourceDir = QLibraryInfo::location(QLibraryInfo::TranslationsPath);
+    if (translator.load(QLatin1String("assistant_") + sysLocale, resourceDir)
+        && qtTranslator.load(QLatin1String("qt_") + sysLocale, resourceDir)
+        && qt_helpTranslator.load(QLatin1String("qt_help_") + sysLocale, resourceDir)) {
+        app.installTranslator(&translator);
+        app.installTranslator(&qtTranslator);
+        app.installTranslator(&qt_helpTranslator);
+    }
+#endif // Q_OS_WIN32
+
     for (int i=1; i<argc; ++i) {
         arg = QString::fromLocal8Bit(argv[i]);
         if (arg == QLatin1String("-o")) {
@@ -339,7 +385,7 @@ int main(int argc, char *argv[])
                 QFileInfo fi(QString::fromLocal8Bit(argv[i]));
                 collectionFile = fi.absoluteFilePath();
             } else {
-                error = QObject::tr("Missing output file name!");
+                error = QCG::tr("Missing output file name.");
             }
         } else if (arg == QLatin1String("-h")) {
             showHelp = true;
@@ -353,14 +399,15 @@ int main(int argc, char *argv[])
     }
 
     if (showVersion) {
-        fprintf(stdout, "Qt Collection Generator version 1.0 (Qt %s)\n", QT_VERSION_STR);
+        fputs(qPrintable(QCG::tr("Qt Collection Generator version 1.0 (Qt %1)\n")
+                .arg(QT_VERSION_STR)), stdout);
         return 0;
     }
 
     if (configFile.isEmpty() && !showHelp)
-        error = QObject::tr("Missing collection config file!");
+        error = QCG::tr("Missing collection config file.");
 
-    QString help = QObject::tr("\nUsage:\n\n"
+    QString help = QCG::tr("\nUsage:\n\n"
         "qcollectiongenerator <collection-config-file> [options]\n\n"
         "  -o <collection-file>   Generates a collection file\n"
         "                         called <collection-file>. If\n"
@@ -370,7 +417,7 @@ int main(int argc, char *argv[])
         "                         qcollectiongenerator.\n\n");
 
     if (showHelp) {
-        fprintf(stdout, "%s", qPrintable(help));
+        fputs(qPrintable(help), stdout);
         return 0;
     }else if (!error.isEmpty()) {
         fprintf(stderr, "%s\n\n%s", qPrintable(error), qPrintable(help));
@@ -379,7 +426,7 @@ int main(int argc, char *argv[])
 
     QFile file(configFile);
     if (!file.open(QIODevice::ReadOnly)) {
-        fprintf(stderr, "Could not open %s!\n", qPrintable(configFile));
+        fputs(qPrintable(QCG::tr("Could not open %1.\n").arg(configFile)), stderr);
         return -1;
     }
 
@@ -389,39 +436,39 @@ int main(int argc, char *argv[])
             + fi.baseName() + QLatin1String(".qhc");
     }
 
-    QCoreApplication app(argc, argv);
-
-    fprintf(stdout, "Reading collection config file...\n");
+    fputs(qPrintable(QCG::tr("Reading collection config file...\n")), stdout);
     CollectionConfigReader config;
     config.readData(file.readAll());
     if (config.hasError()) {
-        fprintf(stderr, "Collection config file error: %s\n", qPrintable(config.errorString()));
+        fputs(qPrintable(QCG::tr("Collection config file error: %1\n")
+                         .arg(config.errorString())), stderr);
         return -1;
     }
 
     QMap<QString, QString>::const_iterator it = config.filesToGenerate().constBegin();
     while (it != config.filesToGenerate().constEnd()) {
-        fprintf(stdout, "Generating help for %s...\n", qPrintable(it.key()));
+        fputs(qPrintable(QCG::tr("Generating help for %1...\n").arg(it.key())), stdout);
         QHelpProjectData helpData;
-        if (!helpData.readData(basePath + QDir::separator() + it.key())) {
+        if (!helpData.readData(absoluteFileName(basePath, it.key()))) {
             fprintf(stderr, "%s\n", qPrintable(helpData.errorMessage()));
             return -1;
         }
 
         HelpGenerator helpGenerator;
-        if (!helpGenerator.generate(&helpData, basePath + QDir::separator() + it.value())) {
+        if (!helpGenerator.generate(&helpData, absoluteFileName(basePath, it.value()))) {
             fprintf(stderr, "%s\n", qPrintable(helpGenerator.error()));
             return -1;
         }
         ++it;
     }
 
-    fprintf(stdout, "Creating collection file...\n");
+    fputs(qPrintable(QCG::tr("Creating collection file...\n")), stdout);
 
     QFileInfo colFi(collectionFile);
     if (colFi.exists()) {
         if (!colFi.dir().remove(colFi.fileName())) {
-            fprintf(stderr, "The file %s cannot be overwritten!\n", qPrintable(collectionFile));
+            fputs(qPrintable(QCG::tr("The file %1 cannot be overwritten.\n")
+                             .arg(collectionFile)), stderr);
             return -1;
         }
     }
@@ -433,49 +480,58 @@ int main(int argc, char *argv[])
     }
 
     foreach (const QString &file, config.filesToRegister()) {
-        if (!helpEngine.registerDocumentation(basePath + QDir::separator() + file)) {
+        if (!helpEngine.registerDocumentation(absoluteFileName(basePath, file))) {
             fprintf(stderr, "%s\n", qPrintable(helpEngine.error()));
             return -1;
         }
     }
+    if (!config.filesToRegister().isEmpty())
+        CollectionConfiguration::updateLastRegisterTime(helpEngine);
 
     if (!config.title().isEmpty())
-        helpEngine.setCustomValue(QLatin1String("WindowTitle"), config.title());
+        CollectionConfiguration::setWindowTitle(helpEngine, config.title());
 
     if (!config.homePage().isEmpty()) {
-        helpEngine.setCustomValue(QLatin1String("defaultHomepage"),
-        config.homePage());
+        CollectionConfiguration::setDefaultHomePage(helpEngine,
+            config.homePage());
     }
 
-    if (!config.startPage().isEmpty())
-        helpEngine.setCustomValue(QLatin1String("LastShownPages"), config.startPage());
+    if (!config.startPage().isEmpty()) {
+        CollectionConfiguration::setLastShownPages(helpEngine,
+            QStringList(config.startPage()));
+    }
 
-    if (!config.currentFilter().isEmpty())
-        helpEngine.setCustomValue(QLatin1String("CurrentFilter"), config.currentFilter());
+    if (!config.currentFilter().isEmpty()) {
+        helpEngine.setCurrentFilter(config.currentFilter());
+    }
 
-    if (!config.cacheDirectory().isEmpty())
-        helpEngine.setCustomValue(QLatin1String("CacheDirectory"), config.cacheDirectory());
+    if (!config.cacheDirectory().isEmpty()) {
+        CollectionConfiguration::setCacheDir(helpEngine, config.cacheDirectory(),
+            config.cacheDirRelativeToCollection());
+    }
 
-    helpEngine.setCustomValue(QLatin1String("EnableFilterFunctionality"),
+    CollectionConfiguration::setFilterFunctionalityEnabled(helpEngine,
         config.enableFilterFunctionality());
-    helpEngine.setCustomValue(QLatin1String("HideFilterFunctionality"),
-        config.hideFilterFunctionality());
-    helpEngine.setCustomValue(QLatin1String("EnableDocumentationManager"),
+    CollectionConfiguration::setFilterToolbarVisible(helpEngine,
+        !config.hideFilterFunctionality());
+    CollectionConfiguration::setDocumentationManagerEnabled(helpEngine,
         config.enableDocumentationManager());
-    helpEngine.setCustomValue(QLatin1String("EnableAddressBar"),
+    CollectionConfiguration::setAddressBarEnabled(helpEngine,
         config.enableAddressBar());
-    helpEngine.setCustomValue(QLatin1String("HideAddressBar"),
-        config.hideAddressBar());
-    helpEngine.setCustomValue(QLatin1String("CreationTime"),
+    CollectionConfiguration::setAddressBarVisible(helpEngine,
+         !config.hideAddressBar());
+    CollectionConfiguration::setCreationTime(helpEngine,
         QDateTime::currentDateTime().toTime_t());
+    CollectionConfiguration::setFullTextSearchFallbackEnabled(helpEngine,
+        config.fullTextSearchFallbackEnabled());
 
     if (!config.applicationIcon().isEmpty()) {
-        QFile icon(basePath + QDir::separator() + config.applicationIcon());
+        QFile icon(absoluteFileName(basePath, config.applicationIcon()));
         if (!icon.open(QIODevice::ReadOnly)) {
-            fprintf(stderr, "Cannot open %s!\n", qPrintable(icon.fileName()));
+            fputs(qPrintable(QCG::tr("Cannot open %1.\n").arg(icon.fileName())), stderr);
             return -1;
         }
-        helpEngine.setCustomValue(QLatin1String("ApplicationIcon"), icon.readAll());
+        CollectionConfiguration::setApplicationIcon(helpEngine, icon.readAll());
     }
 
     if (config.aboutMenuTexts().count()) {
@@ -487,16 +543,16 @@ int main(int argc, char *argv[])
             s << it.value();
             ++it;
         }
-        helpEngine.setCustomValue(QLatin1String("AboutMenuTexts"), ba);
+        CollectionConfiguration::setAboutMenuTexts(helpEngine, ba);
     }
 
     if (!config.aboutIcon().isEmpty()) {
-        QFile icon(basePath + QDir::separator() + config.aboutIcon());
+        QFile icon(absoluteFileName(basePath, config.aboutIcon()));
         if (!icon.open(QIODevice::ReadOnly)) {
-            fprintf(stderr, "Cannot open %s!\n", qPrintable(icon.fileName()));
+            fputs(qPrintable(QCG::tr("Cannot open %1.\n").arg(icon.fileName())), stderr);
             return -1;
         }
-        helpEngine.setCustomValue(QLatin1String("AboutIcon"), icon.readAll());
+        CollectionConfiguration::setAboutIcon(helpEngine, icon.readAll());
     }
 
     if (config.aboutTextFiles().count()) {
@@ -512,10 +568,10 @@ int main(int argc, char *argv[])
 
         while (it != config.aboutTextFiles().constEnd()) {
             s << it.key();
-            QFileInfo fi(basePath + QDir::separator() + it.value());
+            QFileInfo fi(absoluteFileName(basePath, it.value()));
             QFile f(fi.absoluteFilePath());
             if (!f.open(QIODevice::ReadOnly)) {
-                fprintf(stderr, "Cannot open %s!\n", qPrintable(f.fileName()));
+                fputs(qPrintable(QCG::tr("Cannot open %1.\n").arg(f.fileName())), stderr);
                 return -1;
             }
             QByteArray data = f.readAll();
@@ -537,21 +593,21 @@ int main(int argc, char *argv[])
                         if (!imgData.contains(src))
                             imgData.insert(src, img.readAll());
                     } else {
-                        fprintf(stderr, "Cannot open referenced image file %s!\n",
-                            qPrintable(img.fileName()));
+                        fputs(qPrintable(QCG::tr("Cannot open referenced image file %1.\n")
+                                         .arg(img.fileName())), stderr);
                     }
                 }
             }
             ++it;
         }
-        helpEngine.setCustomValue(QLatin1String("AboutTexts"), ba);
+        CollectionConfiguration::setAboutTexts(helpEngine, ba);
         if (imgData.count()) {
             QByteArray imageData;
             QBuffer buffer(&imageData);
             buffer.open(QIODevice::WriteOnly);
             QDataStream out(&buffer);
             out << imgData;
-            helpEngine.setCustomValue(QLatin1String("AboutImages"), imageData);
+            CollectionConfiguration::setAboutImages(helpEngine, imageData);
         }
     }
 
