@@ -1,17 +1,18 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
-** All rights reserved.
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the Qt Designer of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
@@ -21,8 +22,8 @@
 ** ensure the GNU Lesser General Public License version 2.1 requirements
 ** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
@@ -33,8 +34,7 @@
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
+**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -57,6 +57,7 @@
 #include <QtGui/QVBoxLayout>
 #include <QtGui/QGridLayout>
 #include <QtGui/QFormLayout>
+#include <QtGui/QApplication>
 #include <QtGui/qevent.h>
 
 #include <QtCore/qdebug.h>
@@ -601,7 +602,7 @@ QRect LayoutHelper::itemInfo(QLayout *lt, const QWidget *widget) const
         }
     }
 
-    // Grid Layout state. Datatypically store the state of a GridLayout as a map of
+    // Grid Layout state. Datatype storing the state of a GridLayout as a map of
     // widgets to QRect(columns, rows) and size. Used to store the state for undo operations
     // that do not change the widgets within the layout; also provides some manipulation
     // functions and ability to apply the state to a layout provided its widgets haven't changed.
@@ -634,7 +635,11 @@ QRect LayoutHelper::itemInfo(QLayout *lt, const QWidget *widget) const
         static CellStates cellStates(const QList<QRect> &rects, int numRows, int numColumns);
 
         typedef QMap<QWidget *, QRect> WidgetItemMap;
+        typedef QMap<QWidget *, Qt::Alignment> WidgetAlignmentMap;
+
         WidgetItemMap widgetItemMap;
+        WidgetAlignmentMap widgetAlignmentMap;
+
         int rowCount;
         int colCount;
     };
@@ -706,8 +711,11 @@ QRect LayoutHelper::itemInfo(QLayout *lt, const QWidget *widget) const
         const int count = l->count();
         for (int i = 0; i < count; i++) {
             QLayoutItem *item = l->itemAt(i);
-            if (!LayoutInfo::isEmptyItem(item))
+            if (!LayoutInfo::isEmptyItem(item)) {
                 widgetItemMap.insert(item->widget(), gridItemInfo(l, i));
+                if (item->alignment())
+                    widgetAlignmentMap.insert(item->widget(), item->alignment());
+            }
         }
     }
 
@@ -743,7 +751,8 @@ QRect LayoutHelper::itemInfo(QLayout *lt, const QWidget *widget) const
         const LayoutItemRectMap::const_iterator icend = itemMap.constEnd();
         for (LayoutItemRectMap::const_iterator it = itemMap.constBegin(); it != icend; ++it) {
             const QRect info = it.value();
-            grid->addItem(it.key(), info.y(), info.x(), info.height(), info.width());
+            const Qt::Alignment alignment = widgetAlignmentMap.value(it.key()->widget(), Qt::Alignment(0));
+            grid->addItem(it.key(), info.y(), info.x(), info.height(), info.width(), alignment);
         }
         // create spacers
         const CellStates cs = cellStates(itemMap.values(), rowCount, colCount);
@@ -1181,8 +1190,7 @@ QRect LayoutHelper::itemInfo(QLayout *lt, const QWidget *widget) const
             } else {
                 for (int c = 0; c < FormLayoutColumns; c++) {
                     const QFormLayout::ItemRole role = c == 0 ? QFormLayout::LabelRole : QFormLayout::FieldRole;
-                    if (widgets[c]) {
-                        Q_ASSERT(BoxLayoutHelper::findItemOfWidget(items, widgets[c]));
+                    if (widgets[c] && BoxLayoutHelper::findItemOfWidget(items, widgets[c])) {
                         formLayout->setWidget(r, role, widgets[c]);
                     } else {
                         formLayout->setItem(r, role, createFormSpacer());
@@ -1381,7 +1389,9 @@ void QLayoutSupport::adjustIndicator(const QPoint &pos, int index)
                 const bool closeToLeft = fromLeftRight == fromLeft;
                 showIndicator(RightIndicator, QRect(closeToLeft ? g.x() : g.right() + 1 - indicatorSize, 0, indicatorSize, r.height()), bluePalette);
 
-                const int incr = closeToLeft ? 0 : +1;
+                const QWidget *parent = layout()->parentWidget();
+                const bool leftToRight = Qt::LeftToRight == (parent ? parent->layoutDirection() : QApplication::layoutDirection());
+                const int incr = leftToRight == closeToLeft ? 0 : +1;
                 setCurrentCellFromIndicator(indicatorOrientation, m_currentIndex, incr);
             }
             break;
