@@ -1,17 +1,18 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
-** All rights reserved.
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the tools applications of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
@@ -21,8 +22,8 @@
 ** ensure the GNU Lesser General Public License version 2.1 requirements
 ** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
@@ -33,8 +34,7 @@
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
+**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -50,12 +50,16 @@
 #include <QtDBus/QtDBus>
 #include <private/qdbusutil_p.h>
 
+QT_BEGIN_NAMESPACE
+Q_DBUS_EXPORT extern bool qt_dbus_metaobject_skip_annotations;
+QT_END_NAMESPACE
+
 static QDBusConnection connection(QLatin1String(""));
 static bool printArgumentsLiterally = false;
 
 static void showUsage()
 {
-    printf("Usage: qdbus [--system] [--literal] [servicename] [path] [method] [args]\n"
+    printf("Usage: qdbus [--system | --address ADDRESS] [--literal] [servicename] [path] [method] [args]\n"
            "\n"
            "  servicename       the service to connect to (e.g., org.freedesktop.DBus)\n"
            "  path              the path to the object (e.g., /)\n"
@@ -67,6 +71,7 @@ static void showUsage()
            "\n"
            "Options:\n"
            "  --system          connect to the system bus\n"
+           "  --address ADDRESS connect to the given bus\n"
            "  --literal         print replies literally\n");
 }
 
@@ -99,7 +104,7 @@ static void printArg(const QVariant &v)
         else if (arg.currentSignature() == QLatin1String("a{sv}"))
             printArg(qdbus_cast<QVariantMap>(arg));
         else
-            printf("qdbus: I don't know how to display an argument of type '%s'\n",
+            printf("qdbus: I don't know how to display an argument of type '%s', run with --literal.\n",
                    qPrintable(arg.currentSignature()));
     } else {
         printf("%s\n", qPrintable(v.toString()));
@@ -248,123 +253,157 @@ static QStringList readList(QStringList &args)
     return retval;
 }
 
-static void placeCall(const QString &service, const QString &path, const QString &interface,
-               const QString &member, QStringList args)
+static int placeCall(const QString &service, const QString &path, const QString &interface,
+               const QString &member, const QStringList& arguments, bool try_prop=true)
 {
     QDBusInterface iface(service, path, interface, connection);
 
     // Don't check whether the interface is valid to allow DBus try to
     // activate the service if possible.
 
+    QList<int> knownIds;
+    bool matchFound = false;
+    QStringList args = arguments;
     QVariantList params;
     if (!args.isEmpty()) {
         const QMetaObject *mo = iface.metaObject();
         QByteArray match = member.toLatin1();
         match += '(';
 
-        int midx = -1;
         for (int i = mo->methodOffset(); i < mo->methodCount(); ++i) {
             QMetaMethod mm = mo->method(i);
             QByteArray signature = mm.signature();
-            if (signature.startsWith(match)) {
-                midx = i;
-                break;
-            }
+            if (signature.startsWith(match))
+                knownIds += i;
          }
 
-        if (midx == -1) {
-            fprintf(stderr, "Cannot find '%s.%s' in object %s at %s\n",
-                    qPrintable(interface), qPrintable(member), qPrintable(path),
-                    qPrintable(service));
-            exit(1);
-        }
 
-        QMetaMethod mm = mo->method(midx);
-        QList<QByteArray> types = mm.parameterTypes();
-        for (int i = 0; i < types.count(); ++i) {
-            if (types.at(i).endsWith('&')) {
-                // reference (and not a reference to const): output argument
-                // we're done with the inputs
-                while (types.count() > i)
-                    types.removeLast();
-                break;
+        while (!matchFound) {
+            args = arguments; // reset
+            params.clear();
+            if (knownIds.isEmpty()) {
+                // Failed to set property after falling back?
+                // Bail out without displaying an error
+                if (!try_prop)
+                    return 1;
+                if (try_prop && args.size() == 1) {
+                    QStringList proparg;
+                    proparg += interface;
+                    proparg += member;
+                    proparg += args.first();
+                    if (!placeCall(service, path, "org.freedesktop.DBus.Properties", "Set", proparg, false))
+                        return 0;
+                }
+                fprintf(stderr, "Cannot find '%s.%s' in object %s at %s\n",
+                        qPrintable(interface), qPrintable(member), qPrintable(path),
+                        qPrintable(service));
+                return 1;
             }
-        }
 
-        for (int i = 0; !args.isEmpty() && i < types.count(); ++i) {
-            int id = QVariant::nameToType(types.at(i));
-            if (id == QVariant::UserType)
-                id = QMetaType::type(types.at(i));
-            Q_ASSERT(id);
-
-            QVariant p;
-            QString argument;
-            if ((id == QVariant::List || id == QVariant::StringList)
-                 && args.at(0) == QLatin1String("("))
-                p = readList(args);
-            else
-                p = argument = args.takeFirst();
-
-            if (id == int(QMetaType::UChar)) {
-                // special case: QVariant::convert doesn't convert to/from
-                // UChar because it can't decide if it's a character or a number
-                p = qVariantFromValue<uchar>(p.toUInt());
-            } else if (id < int(QMetaType::User) && id != int(QVariant::Map)) {
-                p.convert(QVariant::Type(id));
-                if (p.type() == QVariant::Invalid) {
-                    fprintf(stderr, "Could not convert '%s' to type '%s'.\n",
-                            qPrintable(argument), types.at(i).constData());
-                    exit(1);
+            QMetaMethod mm = mo->method(knownIds.takeFirst());
+            QList<QByteArray> types = mm.parameterTypes();
+            for (int i = 0; i < types.count(); ++i) {
+                if (types.at(i).endsWith('&')) {
+                    // reference (and not a reference to const): output argument
+                    // we're done with the inputs
+                    while (types.count() > i)
+                        types.removeLast();
+                    break;
                 }
-            } else if (id == qMetaTypeId<QDBusVariant>()) {
-                QDBusVariant tmp(p);
-                p = qVariantFromValue(tmp);
-            } else if (id == qMetaTypeId<QDBusObjectPath>()) {
-                QDBusObjectPath path(argument);
-                if (path.path().isNull()) {
-                    fprintf(stderr, "Cannot pass argument '%s' because it is not a valid object path.\n",
-                            qPrintable(argument));
-                    exit(1);
-                }
-                p = qVariantFromValue(path);
-            } else if (id == qMetaTypeId<QDBusSignature>()) {
-                QDBusSignature sig(argument);
-                if (sig.signature().isNull()) {
-                    fprintf(stderr, "Cannot pass argument '%s' because it is not a valid signature.\n",
-                            qPrintable(argument));
-                    exit(1);
-                }
-                p = qVariantFromValue(sig);
-            } else {
-                fprintf(stderr, "Sorry, can't pass arg of type '%s'.\n",
-                        types.at(i).constData());
-                exit(1);
             }
-            params += p;
-        }
-        if (params.count() != types.count() || !args.isEmpty()) {
-            fprintf(stderr, "Invalid number of parameters\n");
-            exit(1);
-        }
-    }
+
+            for (int i = 0; !args.isEmpty() && i < types.count(); ++i) {
+                int id = QVariant::nameToType(types.at(i));
+                if (id == QVariant::UserType)
+                    id = QMetaType::type(types.at(i));
+                if (!id) {
+                    fprintf(stderr, "Cannot call method '%s' because type '%s' is unknown to this tool\n",
+                            qPrintable(member), types.at(i).constData());
+                    return 1;
+                }
+
+                QVariant p;
+                QString argument;
+                if ((id == QVariant::List || id == QVariant::StringList)
+                     && args.at(0) == QLatin1String("("))
+                    p = readList(args);
+                else
+                    p = argument = args.takeFirst();
+
+                if (id == int(QMetaType::UChar)) {
+                    // special case: QVariant::convert doesn't convert to/from
+                    // UChar because it can't decide if it's a character or a number
+                    p = QVariant::fromValue<uchar>(p.toUInt());
+                } else if (id < int(QMetaType::User) && id != int(QVariant::Map)) {
+                    p.convert(QVariant::Type(id));
+                    if (p.type() == QVariant::Invalid) {
+                        fprintf(stderr, "Could not convert '%s' to type '%s'.\n",
+                                qPrintable(argument), types.at(i).constData());
+                        return 1 ;
+                    }
+                } else if (id == qMetaTypeId<QDBusVariant>()) {
+                    QDBusVariant tmp(p);
+                    p = QVariant::fromValue(tmp);
+                } else if (id == qMetaTypeId<QDBusObjectPath>()) {
+                    QDBusObjectPath path(argument);
+                    if (path.path().isNull()) {
+                        fprintf(stderr, "Cannot pass argument '%s' because it is not a valid object path.\n",
+                                qPrintable(argument));
+                        return 1;
+                    }
+                    p = QVariant::fromValue(path);
+                } else if (id == qMetaTypeId<QDBusSignature>()) {
+                    QDBusSignature sig(argument);
+                    if (sig.signature().isNull()) {
+                        fprintf(stderr, "Cannot pass argument '%s' because it is not a valid signature.\n",
+                                qPrintable(argument));
+                        return 1;
+                    }
+                    p = QVariant::fromValue(sig);
+                } else {
+                    fprintf(stderr, "Sorry, can't pass arg of type '%s'.\n",
+                            types.at(i).constData());
+                    return 1;
+                }
+                params += p;
+            }
+            if (params.count() == types.count() && args.isEmpty())
+                matchFound = true;
+            else if (knownIds.isEmpty()) {
+                fprintf(stderr, "Invalid number of parameters\n");
+                return 1;
+            }
+        } // while (!matchFound)
+    } // if (!args.isEmpty()
 
     QDBusMessage reply = iface.callWithArgumentList(QDBus::Block, member, params);
     if (reply.type() == QDBusMessage::ErrorMessage) {
         QDBusError err = reply;
+        // Failed to retrieve property after falling back?
+        // Bail out without displaying an error
+        if (!try_prop)
+            return 1;
+        if (err.type() == QDBusError::UnknownMethod && try_prop) {
+            QStringList proparg;
+            proparg += interface;
+            proparg += member;
+            if (!placeCall(service, path, "org.freedesktop.DBus.Properties", "Get", proparg, false))
+                return 0;
+        }
         if (err.type() == QDBusError::ServiceUnknown)
             fprintf(stderr, "Service '%s' does not exist.\n", qPrintable(service));
         else
             printf("Error: %s\n%s\n", qPrintable(err.name()), qPrintable(err.message()));
-        exit(2);
+        return 2;
     } else if (reply.type() != QDBusMessage::ReplyMessage) {
         fprintf(stderr, "Invalid reply type %d\n", int(reply.type()));
-        exit(1);
+        return 1;
     }
 
     foreach (QVariant v, reply.arguments())
         printArg(v);
 
-    exit(0);
+    return 0;
 }
 
 static bool globServices(QDBusConnectionInterface *bus, const QString &glob)
@@ -405,6 +444,7 @@ static void printAllServices(QDBusConnectionInterface *bus)
 
 int main(int argc, char **argv)
 {
+    QT_PREPEND_NAMESPACE(qt_dbus_metaobject_skip_annotations) = true;
     QCoreApplication app(argc, argv);
     QStringList args = app.arguments();
     args.takeFirst();
@@ -415,6 +455,11 @@ int main(int argc, char **argv)
         if (arg == QLatin1String("--system")) {
             connection = QDBusConnection::systemBus();
             connectionOpened = true;
+        } else if (arg == QLatin1String("--address")) {
+            if (!args.isEmpty()) {
+                connection = QDBusConnection::connectToBus(args.takeFirst(), "bus");
+                connectionOpened = true;
+            }
         } else if (arg == QLatin1String("--literal")) {
             printArgumentsLiterally = true;
         } else if (arg == QLatin1String("--help")) {
@@ -483,6 +528,7 @@ int main(int argc, char **argv)
         exit(1);
     }
 
-    placeCall(service, path, interface, member, args);
+    int ret = placeCall(service, path, interface, member, args);
+    exit(ret);
 }
 
