@@ -1,17 +1,18 @@
 /****************************************************************************
 **
-** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
-** All rights reserved.
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the tools applications of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
@@ -21,8 +22,8 @@
 ** ensure the GNU Lesser General Public License version 2.1 requirements
 ** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
@@ -33,8 +34,7 @@
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
+**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -57,6 +57,12 @@
 
 QT_BEGIN_NAMESPACE
 
+typedef QMap<QString, const Node*> NodeMap;
+typedef QMultiMap<QString, Node*> NodeMultiMap;
+typedef QMap<QString, NodeMultiMap> NewSinceMaps;
+typedef QMap<Node*, NodeMultiMap> ParentMaps;
+typedef QMap<QString, NodeMap> NewClassMaps;
+
 class ClassNode;
 class Config;
 class CodeMarker;
@@ -78,7 +84,7 @@ class Generator
     virtual void terminateGenerator();
     virtual QString format() = 0;
     virtual bool canHandleFormat(const QString &format) { return format == this->format(); }
-    virtual void generateTree(const Tree *tree, CodeMarker *marker) = 0;
+    virtual void generateTree(const Tree *tree) = 0;
 
     static void initialize(const Config& config);
     static void terminate();
@@ -106,6 +112,7 @@ class Generator
 #endif
     virtual void generateBody(const Node *node, CodeMarker *marker);
     virtual void generateAlsoList(const Node *node, CodeMarker *marker);
+    virtual void generateMaintainerList(const InnerNode* node, CodeMarker* marker);
     virtual void generateInherits(const ClassNode *classe,
                                   CodeMarker *marker);
     virtual void generateInheritedBy(const ClassNode *classe,
@@ -114,7 +121,7 @@ class Generator
     void generateThreadSafeness(const Node *node, CodeMarker *marker);
     void generateSince(const Node *node, CodeMarker *marker);
     void generateStatus(const Node *node, CodeMarker *marker);
-    const Atom *generateAtomList(const Atom *atom, 
+    const Atom* generateAtomList(const Atom *atom, 
                                  const Node *relative,
                                  CodeMarker *marker, 
                                  bool generate,
@@ -124,13 +131,14 @@ class Generator
                           Node::SubType subtype,
                           const QString& tag);
     void generateExampleFiles(const FakeNode *fake, CodeMarker *marker);
-    void generateModuleWarning(const ClassNode *classe, CodeMarker *marker);
 
     virtual int skipAtoms(const Atom *atom, Atom::Type type) const;
     virtual QString fullName(const Node *node, 
                              const Node *relative,
                              CodeMarker *marker) const;
 
+    virtual QString outFileName() { return QString(); }
+
     const QString& outputDir() { return outDir; }
     QString indent(int level, const QString& markedCode);
     QString plainCode(const QString& markedCode);
@@ -140,16 +148,19 @@ class Generator
     void unknownAtom(const Atom *atom);
     QMap<QString, QString> &formattingLeftMap();
     QMap<QString, QString> &formattingRightMap();
-
     QMap<QString, QStringList> editionModuleMap;
     QMap<QString, QStringList> editionGroupMap;
 
     static QString trimmedTrailing(const QString &string);
     static bool matchAhead(const Atom *atom, Atom::Type expectedAtomType);
     static void supplementAlsoList(const Node *node, QList<Text> &alsoList);
+    static QString outputPrefix(const QString &nodeType);
+
+    QString getMetadataElement(const InnerNode* inner, const QString& t);
+    QStringList getMetadataElements(const InnerNode* inner, const QString& t);
+    void findAllSince(const InnerNode *node);
 
  private:
-    void generateOverload(const Node *node, CodeMarker *marker);
     void generateReimplementedFrom(const FunctionNode *func,
                                    CodeMarker *marker);
     void appendFullName(Text& text, 
@@ -170,6 +181,18 @@ class Generator
                            const QList<RelatedClass> &classes,
                            CodeMarker *marker);
 
+ protected:
+    void appendSortedQmlNames(Text& text,
+                              const Node* base,
+                              const NodeList& subs,
+                              CodeMarker *marker);
+
+    static QString sinceTitles[];
+    NewSinceMaps newSinceMaps;
+    NewClassMaps newClassMaps;
+    NewClassMaps newQmlClassMaps;
+
+ private:
     QString amp;
     QString lt;
     QString gt;
@@ -185,8 +208,13 @@ class Generator
     static QStringList imageDirs;
     static QStringList exampleDirs;
     static QStringList exampleImgExts;
+    static QStringList scriptFiles;
+    static QStringList scriptDirs;
+    static QStringList styleFiles;
+    static QStringList styleDirs;
     static QString outDir;
     static QString project;
+    static QHash<QString, QString> outputPrefixes;
 };
 
 QT_END_NAMESPACE
